@@ -23,7 +23,7 @@
 
 (define-command-table application-commands)
 
-(define-command-table lisp-dev-commands :inherit-from nil) ;; "Abstract" command table used for defining some translators in
+(define-command-table lisp-dev-commands :inherit-from nil) ;; Translators live here
 (define-command-table lisp-commands :inherit-from (lisp-dev-commands))
 
 (define-command-table show-commands :inherit-from (lisp-dev-commands))
@@ -37,10 +37,20 @@
 
 ;;; Presentation types
 
-(define-presentation-type class () :inherit-from 'expression)
+(define-presentation-type specializer () :inherit-from 'expression)
+(define-presentation-type class () :inherit-from 'specializer)
+(define-presentation-type eql-specializer () :inherit-from 'specializer)
 (define-presentation-type class-name () :inherit-from 'symbol)
 (define-presentation-type slot-definition () :inherit-from 'expression)
-(define-presentation-type function-name () :inherit-from 'symbol)
+
+(define-presentation-type-abbreviation function-name () 
+  `(and expression (satisfies legal-and-fboundp)))
+
+(defun legal-and-fboundp (object)
+  (and #+sbcl (sb-int:valid-function-name-p object)
+       #-sbcl (typep object '(or symbol (cons (eql setf))))
+       (fboundp object)))
+
 (define-presentation-type process () :inherit-from 'expression)
 (define-presentation-type generic-function () :inherit-from 't)
 
@@ -67,9 +77,7 @@
 
 (define-presentation-type package-name () :inherit-from 'string)
 (define-presentation-method presentation-typep (object (type package-name))
-  (find-package 'object))
-    
-
+  (find-package object))
 
 ;;; Presentation methods
 
@@ -98,8 +106,11 @@
 		 (write-char #\( stream)
 		 (present arg 'symbol :stream stream)
 		 (write-char #\space  stream)
-		 (with-output-as-presentation (stream spec 'class)
-		   (format stream "~S" (clim-mop:class-name spec)))
+		 (with-output-as-presentation (stream spec 'specializer
+                                                      :single-box t)
+                   (if (typep spec 'class)
+                       (format stream "~S" (clim-mop:class-name spec))
+                       (format stream "~S" `(eql ,(clim-mop:eql-specializer-object spec)))))
                  (write-char #\) stream))))
       (when optional
 	(format stream " &optional ~{~A ~^ ~}" optional))
@@ -158,42 +169,60 @@
 (define-presentation-translator class-name-to-class
   (class-name class lisp-dev-commands
      :documentation ((object stream) (format stream "Class object ~A" object))
-     :gesture T)
+     :gesture t)
   (object)
   (find-class object))
 
 (define-presentation-translator symbol-to-class
   (symbol class lisp-dev-commands
      :documentation ((object stream) (format stream "Class object ~A" object))
-     :gesture T
+     :gesture t
      :tester ((object) (not (not (find-class object nil))))
-     :tester-definitive T)
+     :tester-definitive t)
   (object)
   (find-class object))
 
 (define-presentation-translator symbol-to-class-name
   (symbol class-name lisp-dev-commands
      :documentation ((object stream) (format stream "Class ~A" object))
-     :gesture T
+     :gesture t
      :tester ((object) (not (not (find-class object nil))))
-     :tester-definitive T)
+     :tester-definitive t)
   (object)
   object)
 
 (define-presentation-translator class-to-class-name
   (class class-name lisp-dev-commands
      :documentation ((object stream) (format stream "Class of ~A" object))
-     :gesture T)
+     :gesture t)
   (object)
   (clim-mop:class-name object))
 
+(define-presentation-translator expression-to-function-name
+  (expression function-name lisp-dev-commands
+     :documentation ((object stream) (format stream "~A" object))
+     :gesture t
+     :tester ((object) (legal-and-fboundp object))
+     :tester-definitive t)
+  (object)
+  object)
 (define-presentation-translator symbol-to-function-name
   (symbol function-name lisp-dev-commands
-     :documentation ((object stream) (format stream "Function ~A" object))
-     :gesture T
-     :tester ((object) (fboundp object))
-     :tester-definitive T)
-  (object) object)
+     :documentation ((object stream) (format stream "~A" object))
+     :gesture t
+     :tester ((object) (legal-and-fboundp object))
+     :tester-definitive t)
+  (object)
+  object)
+#+nil ; doesn't work for some reason
+(define-presentation-translator sequence-to-function-name
+  ((sequence t) function-name lisp-dev-commands
+     :documentation ((object stream) (format stream "~A" object))
+     :gesture t
+     :tester ((object) (legal-and-fboundp object))
+     :tester-definitive t)
+  (object)
+  object)
 
 ;;; Application commands
 
@@ -214,7 +243,7 @@
 			 :provide-output-destination-keyword t)
   ((program 'string :prompt "command")
    (args '(sequence string) :default nil :prompt "args"))
-  (run-program program args :wait T :input nil))
+  (run-program program args :wait t :input nil))
 
 ;; I could replace this command with a keyword to COM-RUN..
 (define-command (com-background-run :name "Background Run"
@@ -327,32 +356,32 @@
     (let ((symbols (remove-if-not (lambda (sym) (apropos-applicable-p domain sym))
                                   (apropos-list string real-package))))
       (dolist (sym symbols)
-        (apropos-present-symbol sym *standard-output* T)
+        (apropos-present-symbol sym *standard-output* t)
         (terpri))
       (setf *apropos-list* symbols)
-      (note "Results have been saved to ~W~%" '*APROPOS-LIST*))))
+      (note "Results have been saved to ~W~%" '*apropos-list*))))
 
 (define-command (com-trace :name "Trace"
 			   :command-table lisp-commands
                            :menu t
 			   :provide-output-destination-keyword nil)
-    ((fsym 'function-name :prompt "function-name"))
+    ((fsym 'function-name :prompt "function name"))
   (if (fboundp fsym)
       (progn 
 	(eval `(trace ,fsym))
-	(format T "~&Tracing ~W.~%" fsym))
-    (format T "~&Function ~W is not defined.~%" fsym)))
+	(format t "~&Tracing ~W.~%" fsym))
+    (format t "~&Function ~W is not defined.~%" fsym)))
 
 (define-command (com-untrace :name "Untrace"
 			     :command-table lisp-commands
                              :menu t
 			     :provide-output-destination-keyword nil)
-    ((fsym 'symbol :prompt "function name"))
+    ((fsym 'function-name :prompt "function name"))
   (if (fboundp fsym)
       (progn
 	(eval `(untrace ,fsym))
-	(format T "~&~W will no longer be traced.~%" fsym))
-    (format T "~&Function ~W is not defined.~%" fsym)))
+	(format t "~&~W will no longer be traced.~%" fsym))
+    (format t "~&Function ~W is not defined.~%" fsym)))
 
 
 (define-command (com-load-file :name "Load File"
@@ -396,19 +425,18 @@
               :menu t
               :documentation "Show Class Subclasses"
               :pointer-documentation "Show Class Subclasses")
-  (presentation)
-  (list (presentation-object presentation)))
+  (object)
+  (list object))
 
 
 (define-presentation-to-command-translator com-show-class-superclasses-translator
   (class-name com-show-class-superclasses lisp-commands
               :menu t
-              :tester ((presentation)
-                       (not (eq t (presentation-object presentation))))
+              :tester ((object) (not (eq t object)))
               :documentation "Show Class Superclasses"
               :pointer-documentation "Show Class Superclasses")
-  (presentation)
-  (list (presentation-object presentation)))
+  (object)
+  (list object))
 
 
 (define-presentation-to-command-translator com-show-class-generic-functions-translator
@@ -416,8 +444,8 @@
               :menu t
               :documentation "Show Class Generic Functions"
               :pointer-documentation "Show Class Generic Functions")
-  (presentation)
-  (list (presentation-object presentation)))
+  (object)
+  (list object))
 
 
 (define-presentation-to-command-translator com-show-class-slots-translator
@@ -425,8 +453,8 @@
               :menu t
               :documentation "Show Class Slots"
               :pointer-documentation "Show Class Slots")
-  (presentation)
-  (list (presentation-object presentation)))
+  (object)
+  (list object))
 
 
 ;;; CLOS introspection commands
@@ -448,12 +476,13 @@
 								   :text-style text-style)
 				       ;; Present class name rather than class here because the printing of the
 				       ;; class object itself is rather long and freaks out the pointer doc pane.
-				       (with-output-as-presentation (stream (clim-mop:class-name class) 'class-name)
+				       (with-output-as-presentation (stream (clim-mop:class-name class) 'class-name
+                                                                            :single-box t)
 					; (surrounding-output-with-border (stream :shape :drop-shadow)
 					 (princ (clim-mop:class-name class) stream)))) ;)
 				 inferior-fun
 				 :stream stream
-				 :merge-duplicates T
+				 :merge-duplicates t
 				 :graph-type :tree
 				 :orientation orientation
 				 :arc-drawer
@@ -489,11 +518,11 @@
     ((class-spec 'class-name :prompt "class")
      &key
      (orientation 'keyword :prompt "orientation" :default :horizontal))     
-  (let ((class (frob-to-class class-spec)))
-    (if (not (null class))
+  (let ((class (frob-to-class class-spec)))    
+    (if (not (null class))        
         (class-grapher *standard-output* class #'clim-mop:class-direct-subclasses
                        :orientation orientation)
-      (note "~A is not a defined class." class-spec))))
+        (note "~A is not a defined class." class-spec))))
 
 
 ; Lookup direct slots from along the CPL given a class and a slot name.
@@ -528,30 +557,30 @@
          (direct-slots (direct-slot-definitions class name))
          (readers (reduce #'append (filtermap direct-slots #'clim-mop:slot-definition-readers)))
          (writers (reduce #'append (filtermap direct-slots #'clim-mop:slot-definition-writers)))
-         (documentation (first (filtermap direct-slots (lambda (x) (documentation x T)))))
+         (documentation (first (filtermap direct-slots (lambda (x) (documentation x t)))))
          (*standard-output* stream))
 
   (macrolet ((with-ink ((var) &body body)
-               `(with-drawing-options (T :ink ,(intern (concatenate 'string "*SLOT-" (symbol-name var) "-INK*")))
+               `(with-drawing-options (t :ink ,(intern (concatenate 'string "*SLOT-" (symbol-name var) "-INK*")))
                      ,@body))
              (fcell ((var align-x &rest cell-opts) &body body)
-                `(formatting-cell (T :align-x ,align-x ,@cell-opts)
+                `(formatting-cell (t :align-x ,align-x ,@cell-opts)
                    (with-ink (,var) ,@body) )))
     
     (fcell (name :left)
-     (with-output-as-presentation (T slot 'slot-definition)
+     (with-output-as-presentation (t slot 'slot-definition :single-box t)
        (princ name))
-     (unless (eq type T)
+     (unless (eq type t)
        (fresh-line)
        (with-ink (type) (princ type))))
 
     (fcell (initargs :right)
       (dolist (x initargs)
-        (format T "~W~%" x)))
+        (format t "~W~%" x)))
 
     (fcell (initform :left)
       (if initfunc
-          (format T "~W" initform)
+          (format t "~W" initform)
         (note "No initform")))
 
     #+NIL   ; argh, shouldn't this work?
@@ -567,19 +596,25 @@
                      (dolist (writer writers)  (format T "~A~%" writer))
                    (note "No writers"))))))
 
-    (formatting-cell (T :align-x :left)
+    (formatting-cell (t :align-x :left)
       (if (not (or readers writers))
           (note "No accessors")
         (progn
           (with-ink (readers)
-            (if readers (dolist (reader readers)  (format T "~A~%" reader))
-              (note "No readers~%")))
+            (if readers 
+                (dolist (reader readers)
+                  (present reader (presentation-type-of reader))
+                  (terpri))
+                (note "No readers~%")))
           (with-ink (writers)
-            (if writers (dolist (writer writers)  (format T "~A~%" writer))
+            (if writers 
+                (dolist (writer writers) 
+                  (present writer (presentation-type-of writer))
+                  (terpri))
               (note "No writers"))))))
 
     (fcell (documentation :left)
-      (when documentation (with-text-family (T :serif) (princ documentation))))
+      (when documentation (with-text-family (t :serif) (princ documentation))))
 )))
 
 
@@ -601,18 +636,18 @@
                (position (earliest-slot-definer b class) cpl))))))
 
 (defun print-slot-table-heading ()
-  (formatting-row (T)
+  (formatting-row (t)
     (dolist (name '("Slot name" "Initargs" "Initform" "Accessors"))
-      (formatting-cell (T :align-x :center)
-        (underlining (T)
-          (with-text-family (T :sans-serif)
+      (formatting-cell (t :align-x :center)
+        (underlining (t)
+          (with-text-family (t :sans-serif)
             (princ name)))))))
 
 (defun present-slot-list (slots class)
-  (formatting-table (T)
+  (formatting-table (t)
     (print-slot-table-heading)
     (dolist (slot slots)
-      (formatting-row (T)
+      (formatting-row (t)
         (present-slot slot class)))))
 
 (defun friendly-slot-allocation-type (allocation)
@@ -626,11 +661,11 @@
          (other-slots (set-difference slots instance-slots))
          (allocation-types (remove-duplicates (mapcar #'clim-mop:slot-definition-allocation other-slots))))
     (when other-slots
-      (underlining (T) (format T "~&Instance Slots~%")))
+      (underlining (t) (format t "~&Instance Slots~%")))
     (present-slot-list instance-slots class)
     (dolist (alloc allocation-types)
-      (underlining (T)
-        (format T "~&Allocation: ~A~%" (friendly-slot-allocation-type alloc)))
+      (underlining (t)
+        (format t "~&Allocation: ~A~%" (friendly-slot-allocation-type alloc)))
       (present-slot-list (remove-if (lambda (x)
                                       (not (eq alloc (clim-mop:slot-definition-allocation x))))
                                     other-slots)
@@ -643,17 +678,17 @@
     ((class-name 'clim:symbol :prompt "class name"))
   (let ((class (find-class class-name nil)))
     (if (null class)
-	(format T "~&~A is not a defined class.~%" class-name)
+	(format t "~&~A is not a defined class.~%" class-name)
       (let ((slots (clim-mop:class-slots class)))
 	(if (null slots)
 	    (note "~%This class has no slots.~%~%")
             (progn
             ; oddly, looks much better in courier, because of all the capital letters.
-;            (with-text-family (T :sans-serif)
+;            (with-text-family (t :sans-serif)
               (invoke-as-heading
                (lambda ()
-                 (format T "~&Slots for ")
-                 (with-output-as-presentation (T (clim-mop:class-name class) 'class-name)
+                 (format t "~&Slots for ")
+                 (with-output-as-presentation (t (clim-mop:class-name class) 'class-name :single-box t)
                    (princ (clim-mop:class-name class)))))
               (present-the-slots class) ))))))
 
@@ -697,7 +732,7 @@
                           (symbol-package b)))
                  (string< (package-name (symbol-package a))
                           (package-name (symbol-package b))))
-                (T (string< (symbol-name a)
+                (t (string< (symbol-name a)
                             (symbol-name b))))
           (string< (princ-to-string a)
                    (princ-to-string b))))))
@@ -714,10 +749,10 @@
       (let ((funcs (sort (class-funcs class) (lambda (a b)
                                                (slot-name-sortp (clim-mop:generic-function-name a)
                                                                 (clim-mop:generic-function-name b))))))
-        (with-text-size (T :small)
+        (with-text-size (t :small)
           (format-items funcs :printer (lambda (item stream)
                                          (present item 'generic-function :stream stream))
-                        :move-cursor T))))))
+                        :move-cursor t))))))
 
 (defun method-applicable-to-args-p (method args arg-types)
   (loop
@@ -882,7 +917,8 @@
 	   do (progn
 		(with-output-as-presentation (*standard-output*
 					      (clim-mop:class-name class)
-					      'class-name)
+					      'class-name
+                                              :single-box t)
 		  (format *standard-output*
 			  "~S~%" (clim-mop:class-name class)))))))
     (when methods
@@ -975,7 +1011,8 @@
                                                                             normal-ink
                                                                             (make-rgb-color 0.4 0.4 0.4))
                                                                    :text-style text-style)
-                                       (with-output-as-presentation (stream package 'package)
+                                       (with-output-as-presentation (stream package 'package
+                                                                            :single-box t)
                                          (format stream "~A (~D/~D)" (package-name package) internal external)))))
                                inferior-fun
                                :stream stream
@@ -1026,8 +1063,9 @@
                                  :type (pathname-type pathname)
                                  :version (pathname-version pathname))))))
 
-(defun pretty-pretty-pathname (pathname stream &key (long-name T))
-  (with-output-as-presentation (stream pathname 'clim:pathname)
+(defun pretty-pretty-pathname (pathname stream &key (long-name t))
+  (with-output-as-presentation (stream pathname 'clim:pathname
+                                       :single-box t)
     (let ((icon (icon-of pathname)))
       (when icon  (draw-icon stream icon :extra-spacing 3)))
     (princ (pathname-printing-name pathname long-name) stream))
@@ -1077,10 +1115,10 @@
      &key
      (sort-by '(member name size modify none) :default 'name)
      (show-hidden  'boolean :default nil :prompt "show hidden")
-     (hide-garbage 'boolean :default T   :prompt "hide garbage")
+     (hide-garbage 'boolean :default t   :prompt "hide garbage")
      (show-all     'boolean :default nil :prompt "show all")
      (style '(member items list) :default 'items :prompt "listing style")
-     (group-directories 'boolean :default T :prompt "group directories?")
+     (group-directories 'boolean :default t :prompt "group directories?")
      (full-names 'boolean :default nil :prompt "show full name?")
      (list-all-direct-subdirectories 'boolean :default nil :prompt "list all direct subdirectories?"))
 
@@ -1092,18 +1130,18 @@
                   (list-directory-with-all-direct-subdirectories wild-pathname)
                   (list-directory wild-pathname))))
 
-    (with-text-family (T :sans-serif)      
+    (with-text-family (t :sans-serif)      
       (invoke-as-heading
         (lambda ()
-          (format T "Directory contents of ")
+          (format t "Directory contents of ")
           (present (directory-namestring pathname) 'pathname)
           (when (pathname-type pathname)
-            (format T " (only files of type ~a)" (pathname-type pathname)))))
+            (format t " (only files of type ~a)" (pathname-type pathname)))))
     
       (when (parent-directory pathname)
-        (with-output-as-presentation (T (strip-filespec (parent-directory pathname)) 'clim:pathname)
-          (draw-icon T (standard-icon "up-folder.xpm") :extra-spacing 3)
-          (format T "Parent Directory~%")))
+        (with-output-as-presentation (t (strip-filespec (parent-directory pathname)) 'clim:pathname :single-box t)
+          (draw-icon t (standard-icon "up-folder.xpm") :extra-spacing 3)
+          (format t "Parent Directory~%")))
 
       (dolist (group (split-sort-pathnames dir group-directories sort-by))
         (unless show-all
@@ -1120,7 +1158,7 @@
                                           (declare (ignore stream))
                                           (pretty-pretty-pathname x *standard-output* :long-name full-names)))
                  (goatee::reposition-stream-cursor *standard-output*)                 
-                 (vertical-gap T))
+                 (vertical-gap t))
           (list (dolist (ent group)
                   (let ((ent (merge-pathnames ent pathname))) ;; This is for CMUCL, see above. (fixme!)
                                                               ;; And breaks some things for SBCL.. (mgr) 
@@ -1131,7 +1169,7 @@
   (clim:pathname com-show-directory filesystem-commands :gesture :select
 		 :pointer-documentation ((object stream)
 					 (format stream "Show directory ~A" object))
-                 :tester-definitive T
+                 :tester-definitive t
 		 :tester ((object)
 			  (directoryp object)))
   (object)
@@ -1147,7 +1185,7 @@
            (note "~A does not exist." pathname))
           ((not (directoryp pathname))
            (note "~A is not a directory." pathname))
-          (T (change-directory (merge-pathnames pathname))) )))
+          (t (change-directory (merge-pathnames pathname))) )))
 
 (define-command (com-up-directory :name "Up Directory"
                                   :menu t
@@ -1156,8 +1194,8 @@
   (let ((parent (parent-directory *default-pathname-defaults*)))
     (when parent
       (change-directory parent)
-      (italic (T)
-        (format T "~&The current directory is now ")
+      (italic (t)
+        (format t "~&The current directory is now ")
         (present (truename parent))
         (terpri)))))
   
@@ -1222,7 +1260,7 @@
 ;; So.. yeah.
 
 (defun automagic-translator (pathname)
-  "Returns  values, the command translation, and a documentation string for the translation."  
+  "Returns 2 values: the command translation, and a documentation string for the translation."
   (cond ((wild-pathname-p pathname)
          (values `(com-show-directory ,pathname)
                  "Show Matching Files"
@@ -1283,18 +1321,18 @@
              (directoryp pathname));; FIXME: Need smart conversion to directories, here and elsewhere.
         (progn (push *default-pathname-defaults* *directory-stack*)
                (com-change-directory pathname))
-        (italic (T)
+        (italic (t)
            (fresh-line) (present (truename pathname))
-           (format T " does not exist or is not a directory.~%")) ))
+           (format t " does not exist or is not a directory.~%")) ))
   (compute-dirstack-command-eligibility *application-frame*))
 
 (defun comment-on-dir-stack ()
   (if *directory-stack*
       (progn
-        (format T "~&The top of the directory stack is now ")
+        (format t "~&The top of the directory stack is now ")
         (present (truename (first *directory-stack*)))
         (terpri))
-    (format T "~&The directory stack is now empty.~%")))
+    (format t "~&The directory stack is now empty.~%")))
 
 (define-command (com-pop-directory :name "Pop Directory"
                                   :menu t
@@ -1304,16 +1342,16 @@
       (note "The directory stack is empty!")
     (progn 
       (com-change-directory (pop *directory-stack*))
-      (italic (T) (comment-on-dir-stack))))
+      (italic (t) (comment-on-dir-stack))))
   (compute-dirstack-command-eligibility *application-frame*))
 
 (define-command (com-drop-directory :name "Drop Directory"
                                     :menu t
                                     :command-table directory-stack-commands)
   ()
-  (italic (T)
+  (italic (t)
     (if (null *directory-stack*)
-        (format T "~&The directory stack is empty!~%")
+        (format t "~&The directory stack is empty!~%")
       (progn
         (setf *directory-stack* (rest *directory-stack*))
         (comment-on-dir-stack))))
@@ -1323,9 +1361,9 @@
                                     :menu t
                                     :command-table directory-stack-commands)
   ()
-  (italic (T)
+  (italic (t)
     (if (null *directory-stack*)
-        (format T "~&The directory stack is empty!~%")
+        (format t "~&The directory stack is empty!~%")
       (progn
         (psetf (first *directory-stack*) *default-pathname-defaults*
                *default-pathname-defaults* (first *directory-stack*))
@@ -1379,19 +1417,14 @@
 				     :command-table lisp-commands
                                      :menu t
 				     :provide-output-destination-keyword nil)
-  ((symbol 'symbol :prompt "function-name"))
-  (clim-sys:make-process (lambda () (ed symbol))))
-
-(defun editable-definition-p (symbol)
-  (fboundp symbol))
+  ((function-name 'function-name :prompt "function name"))
+  (clim-sys:make-process (lambda () (ed function-name))))
 
 (define-presentation-to-command-translator edit-definition
-  (symbol com-edit-definition lisp-commands :gesture :select
+  (function-name com-edit-definition lisp-commands :gesture :select
 	  :pointer-documentation ((object stream)
 				  (format stream "Edit Definition of ~A" object))
-	  :documentation ((stream) (format stream "Edit Definition"))
-	  :tester ((object)
-		   (editable-definition-p object)))
+	  :documentation ((stream) (format stream "Edit Definition")))
   (object)
   (list object))
 		   
@@ -1408,27 +1441,47 @@
 
 ;;; Eval
 
-(defun hackish-present (object)
-  "Hack of the day.. let McCLIM determine presentation type to use, except for lists, because the list presentation method is inappropriate for lisp return values."
-  (typecase object
-    (sequence (present object 'expression))
-    (T (present object))))
-
 (defun display-evalues (values)
-  (with-drawing-options (T :ink +olivedrab+)
-    (cond ((null values)
-           (format T "No values.~%"))
-          ((= 1 (length values))           
-           (hackish-present (first values))
-           (fresh-line))
-          (T (do ((i 0 (1+ i))
-                  (item values (rest item)))
-                 ((null item))           
-               (with-drawing-options (T :ink +limegreen+)
-                 (with-text-style (T (make-text-style nil :italic :small))
-                   (format T "~A  " i)))
-                 (hackish-present (first item))
-                 (fresh-line))))))
+  (labels
+      ((present-value (value)         
+         ;; I would really prefer this to behave as below, as presenting
+         ;; things as expressions causes translators applicable to expression
+         ;; to override those which would be otherwise applicable (such as
+         ;; the set-current-package translator). I retain the use of w-o-a-p,
+         ;; swapping the inner/outer presentation types, with the assumption
+         ;; that someone (the form reader?) really does want expressions, and
+         ;; the presentation-type-of is seldom a subtype of expression.
+         ;; Aside from that, the problem with my code below is that it
+         ;; will use the default presentation method for the type, which will
+         ;; not necessarily print in the fashion expected from the lisp REPL.
+         ;; Possibly this +listener-view+ could save the day here, but I'm
+         ;; unclear on why it exists.   --Hefner
+
+         ;; Okay, set-current-package translator now mysteriously works, but
+         ;; I stand by the notion that 'expression should not be the type of
+         ;; the innermost presentation.
+         
+         #+(or)
+         (with-output-as-presentation (t value 'expression :single-box t)
+           (present value (presentation-type-of value) :single-box t))
+
+         (with-output-as-presentation (t value (presentation-type-of value)
+                                         :single-box t)
+           (present value 'expression))))
+    (with-drawing-options (t :ink +olivedrab+)
+      (cond ((null values) #+NIL (format t "No values.~%"))
+            ((= 1 (length values))             
+             (present-value (first values))
+             (fresh-line))
+            (t (do* ((i 0 (1+ i))
+                     (items values (rest items))
+                     (object (first items) (first items)))
+                    ((null items))
+               (with-drawing-options (t :ink +limegreen+)
+                 (with-text-style (t (make-text-style nil :italic :small))
+                   (format t "~A  " i)))
+                 (present-value object)
+                 (fresh-line)))))))
 
 (defun shuffle-specials (form values)
   (setf +++ ++
@@ -1442,8 +1495,9 @@
         *   (first values)))
 
 (define-command (com-eval :menu t :command-table lisp-commands)
-    ((form 'clim:form :prompt "form"))  
-  (let ((values (multiple-value-list (eval form))))
+    ((form 'clim:form :prompt "form"))
+  (let* ((- form)
+         (values (multiple-value-list (eval form))))
     (fresh-line)
     (shuffle-specials form values)
     (display-evalues values)
@@ -1484,8 +1538,8 @@
               (commands (cdr foo)))
           (invoke-as-heading
            (lambda ()
-             (format T "Command table ")
-             (with-output-as-presentation (t ct 'clim:command-table)
+             (format t "Command table ")
+             (with-output-as-presentation (t ct 'clim:command-table :single-box t)
                (princ (command-table-name ct)))))
           (if commands
               (format-items commands :printer (lambda (cmd s) (present cmd 'clim:command-name :stream s))
@@ -1528,3 +1582,14 @@
 				 :provide-output-destination-keyword nil)
     ((p 'package))
   (setf *package* p))
+
+(define-presentation-to-command-translator set-current-package
+    (package com-set-package lisp-commands
+             :pointer-documentation ((object stream)
+                                     (format stream "Set current package to ~A" (package-name object)))
+             :documentation ((stream) (format stream "Set Package"))
+             :menu t
+             :tester ((object) (not (eql *package* object))))
+    (object)
+  (list object))
+    
