@@ -6,6 +6,7 @@
 ;;;           Julien Boninfante (boninfan@emi.u-bordeaux.fr)
 ;;;           Robert Strandh (strandh@labri.u-bordeaux.fr)
 ;;;  (c) copyright 2001 by Arnaud Rouanet (rouanet@emi.u-bordeaux.fr)
+;;;  (c) copyright 1998,1999 by Gilbert Baumann
 
 ;;; This library is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU Library General Public
@@ -172,6 +173,11 @@
 		(setf (xlib:gcontext-clip-mask gc :unsorted) rect-seq)))))
       gc)))
 
+(defmethod medium-gcontext ((medium clx-medium) (ink (eql +transparent-ink+)))
+  (let ((drawable (port-lookup-mirror (port medium) (medium-sheet medium))))
+    (with-slots (gc) medium
+      (or gc (setf gc (xlib:create-gcontext :drawable drawable))))))
+
 (defmethod medium-gcontext ((medium clx-medium) (ink (eql +foreground-ink+)))
   (medium-gcontext medium (medium-foreground medium)))
 
@@ -220,30 +226,278 @@
           (setf (gethash ink design-cache)
                 (call-next-method))))))
 
+(defun st3 (x y z)
+  (values (logand (truncate (* x 255)) 255)
+          (logand (truncate (* y 255)) 255)
+          (logand (truncate (* z 255)) 255)))
+
+(declaim (ftype (function (sequence)
+                  (values (simple-array (unsigned-byte 8) 1)
+                          (simple-array (unsigned-byte 8) 1)
+                          (simple-array (unsigned-byte 8) 1)
+                          (simple-array (unsigned-byte 8) 1)))
+                inks-to-rgb))
+
+(defun inks-to-rgb (inks)
+  "Returns four values: byte arrays for the red, green, blue, and opacity components [0,255] of a sequence of inks"
+  (let ((red-map (make-array (length inks) :element-type '(unsigned-byte 8)
+                             :initial-element 255))
+        (green-map (make-array (length inks) :element-type '(unsigned-byte 8)
+                             :initial-element 0))
+        (blue-map (make-array (length inks) :element-type '(unsigned-byte 8)
+                             :initial-element 255))
+        (opacity-map (make-array (length inks) :element-type '(unsigned-byte 8)
+                             :initial-element 255))
+        (length (length inks)))
+    (loop for index from 0 below length
+          as ink = (elt inks index)
+          do (flet ((transform (parameter) (logand (truncate (* parameter 255)) 255)))
+               (cond
+                 ((colorp ink)
+                  (multiple-value-bind (r g b) (color-rgb ink)
+                    (setf (elt red-map   index) (transform r)
+                          (elt green-map index) (transform g)
+                          (elt blue-map  index) (transform b)
+                          (elt opacity-map index) 255)))
+                 ((eq ink +transparent-ink+)
+                  (setf (elt opacity-map index) 0)))))
+    (values red-map green-map blue-map opacity-map)))
+
+(defun integer-count-bits (integer)
+  (loop for i from 0 below (integer-length integer)
+        sum (ldb (byte 1 i) integer)))
+
+(defun compute-channel-fields (mask num-bytes)
+  (loop with counted-bits = 0
+        with output-width = (integer-count-bits mask)
+        for index from (1- num-bytes) downto 0
+        as submask = (ldb (byte 8 (* 8 index)) mask)
+        as submask-bits = (integer-count-bits submask)
+        as output-shift-left = (- (integer-length submask) submask-bits)
+        as input-position = (+ (- 8 counted-bits submask-bits))
+        collect (if (zerop submask)
+                    nil
+                    (prog1
+                        (list output-shift-left submask-bits input-position)
+                      (assert (<= output-width 8))
+                      (incf counted-bits submask-bits)))))
+
+(defun compute-channel-expressions (channel-mask-specs num-bytes)
+  (labels ((single-channel-expressions (mask channel-name)
+             (mapcar (lambda (fieldspec)
+                       (and fieldspec
+                            (destructuring-bind (output-shift-left submask-bits input-position)
+                                fieldspec
+                              `(ash (ldb (byte ,submask-bits ,input-position) ,channel-name) ,output-shift-left))))
+                     (compute-channel-fields mask num-bytes) )))
+    (reduce (lambda (left-exprs right-exprs)
+              (mapcar (lambda (left-expr right-expr)
+                        (if right-expr
+                            (cons right-expr left-expr)
+                            left-expr))
+                      left-exprs
+                      right-exprs))
+            channel-mask-specs
+            :key (lambda (channel-mask-spec)
+                   (destructuring-bind (var-name mask) channel-mask-spec
+                   (single-channel-expressions mask var-name)))
+            :initial-value (map 'list #'identity (make-array num-bytes :initial-element nil)))))
+
+(defun generate-pixel-assignments (array-var index-var channel-mask-specs num-bytes byte-order)
+  `(setf ,@(mapcan (lambda (byte-exprs byte-index)
+                     (and byte-exprs
+                          (list `(elt ,array-var (+ ,index-var ,byte-index))
+                                (if (= 1 (length byte-exprs))
+                                    (first byte-exprs)
+                                    `(logior ,@byte-exprs)))))
+                   (compute-channel-expressions channel-mask-specs num-bytes)
+                   (funcall (ecase byte-order
+                              (:lsbfirst #'reverse)
+                              (:msbfirst #'identity))
+                            (loop for i from 0 below num-bytes collect i)))))
+
+(defun generate-indexed-converter-expr (rgb-masks num-bytes byte-order)
+  `(lambda (image-array converted-data mask-data width height inks)  
+    (declare (optimize (speed 3)
+                       (safety 0)
+                       (space 0)
+                       (debug 0))
+     (type xlib:card16 width height)
+     (type (simple-array xlib:card8 1) converted-data mask-data))
+    (macrolet ((conversion-body ()
+                 `(let ((index 0)
+                        (mask-index 0)
+                        (mask-bitcursor 1))
+                   (declare (type (unsigned-byte 9) mask-bitcursor)
+                    (type xlib:array-index mask-index index))
+                   
+                   (multiple-value-bind (red-map green-map blue-map opacity-map) (inks-to-rgb inks)
+                     (dotimes (y height)
+                       (unless (= 1 mask-bitcursor)
+                         (setf mask-bitcursor 1
+                               mask-index (1+ mask-index)))
+                       (dotimes (x width)
+                         (let ((ink-index (aref image-array y x)))
+                           (when (< (elt opacity-map ink-index) #x40)  ; FIXME? Arbitrary threshold.
+                             (setf (elt mask-data mask-index) (logxor (elt mask-data mask-index) mask-bitcursor)))
+                           #+NIL
+                           (setf (elt converted-data (+ index 0)) (elt blue-map  ink-index)
+                                 (elt converted-data (+ index 1)) (elt green-map ink-index)
+                                 (elt converted-data (+ index 2)) (elt red-map   ink-index))
+                           (let ((red   (elt red-map ink-index))
+                                 (green (elt green-map ink-index))
+                                 (blue  (elt blue-map ink-index)))                             
+                             ,',(generate-pixel-assignments 'converted-data 'index
+                                                            (mapcar #'list '(red green blue) rgb-masks)
+                                                            num-bytes byte-order))
+                           (setf index (+ ,',num-bytes index)
+                                 mask-bitcursor (ash mask-bitcursor 1)
+                                 mask-index (+ mask-index (ash mask-bitcursor -8))
+                                 mask-bitcursor (logand (logior mask-bitcursor
+                                                                (ash mask-bitcursor -8))
+                                                        #xff)))))))))
+      ;; We win big if we produce several specialized versions of this according
+      ;; to the type of array holding the color indexes.
+    (typecase image-array
+      ((simple-array xlib:card8 2)      ; 256-color images
+       (locally (declare (type (simple-array xlib:card8 2) image-array)) (conversion-body)))
+      ((simple-array fixnum 2)          ; High-color index images (XPM reader produces these..)
+       (locally (declare (type (simple-array fixnum 2) image-array)) (conversion-body)))
+      (t (conversion-body))))))
+
+(defun convert-indexed->mask (image-array mask-data width height inks)
+  (declare (optimize (speed 3)
+                     (safety 0)
+                     (space 0)
+                     (debug 0))
+           (type xlib:card16 width height)
+           (type (simple-array xlib:card8 1) mask-data))
+  (macrolet ((conversion-body ()
+              '(let ((mask-index 0)
+                     (mask-bitcursor 1))
+                 (declare (type (unsigned-byte 9) mask-bitcursor)
+                          (type xlib:array-index mask-index))
+                 
+                 (multiple-value-bind (red-map green-map blue-map opacity-map) (inks-to-rgb inks)
+                   (declare (ignore red-map green-map blue-map))
+                   
+                   (dotimes (y height)
+                     (unless (= 1 mask-bitcursor)
+                       (setf mask-bitcursor 1
+                             mask-index (1+ mask-index)))
+                     (dotimes (x width)
+                       (let ((ink-index (aref image-array y x)))
+                         (when (< (elt opacity-map ink-index) #x40)  ; FIXME? Arbitrary threshold.
+                           (setf (elt mask-data mask-index) (logxor (elt mask-data mask-index) mask-bitcursor)))
+                         (setf mask-bitcursor (ash mask-bitcursor 1)
+                               mask-index (+ mask-index (ash mask-bitcursor -8))
+                               mask-bitcursor (logand (logior mask-bitcursor
+                                                              (ash mask-bitcursor -8))
+                                                      #xff)))))))))
+    ;; Again, we win big if we produce several specialized versions of this.
+    (typecase image-array
+      ((simple-array xlib:card8 2)      ; 256-color images
+       (locally (declare (type (simple-array xlib:card8 2) image-array)) (conversion-body)))
+      ((simple-array fixnum 2)          ; High-color index images (XPM reader produces these..)
+       (locally (declare (type (simple-array fixnum 2) image-array)) (conversion-body)))
+      (t (conversion-body)))))
+
+(defparameter *pixel-converter-cache* (make-hash-table :test 'equal))
+
+(defun get-indexed-converter (visual-info byte-order bytes-per-pixel)
+  (let* ((rgb-masks (list (xlib:visual-info-red-mask visual-info)
+                          (xlib:visual-info-green-mask visual-info)
+                          (xlib:visual-info-blue-mask visual-info)))
+         (key (list rgb-masks byte-order bytes-per-pixel)))
+    (symbol-macrolet ((fn (gethash key *pixel-converter-cache*)))
+        (or fn (setf fn (compile nil (generate-indexed-converter-expr rgb-masks bytes-per-pixel byte-order)))))))
+
+(defun fill-pixmap-indexed (visual-info depth byte-order array pm pm-gc mask mask-gc w h inks)
+  (assert (= (array-total-size array) (* w h)))
+  (let* ((ceil-w-8 (ceiling w 8))
+         (bytes-per-pixel
+          (case depth
+            ((24 32) 4)
+            ((15 16) 2)
+            (otherwise nil)))
+         (mask-data (make-array (* ceil-w-8 h)
+                                :element-type '(unsigned-byte 8)
+                                :initial-element #xff))
+         (pixel-converter nil))
+         
+    (if (and bytes-per-pixel
+             (member byte-order '(:lsbfirst :msbfirst))
+             (setf pixel-converter (get-indexed-converter visual-info byte-order bytes-per-pixel)))
+        ;; Fast path - Image upload
+        (let ((converted-data (make-array (* bytes-per-pixel (array-total-size array)) :element-type 'xlib:card8)))
+          ;; Fill the pixel arrays
+          (funcall pixel-converter array converted-data mask-data w h inks)
+    
+          ;; Create an xlib "image" and copy it to our pixmap.
+          ;; I do this because I'm not smart enough to operate xlib:put-raw-image.
+          (let ((image (xlib:create-image :bits-per-pixel (* 8 bytes-per-pixel) :depth depth
+                                          :width w :height h
+                                          :format :z-pixmap
+                                          :data converted-data)))
+            (xlib:put-image (pixmap-mirror pm) pm-gc image
+                            :x 0 :y 0
+                            :width w :height h)))
+        
+        ;; Fallback for unsupported visual, plotting pixels
+        (progn
+          (dotimes (y h)
+            (dotimes (x w)
+              (let ((ink (elt inks (aref array y x))))
+                (unless (eq ink +transparent-ink+) 
+                  (draw-point* pm x y :ink ink)))))
+          (convert-indexed->mask array mask-data w h inks)))
+
+    ;; We can use image upload for the mask in either case.
+    (let ((mask-image (xlib:create-image :bits-per-pixel 1 :depth 1
+                                         :bit-lsb-first-p t
+                                         :byte-lsb-first-p t                                          
+                                         :width w :height h
+                                         :data mask-data)))
+      (xlib:put-image mask mask-gc mask-image
+                      :x 0 :y 0
+                      :width w :height h))))
+
 (defmethod design-gcontext ((medium clx-medium) (ink climi::indexed-pattern))
   (let* ((array (slot-value ink 'climi::array))
-         (inks  (slot-value ink 'climi::designs))
+         (inks  (map 'vector
+                     (lambda (ink)
+                       (cond
+                         ((eql ink +foreground-ink+) (medium-foreground medium))
+                         ((eql ink +background-ink+) (medium-background medium))
+                         ((eql ink +flipping-ink+)
+                          (error "Flipping ink within patterns is not supported."))
+                         (t ink)))
+                     (slot-value ink 'climi::designs)))
          (w     (array-dimension array 1))
          (h     (array-dimension array 0)))
-    (let* ((pm   (allocate-pixmap (first (port-grafts (port medium))) w h))
-           (mask (xlib:create-pixmap :drawable (port-lookup-mirror
+    (assert (not (zerop w)))
+    (assert (not (zerop h)))
+
+    ;; Establish color and mask pixmaps
+    (let* ((display (clx-port-display (port medium)))
+           (screen  (clx-port-screen  (port medium)))
+           (drawable (port-lookup-mirror (port medium) (medium-sheet medium)))
+           (pm   (allocate-pixmap (first (port-grafts (port medium))) w h))
+           (mask (xlib:create-pixmap :drawable drawable
+                                     #+NIL
+                                               (port-lookup-mirror
                                                 (port medium)
                                                 (first (port-grafts (port medium))))
                                      :depth 1
                                      :width w
                                      :height h))
+           (pm-gc (xlib:create-gcontext :drawable (pixmap-mirror pm)))
            (mask-gc (xlib:create-gcontext :drawable mask :foreground 1)))
+
       (xlib:draw-rectangle mask mask-gc 0 0 w h t)
       (setf (xlib:gcontext-foreground mask-gc) 0)
-      (dotimes (y h)
-        (dotimes (x w)
-          (let ((ink (elt inks (aref array y x))))
-            (cond ((eq ink +transparent-ink+)
-                   (xlib:draw-point mask mask-gc x y))
-                  (t
-                   (draw-point* pm x y :ink ink))))))
-      (xlib:free-gcontext mask-gc)
-      (let ((gc (xlib:create-gcontext :drawable (port-lookup-mirror (port medium) (medium-sheet medium)))))
+      
+      (let ((gc (xlib:create-gcontext :drawable drawable)))
         (setf (xlib:gcontext-fill-style gc) :tiled
               (xlib:gcontext-tile gc) (port-lookup-mirror (port pm) pm)
               (xlib:gcontext-clip-x gc) 0
@@ -251,6 +505,19 @@
               (xlib:gcontext-ts-x gc) 0
               (xlib:gcontext-ts-y gc) 0
               (xlib:gcontext-clip-mask gc) mask)
+
+        (let ((byte-order (xlib:display-byte-order display))
+              ;; Hmm. Pixmaps are not windows, so you can't query their visual.
+              ;; We'd like to draw to pixmaps as well as windows, so use the
+              ;; depth and visual of the screen root, and hope this works.              
+              ;(visual-info (xlib:window-visual-info drawable))
+              (visual-info (xlib:visual-info display (xlib:screen-root-visual screen)))
+              (depth (xlib:screen-root-depth screen))
+              (*print-base* 16))
+          (fill-pixmap-indexed visual-info depth byte-order array pm pm-gc mask mask-gc w h inks))
+          
+        (xlib:free-gcontext mask-gc)
+        (xlib:free-gcontext pm-gc)
         gc))))
 
 (defmethod design-gcontext ((medium clx-medium) (ink climi::rectangular-tile))
@@ -327,8 +594,9 @@
 	     (ink        (medium-ink ,medium))
 	     (gc         (medium-gcontext ,medium ink)))
 	line-style ink
-	(unwind-protect
-	     (progn ,@body)
+	(unwind-protect             
+	     (unless (eql ink +transparent-ink+)
+               (progn ,@body))
 	  #+ignore(xlib:free-gcontext gc))))))
 
 
@@ -376,9 +644,10 @@
 (defmethod medium-copy-area ((from-drawable pixmap) from-x from-y width height
                              (to-drawable pixmap) to-x to-y)
   (xlib:copy-area (pixmap-mirror from-drawable)
-                  (medium-gcontext from-drawable +background-ink+)
+                  (medium-gcontext (sheet-medium (slot-value to-drawable 'sheet))
+                                   +background-ink+)              
                   (round-coordinate from-x) (round-coordinate from-y)
-		  (round width) (round height)
+                  (round width) (round height)
                   (pixmap-mirror to-drawable)
                   (round-coordinate to-x) (round-coordinate to-y)))
 
@@ -757,7 +1026,7 @@
                                           font-ascent font-descent
                                           direction first-not-done))
                       (multiple-value-bind (minx miny maxx maxy)
-                          (text-bounding-rectangle*
+                          (climi::text-bounding-rectangle*
                            medium string :text-style text-style
                            :start (1+ position-newline) :end end)
                         (values (min minx left) (- ascent)
@@ -890,3 +1159,152 @@
 	  (setf (medium-buffer medium) nil)))
       (funcall continuation)))
 
+
+;;; RGB-IMAGE support, from Closure
+
+(defmethod climi::medium-draw-image-design*
+    ((medium clx-medium) (design climi::rgb-image-design) x y)
+  (let* ((da (sheet-direct-mirror (medium-sheet medium)))
+	 (image (slot-value design 'climi::image))
+	 (width (climi::image-width image))
+	 (height (climi::image-height image)))
+    (destructuring-bind (&optional pixmap mask)
+	(slot-value design 'climi::medium-data)
+      (unless pixmap
+	(setf pixmap (compute-rgb-image-pixmap da image))
+	(when (climi::image-alpha-p image)
+	  (setf mask (compute-rgb-image-mask da image)))
+	(setf (slot-value design 'climi::medium-data) (list pixmap mask)))
+      (multiple-value-bind (x y)
+	  (transform-position
+	   (sheet-device-transformation (medium-sheet medium))
+	   x y)
+	(setf x (round x))
+	(setf y (round y))
+	(let ((gcontext (xlib:create-gcontext :drawable da)))
+	  (cond
+	    (mask
+	      (xlib:with-gcontext (gcontext 
+				   :clip-mask mask
+				   :clip-x x
+				   :clip-y (- y height))
+		(xlib:copy-area pixmap gcontext 0 0 width height
+				da x (- y height))))
+	    (t
+	      (xlib:copy-area pixmap gcontext 0 0 width height
+			      da x (- y height)))))))))
+
+(defmethod climi::medium-free-image-design
+    ((medium clx-medium) (design climi::rgb-image-design))
+  (destructuring-bind (&optional pixmap mask)
+      (slot-value design 'climi::medium-data)
+    (when pixmap
+      (xlib:free-pixmap pixmap)
+      (when mask
+	(xlib:free-pixmap mask))
+      (setf (slot-value design 'climi::medium-data) nil))))
+
+(defun compute-rgb-image-pixmap (drawable image)
+  (let* ((width (climi::image-width image))
+         (height (climi::image-height image))
+         (depth (xlib:drawable-depth drawable))
+         (im (image-to-ximage-for-drawable drawable image)))
+    (setf width (max width 1))
+    (setf height (max height 1))
+    (let* ((pixmap (xlib:create-pixmap :drawable drawable
+				       :width width
+				       :height height
+				       :depth depth))
+	   (gc     (xlib:create-gcontext :drawable pixmap)))
+      (unless (or (>= width 2048) (>= height 2048)) ;### CLX bug
+	(xlib:put-image pixmap gc im 
+			:src-x 0 :src-y 0
+			:x 0 :y 0
+			:width width :height height))
+      (xlib:free-gcontext gc)
+      pixmap)))
+
+(defun compute-rgb-image-mask (drawable image)
+  (let* ((width (climi::image-width image))
+         (height (climi::image-height image))
+         (bitmap (xlib:create-pixmap :drawable drawable
+                                     :width width 
+                                     :height height
+                                     :depth 1))
+         (gc (xlib:create-gcontext :drawable bitmap
+				   :foreground 1
+				   :background 0))
+         (idata (climi::image-data image))
+         (xdata (make-array (list height width)
+			    :element-type '(unsigned-byte 1)))
+         (im (xlib:create-image :width width
+                                :height height
+                                :depth 1
+                                :data xdata)) )
+    (dotimes (y width)
+      (dotimes (x height)
+        (if (> (aref idata x y) #x80000000)
+            (setf (aref xdata x y) 0)
+	    (setf (aref xdata x y) 1))))
+    (unless (or (>= width 2048) (>= height 2048)) ;### CLX breaks here
+      (xlib:put-image bitmap gc im :src-x 0 :src-y 0
+		      :x 0 :y 0 :width width :height height
+		      :bitmap-p nil))
+    (xlib:free-gcontext gc)
+    bitmap))
+
+(defun image-to-ximage-for-drawable (drawable image)
+  (image-to-ximage image
+		   (xlib:drawable-depth drawable) 
+		   (pixel-translator (xlib:window-colormap drawable))))
+
+(defun image-to-ximage (image depth translator)
+  (let* ((width (climi::image-width image))
+         (height (climi::image-height image))
+         (idata (climi::image-data image))
+	 ;; FIXME: this (and the :BITS-PER-PIXEL, below) is a hack on
+	 ;; top of a hack.  At some point in the past, XFree86 and/or
+	 ;; X.org decided that they would no longer support pixmaps
+	 ;; with 24 bpp, which seems to be what most AIMAGEs want to
+	 ;; be.  For now, force everything to a 32-bit pixmap.
+         (xdata (make-array (list height width) :element-type '(unsigned-byte 32)))
+         (ximage (xlib:create-image :width  width
+                                    :height height
+                                    :depth  depth
+				    :bits-per-pixel 32
+                                    :data   xdata)))
+    (declare (type (simple-array (unsigned-byte 32) (* *)) idata))
+    (loop for x fixnum from 0 below width do
+	  (loop for y fixnum from 0 below height do
+		(setf (aref xdata y x) 
+		      (funcall translator
+			       x y
+			       (ldb (byte 24 0) (aref idata y x))))))
+    ximage))
+
+(defun mask->byte (mask)
+  (let ((h (integer-length mask)))
+    (let ((l (integer-length (logxor mask (1- (ash 1 h))))))
+      (byte (- h l) l))))
+
+;; fixme!  This is not just incomplete, but also incorrect: The original
+;; true color code knew how to deal with non-linear RGB value
+;; allocation.
+(defun pixel-translator (colormap)
+  (unless (eq (xlib:visual-info-class (xlib:colormap-visual-info colormap))
+	      :true-color)
+    (error "sorry, cannot draw rgb image for non-true-color drawable yet"))
+  colormap
+  (let* ((info (xlib:colormap-visual-info colormap))
+	 (rbyte (mask->byte (xlib:visual-info-red-mask info)))
+	 (gbyte (mask->byte (xlib:visual-info-green-mask info)))
+	 (bbyte (mask->byte (xlib:visual-info-blue-mask info))))
+    (lambda (x y sample)
+      (declare (ignore x y))
+      (dpb (the (unsigned-byte 8) (ldb (byte 8 0) sample))
+	   rbyte
+	   (dpb (the (unsigned-byte 8) (ldb (byte 8 8) sample))
+		gbyte
+		(dpb (the (unsigned-byte 8) (ldb (byte 8 16) sample))
+		     bbyte
+		     0))))))
