@@ -137,6 +137,7 @@
   (let ((entry (fontset-point point fontset)))
     (if entry
         (destructuring-bind ((range-start . range-stop) font translator) entry
+	  (declare (ignore range-start range-stop))
           (xlib:char-width font (funcall translator point)))
         0)))
 
@@ -163,7 +164,8 @@
    (pointer :reader port-pointer)
    (pointer-grab-sheet :accessor pointer-grab-sheet :initform nil)
    (selection-owner :initform nil :accessor selection-owner)
-   (selection-timestamp :initform nil :accessor selection-timestamp)))
+   (selection-timestamp :initform nil :accessor selection-timestamp)
+   (font-families :accessor font-families)))
 
 (defun parse-clx-server-path (path)
   (pop path)
@@ -349,6 +351,10 @@
 					      :structure-notify
 					      :pointer-motion
 					      :button-motion)))
+  ;; I am declaring BORDER-WIDTH ignore to get a cleaner build, but I
+  ;; don't really understand why the use of it is commented out in favor
+  ;; of the constant 0.  -- RS 2007-07-22
+  (declare (ignore border-width))
   (when (null (port-lookup-mirror port sheet))
     (update-mirror-geometry sheet)
     (let* ((desired-color (typecase sheet
@@ -431,7 +437,7 @@
       (setf (xlib:wm-hints window) (xlib:make-wm-hints :input :on))
       (setf (xlib:wm-name window) (frame-pretty-name frame))
       (setf (xlib:wm-icon-name window) (frame-pretty-name frame))
-      (setf (xlib:wm-protocols window) `(:wm_delete_window :wm_take_focus)))))
+      (setf (xlib:wm-protocols window) `(:wm_delete_window)))))
 
 (defmethod realize-mirror ((port clx-port) (sheet unmanaged-top-level-sheet-pane))
   (realize-mirror-aux port sheet
@@ -662,10 +668,11 @@
 		      type width height x y root-x root-y
 		      data override-redirect-p send-event-p hint-p
                       target property requestor selection
+                      request first-keycode count
                       &allow-other-keys)
+  (declare (ignore display request first-keycode count))
   (declare (special *clx-port*))
-  (let ((sheet (and window
-		    (port-lookup-sheet *clx-port* window))))
+  (let ((sheet (and window (port-lookup-sheet *clx-port* window))))
     (when sheet
       (case event-key
 	((:key-press :key-release)
@@ -680,7 +687,8 @@
 	     :x x :y y
 	     :graft-x root-x
 	     :graft-y root-y
-             :sheet sheet :modifier-state modifier-state :timestamp time)))
+             :sheet (or (frame-properties (pane-frame sheet) 'focus) sheet)
+             :modifier-state modifier-state :timestamp time)))
 	((:button-press :button-release)
 	 (let ((modifier-state (clim-xcommon:x-event-state-modifiers *clx-port*
 								     state)))
@@ -841,22 +849,7 @@
 
 (defmethod port-wm-protocols-message (sheet time (message (eql :wm_delete_window)) data)
   (declare (ignore data))
-  (make-instance 'window-manager-delete-event
-                 :sheet sheet
-                 :timestamp time))
-
-(defmethod port-wm-protocols-message (sheet time (message (eql :wm_take_focus)) data)
-  (when time
-    (format *trace-output* "~&;; In :WM_TAKE_FOCUS, TIME is not NIL: ~S" time))
-  (let* ((frame (pane-frame sheet))
-         (focus (climi::keyboard-input-focus frame))
-	 ;; FIXME: Do I really have to use ELT here?  The CLX manual
-	 ;; says (sequence integer), so I suppose I do.
-	 (timestamp (elt data 1)))
-    (when (and focus (sheet-mirror focus))
-      (xlib:set-input-focus (clx-port-display *clx-port*)
-                            (sheet-mirror focus) :parent timestamp)
-      nil)))
+  (make-instance 'window-manager-delete-event :sheet sheet :timestamp time))
 
 (defmethod port-wm-protocols-message (sheet time (message t) data)
   (warn "Unprocessed WM Protocols message: ~:_message = ~S;~:_ data = ~S;~_ sheet = ~S."
@@ -1154,13 +1147,10 @@
 	       ;; reasonable timestamp.
 	       :timestamp 0))))))))
   
-
-;;; Set the keyboard input focus for the port.
-
-(defmethod %set-port-keyboard-focus ((port clx-port) focus &key timestamp)
-  (let ((mirror (sheet-mirror focus)))
-    (when mirror
-      (xlib:set-input-focus (clx-port-display port) mirror :parent timestamp))))
+(defmethod port-frame-keyboard-input-focus ((port clx-port) frame)
+  (frame-properties frame 'focus))
+(defmethod (setf port-frame-keyboard-input-focus) (focus (port clx-port) frame)
+  (setf (frame-properties frame 'focus) focus))
 
 (defmethod port-force-output ((port clx-port))
   (xlib:display-force-output (clx-port-display port)))
@@ -1428,8 +1418,101 @@
 		     :remote-filename (format nil "/tmp/.X11-unix/X~D" display)
 		     :format :binary)
 		    (socket:make-socket :remote-host (string host)
-					:remote-port (+ *x-tcp-port* display)
+					:remote-port (+ xlib::*x-tcp-port*
+                                                        display)
 					:format :binary))))
     (if (streamp stream)
 	stream
       (error "Cannot connect to server: ~A:~D" host display))))
+
+
+;;;; Font listing implementation:
+
+(defclass clx-font-family (clim-extensions:font-family)
+    ((all-faces :initform nil
+		:accessor all-faces
+		:reader clim-extensions:font-family-all-faces)))
+
+(defclass clx-font-face (clim-extensions:font-face)
+    ((all-sizes :initform nil
+		:accessor all-sizes
+		:reader clim-extensions:font-face-all-sizes)))
+
+(defun split-font-name (name)
+  (loop
+      repeat 12
+      for next = (position #\- name :start 0)
+		 :then (position #\- name :start (1+ next))
+      and prev = nil then next
+      while next
+      when prev
+      collect (subseq name (1+ prev) next)))
+
+(defun reload-font-table (port)
+  (let ((table (make-hash-table :test 'equal)))
+    (dolist (font (xlib:list-font-names (clx-port-display port) "*"))
+      (destructuring-bind
+	  (&optional foundry family weight slant setwidth style pixelsize 
+	   &rest ignore		     	;pointsize xresolution yresolution
+					;spacing averagewidth registry encoding
+	   )
+	  (split-font-name font)
+	(declare (ignore setwidth style ignore))
+	(when family
+	  (let* ((family-name (format nil "~A ~A" foundry family))
+		 (family-instance
+		  (or (gethash family-name table)
+		      (setf (gethash family-name table)
+			    (make-instance 'clx-font-family
+			      :port port
+			      :name family-name))))
+		 (face-name (format nil "~A ~A" weight slant))
+		 (face-instance
+		  (find face-name (all-faces family-instance)
+			:key #'clim-extensions:font-face-name
+			:test #'equal)))
+	    (unless face-instance
+	      (setf face-instance
+		    (make-instance 'clx-font-face
+		      :family family-instance
+		      :name face-name))
+	      (push face-instance (all-faces family-instance)))
+	    (pushnew (parse-integer
+		      ;; FIXME: Python thinks pixelsize is NIL, resulting
+		      ;; in a full WARNING.  Let's COERCE to make it work.
+		      (coerce pixelsize 'string))
+		     (all-sizes face-instance))))))
+    (setf (font-families port)
+	  (sort (loop
+		    for family being each hash-value in table
+		    do
+		      (setf (all-faces family)
+			    (sort (all-faces family)
+				  #'string<
+				  :key #'clim-extensions:font-face-name))
+		      (dolist (face (all-faces family))
+			(setf (all-sizes face) (sort (all-sizes face) #'<)))
+		    collect family)
+		#'string<
+		:key #'clim-extensions:font-family-name))))
+
+(defmethod clim-extensions:port-all-font-families
+    ((port clx-port) &key invalidate-cache)
+  (when (or (not (slot-boundp port 'font-families)) invalidate-cache)
+    (reload-font-table port))
+  (font-families port))
+
+(defmethod clim-extensions:font-face-scalable-p ((face clx-font-face))
+  nil)
+
+(defun make-unfriendly-name (str)
+  (substitute #\- #\space str))
+
+(defmethod clim-extensions:font-face-text-style
+    ((face clx-font-face) &optional size)
+  (make-text-style (make-unfriendly-name
+		    (clim-extensions:font-family-name
+		     (clim-extensions:font-face-family face)))
+		   (make-unfriendly-name
+		    (clim-extensions:font-face-name face))
+		   size))
