@@ -26,7 +26,8 @@
 
 ;;; cheat and use this McCLIM internal class :)
 (defclass screen-area-cursor (clim-internals::cursor-mixin cursor)
-  ((screen-line :accessor screen-line :initarg :screen-line)))
+  ((screen-line :accessor screen-line :initarg :screen-line))
+  (:default-initargs :appearance :solid))
 
 (defmethod* (setf cursor-position) (nx ny (cursor screen-area-cursor))
   (declare (ignore nx ny))
@@ -218,11 +219,11 @@
   (let ((cursor (cursor record)))
     (multiple-value-bind (x1 y1 x2 y2) (call-next-method)
       (values x1 y1
-              (if cursor
+              (if (and cursor (eq (cursor-visibility cursor) :on))
                   (with-slots (climi::x climi::width) cursor
                      (max x2 (+ climi::x climi::width)))
                   x2)
-              (if cursor
+              (if (and cursor (eq (cursor-visibility cursor) :on))
                   (max y2 (+ y1 (climi::cursor-height cursor)))
                   y2)))))
 
@@ -298,7 +299,6 @@
 
 (defmethod climi::map-over-output-records-1 (function (record simple-screen-area)
 				      function-args)
-  (declare (ignore x-offset y-offset))
   (if function-args
       (loop for line = (area-first-line record) then (next line)
 	 while line
