@@ -1,4 +1,4 @@
-(in-package :CLIM-INTERNALS)
+(in-package :clim-internals)
 
 ;;;
 ;
@@ -14,12 +14,35 @@
 ;
 ;;;
 
+;;; TODO: Add units label to slider pane
+;;; TODO: Matching repaint method for the list pane
+;;; TODO: Is there a locking bug, and does it somehow involve pixie?
+;;;       (Or is my computer still haunted?)
+;;; TODO: Colors of buttons in clim-fig are wrong
+
 (export '(pixie-look #+clx pixie/clx-look))
 
 (defclass pixie-look (frame-manager) ())
 #+clx (defclass pixie/clx-look (pixie-look clim-clx::clx-frame-manager) ())
 
-; our stub inside clim proper
+
+(defmacro define-pixie-gadget (abstract-type pixie-type &key (enabled t))
+  `(defmethod make-pane-1 ((fm pixie-look)
+			   (frame application-frame)
+			   (type (eql ',abstract-type))
+			   &rest args)
+    (declare (ignorable fm frame type args))
+    ,(if enabled
+	 `(apply #'make-instance
+           ',pixie-type
+           :frame frame
+           :manager fm
+           :port (port frame)
+	   args)
+	 `(call-next-method))))
+
+;; Let us please stop playing these stupid symbol games.
+#+NIL
 (defmethod make-pane-1 ((fm pixie-look) (frame application-frame) type &rest args)
   (apply #'make-instance
          (or (find-symbol (concatenate 'string "PIXIE-" (symbol-name type)) :climi)
@@ -32,13 +55,45 @@
          :port (port frame)
          args))
 
+;;; Scroll button patterns
+
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defparameter +pixie-arrow-pattern+
+    #2a((0 0 0 1 0 0 0)
+        (0 0 1 1 1 0 0)
+        (0 1 1 1 1 1 0)
+        (1 1 1 1 1 1 1)))
+  
+  (flet ((rotate (array)
+	   (let ((new-array (make-array (reverse (array-dimensions array)))))
+	     (dotimes (i (array-dimension array 0))
+	       (dotimes (j (array-dimension array 1))
+		 (setf (aref new-array j (- (array-dimension array 0) i 1))
+		       (aref array i j))))
+	     new-array)))
+    (let* ((up    +pixie-arrow-pattern+)
+	   (right (rotate up))
+	   (down  (rotate right))
+	   (left  (rotate down)))
+      (macrolet ((def (var) 
+		     `(defparameter ,(intern (format nil "~A~A~A"
+						     (symbol-name '#:+pixie-)
+						     (symbol-name var)
+						     (symbol-name '#:-arrow+))
+					     (find-package :climi))
+		       (make-pattern ,var (list +transparent-ink+ +black+)))))
+	(def up)
+	(def right)
+	(def down)
+	(def left)))))
+
 ; Standard
 
 ; TODO - clean up all of this colour nonsense
 ; which should involve some sensible ideas about tints vs' inks
 
-(defclass pixie-gadget () (
-   (highlighted      :initarg :highlight
+(defclass pixie-gadget ()
+  ((highlighted      :initarg :highlight
                      :initform +gray93+
                      :reader pane-highlight)
    (paper-color      :initarg :paper-color
@@ -56,7 +111,6 @@
 
 ; Convenience
 
-
 (defun draw-up-box (pane x1 y1 x2 y2 foreground)
   (let ((x2 (- x2 1)))
     (draw-rectangle* pane x1 y1 x2 y2 :ink foreground)
@@ -64,15 +118,15 @@
     (draw-line* pane x1 y2 x1 y1 :ink +white+)
     (draw-line* pane x2 y1 x1 y1 :ink +white+)
     ;; now for the gray inline
-    (let ((x1 (+ x1 2))
-          (y1 (+ y1 2))
-          (x2 (- x2 1))
-          (y2 (- y2 1)))
-      (draw-line* pane x1 y2 (+ x2 1) y2 :ink +gray54+) ; <- not a typo
-      (draw-line* pane x2 y1 x2 (+ y2 1) :ink +gray54+))
+    (let ((x1 (+ x1 1))			; I'd prefer this be zero, so that there isn't
+          (y1 (+ y1 1))			; the little sparkling white pixel in both corners
+          (x2 (- x2 1))			; (bothersome in the corner of a scroller-pane),
+          (y2 (- y2 1)))		; but we may be transformed, so too much work. Bah.
+      (draw-line* pane x1 y2 x2 y2 :ink +gray54+)
+      (draw-line* pane x2 y1 x2 y2 :ink +gray54+))
     ;; now for the black outline
-    (draw-line* pane x1 y2 (+ x2 1) y2 :ink +black+)
-    (draw-line* pane x2 y1 x2 (+ y2 1) :ink +black+)
+    (draw-line* pane x1 y2 x2 y2 :ink +black+)
+    (draw-line* pane x2 y1 x2 y2 :ink +black+)
     (draw-label* pane x1 y1 x2 y2
                       :ink (pane-inking-color pane))))
 
@@ -88,13 +142,13 @@
         (y2 (- y2 2)))
     (draw-line* pane x1 y1 (+ x2 1) y1 :ink +black+)
     (draw-line* pane x1 y1 x1 (+ y2 1) :ink +black+))
-  ;; now for the black outline
-  (draw-line* pane x1 y2 (+ x2 1) y2 :ink +white+)
-  (draw-line* pane x2 y1 x2 (+ y2 1) :ink +white+)
+  ;; now for the white outline
+  (draw-line* pane x1 y2 x2 y2 :ink +white+)
+  (draw-line* pane x2 y1 x2 y2 :ink +white+)
   (draw-label* pane x1 y1 x2 y2
                :ink (pane-inking-color pane)))
 
-; Highlighting (could the defaults be less horrible?)
+; Highlighting
 
 (defmethod gadget-highlight-background ((gadget pixie-gadget))
   +gray93+)
@@ -141,6 +195,7 @@
 (defconstant +pixie-slider-thumb-height+      34)
 (defconstant +pixie-slider-thumb-half-width+   8)
 
+
 (defclass pixie-slider-pane (pixie-gadget draggable-arming-mixin slider-pane)
   ((dragging
     :initform nil)
@@ -160,6 +215,8 @@
     :border-style :inset
     :border-width 1))
 
+(define-pixie-gadget slider pixie-slider-pane)
+
 (defmethod compose-space ((pane pixie-slider-pane) &key width height)
   (declare (ignore width height))
   (if (eq (gadget-orientation pane) :vertical)
@@ -227,11 +284,13 @@
         (case token
           ((up-notch)
            (when (< (gadget-value pane) (gadget-max-value pane))
+	     #+NIL
              (clim-internals::schedule-timer-event pane token 0.1)
              (incf (gadget-value pane))
              (dispatch-repaint pane (sheet-region pane))))
           ((down-notch)
            (when (> (gadget-value pane) (gadget-min-value pane))
+	     #+NIL
              (clim-internals::schedule-timer-event pane token 0.1)
              (decf (gadget-value pane))
              (dispatch-repaint pane (sheet-region pane)))))))))
@@ -256,12 +315,14 @@
            ; move up or down one notch
            (cond
              ((< y (bounding-rectangle-min-y thumb))
+	      #+NIL
               (clim-internals::schedule-timer-event pane 'down-notch 0.1)
               ; move toward the min
               (when (> (gadget-value pane) (gadget-min-value pane))
                 (decf (gadget-value pane))
                 (dispatch-repaint pane (sheet-region pane))))
              ((> y (bounding-rectangle-max-y thumb))
+	      #+NIL
               (clim-internals::schedule-timer-event pane 'up-notch 0.1)
               ; move toward the max
               (when (< (gadget-value pane) (gadget-max-value pane))
@@ -334,13 +395,14 @@
                      (x1     (+ x1 2))
                      (x2     (- x2 3)))
                 (draw-line* pane x1 y1 x2 y1       :ink +gray58+)
-                (draw-line* pane x1 y2 (+ x2 1) y2 :ink +white+)
-                (draw-line* pane x2 y1 x2 (+ y2 1) :ink +white+)))))))))
+                (draw-line* pane x1 y2 x2 y2 :ink +white+)
+                (draw-line* pane x2 y1 x2 y2 :ink +white+)))))))))
 
 ; Scrollbar
 
 ; We derive from the slider, since the slider is the same, only
 ; less so.
+;;; XXX Probably should derive from scroll-bar too.
 
 (defconstant +pixie-scroll-bar-pane-thumb-size+  5000.0)
 (defconstant +pixie-scroll-bar-thumb-half-height+ 17)
@@ -386,6 +448,8 @@
      :max-value 1
      :orientation :vertical))
 
+(define-pixie-gadget scroll-bar pixie-scroll-bar-pane)
+
 (defmethod compose-space ((pane pixie-scroll-bar-pane) &key width height)
   (declare (ignore width height))
   (if (eq (gadget-orientation pane) :vertical)
@@ -476,6 +540,14 @@
                   (yb (translate-range-value (+ v ts) minv (+ maxv ts) y1 y2)))
               (make-rectangle* x1 (- ya 1) x2 (+ yb 1)))))))))
 
+(defmethod* (setf scroll-bar-values)
+  (min-value max-value thumb-size value (scroll-bar pixie-scroll-bar-pane))
+  (setf (slot-value scroll-bar 'min-value) min-value
+	(slot-value scroll-bar 'max-value) max-value
+	(slot-value scroll-bar 'thumb-size) thumb-size
+	(slot-value scroll-bar 'value) value)
+  (dispatch-repaint scroll-bar (sheet-region scroll-bar)))
+
 (defmethod handle-event ((pane pixie-scroll-bar-pane) (event pointer-button-release-event))
   (with-slots (armed dragging repeating was-repeating) pane
     (setf was-repeating repeating)
@@ -496,6 +568,7 @@
   (let ((token (clim-internals::event-token event)))
     (with-slots (was-repeating repeating) pane
       (unless (eql was-repeating repeating)
+	#+NIL
         (clim-internals::schedule-timer-event pane token 0.1)
         (case token
           ((up-line)
@@ -522,12 +595,14 @@
                  armed    t
                  drag-delta (- y (bounding-rectangle-min-y thumb))))
           ((region-contains-position-p (gadget-up-region pane) x y)
+	   #+NIL
            (clim-internals::schedule-timer-event pane 'up-line 0.1)
            ; Up Arrow
            (scroll-up-line-callback pane (gadget-client pane) (gadget-id pane))
            (setf (slot-value pane 'armed) :up)
            (dispatch-repaint pane +everywhere+))
           ((region-contains-position-p (gadget-down-region pane) x y)
+	   #+NIL
            (clim-internals::schedule-timer-event pane 'down-line 0.1)
            ; Down Arrow
            (scroll-down-line-callback pane (gadget-client pane) (gadget-id pane))
@@ -537,9 +612,11 @@
            ; Bed
            (cond
              ((< y (bounding-rectangle-min-y thumb))
+	      #+NIL
               (clim-internals::schedule-timer-event pane 'up-page 0.1)
               (scroll-up-page-callback pane (gadget-client pane) (gadget-id pane)))
              (t
+	      #+NIL
               (clim-internals::schedule-timer-event pane 'down-page 0.1)
               (scroll-down-page-callback pane (gadget-client pane) (gadget-id pane)))))
           (t
@@ -593,26 +670,17 @@
                                    :border-width 1)
             ;; draw up arrow
             (with-bounding-rectangle* (x1 y1 x2 y2) gadget-up-region
-               (if (eq (slot-value pane 'armed) :up)
+	       (if (eq (slot-value pane 'armed) :up)
                    (draw-down-box pane x1 y1 x2 y2 +gray83+)
                    (draw-up-box   pane x1 y1 x2 y2 +gray83+))
                 ;; draw decoration in the region
-                ;; for this, we want to have an odd width and height
-                (flet ((oddify (v) (let ((v (floor v))) (if (oddp v) v (+ v 1)))))
-                  (let* ((width  (oddify (- x2 x1)))
-                         (height (oddify (- y2 y1)))
-                         (arrow (list (make-point (floor (/ (+ x1 x2) 2))
-                                                  (floor (+ y1 (* height 5/13))))
-                                      (make-point (floor (+ x1 (* width 4/13)))
-                                                  (floor (- y2 (* height 6/13))))
-                                      (make-point (floor (+ x1 (* width 4/13)))
-                                                  (floor (- y2 (* height 5/13))))
-                                      (make-point (floor (- x2 (* width 4/13)))
-                                                  (floor (- y2 (* height 5/13))))
-                                      (make-point (floor (- x2 (* width 4/13)))
-                                                  (floor (- y2 (* height 6/13)))))))
-                    (draw-polygon pane arrow :filled t :ink +black+))))
-            ; old
+	       (multiple-value-bind (pattern fudge-x fudge-y)
+		   (if (eq (gadget-orientation pane) :vertical)
+		       (values +pixie-up-arrow+ -1 1)
+		       (values +pixie-left-arrow+ -1 1))
+		 (draw-pattern* pane pattern
+				(+ fudge-x (floor (- (+ x1 x2) (pattern-width  pattern)) 2))
+				(+ fudge-y (floor (- (+ y1 y2) (pattern-height pattern)) 2)))))
   
             ;; draw down arrow
             (with-bounding-rectangle* (x1 y1 x2 y2) gadget-down-region
@@ -620,20 +688,13 @@
                    (draw-down-box pane x1 y1 x2 y2 +gray83+)
                    (draw-up-box   pane x1 y1 x2 y2 +gray83+))
                 ;; draw decoration in the region
-                (flet ((oddify (v) (let ((v (floor v))) (if (oddp v) v (+ v 1)))))
-                  (let* ((width  (oddify (- x2 x1)))
-                         (height (oddify (- y2 y1)))
-                         (arrow (list (make-point (floor (/ (+ x1 x2) 2))
-                                                  (floor (- y2 (* height 5/13))))
-                                      (make-point (floor (+ x1 (* width 4/13)))
-                                                  (floor (+ y1 (* height 6/13))))
-                                      (make-point (floor (+ x1 (* width 4/13)))
-                                                  (floor (+ y1 (* height 5/13))))
-                                      (make-point (floor (- x2 (* width 4/13)))
-                                                  (floor (+ y1 (* height 5/13))))
-                                      (make-point (floor (- x2 (* width 4/13)))
-                                                  (floor (+ y1 (* height 6/13)))))))
-                    (draw-polygon pane arrow :filled t :ink +black+))))
+	       (multiple-value-bind (pattern fudge-x fudge-y)
+		   (if (eq (gadget-orientation pane) :vertical)
+		       (values +pixie-down-arrow+  -1 1)
+		       (values +pixie-right-arrow+ -1 2))
+		 (draw-pattern* pane pattern
+				(+ fudge-x (floor (- (+ x1 x2) (pattern-width  pattern)) 2))
+				(+ fudge-y (floor (- (+ y1 y2) (pattern-height pattern)) 2)))))
   
             ;; draw thumb
             (with-bounding-rectangle* (x1 y1 x2 y2) gadget-thumb-region
@@ -645,34 +706,43 @@
 
 (defclass pixie-menu-bar-pane (pixie-gadget menu-bar) ())
 
-; silly menu-bar isn't named pane, so this catches it
-(defclass pixie-menu-bar (pixie-menu-bar-pane) ())
+(define-pixie-gadget menu-bar pixie-menu-bar-pane :enabled t)
 
 (defmethod handle-repaint ((pane pixie-menu-bar-pane) region)
   (declare (ignore region))
   (with-special-choices (pane)
     (let* ((region (sheet-region pane))
            (frame (polygon-points (bounding-rectangle region))))
-      (draw-polygon pane frame :ink +Blue+ :filled t)
+      #+NIL      (draw-polygon pane frame :ink +Blue+ :filled t)
       (draw-bordered-polygon pane frame :style :outset :border-width 1))))
 
-(defmethod compose-space ((gadget pixie-menu-bar-pane) &key width height)
-  (declare (ignore width height))
-  (multiple-value-bind (width min-width max-width height min-height max-height)
-      (space-requirement-components (call-next-method))
-    (make-space-requirement
-      :width width
-      :min-width min-width
-      :max-width max-width
-      :height min-height
-      :min-height min-height
-      :max-height min-height)))
+(define-pixie-gadget menu-button pixie-menu-button-pane)
 
-(defclass pixie-menu-button-pane (pixie-gadget menu-button-pane) ()
+(defclass pixie-menu-button-pane (pixie-gadget menu-button-pane)
+  ((left-margin :reader left-margin)
+   (right-margin :reader right-margin))                
   (:default-initargs
     :align-x :left
     :align-y :center))
 
+(defparameter *pixie-menu-button-left-margin*  26)
+(defparameter *pixie-menu-button-right-margin* 26)
+(defparameter *pixie-menubar-item-left-margin* 8)
+(defparameter *pixie-menubar-item-right-margin* 8)
+(defparameter *pixie-menubar-item-spacing* 0)
+
+(defmethod initialize-instance :after ((pane pixie-menu-button-pane)
+                                       &rest args &key vertical &allow-other-keys)
+  (declare (ignore args))
+  (with-slots (left-margin right-margin) pane
+    (setf (values left-margin right-margin)
+          (if (or (typep (slot-value pane 'client) 'menu-bar)
+                  (not vertical))
+              (values *pixie-menubar-item-left-margin* *pixie-menubar-item-right-margin*)
+              (values *pixie-menu-button-left-margin* *pixie-menu-button-right-margin*)))))
+
+;; What even uses this? All the subclasses have their own handle-repaint methods!
+#+NIL
 (defmethod handle-repaint ((pane pixie-menu-button-pane) region)
   (declare (ignore region))
   (with-special-choices (pane)
@@ -690,25 +760,23 @@
                            :border-width 1)))
           (t
            (draw-polygon pane frame :filled t :ink (effective-gadget-foreground pane))))
-        (draw-label* pane (+ x1 5) y1 x2 y2 :ink (pane-inking-color pane))))))
+        (draw-label* pane (+ x1 (left-margin pane)) y1 (- x2 (right-margin pane)) y2 :ink +red+ #+NIL (pane-inking-color pane))))))
 
 (defmethod compose-space ((gadget pixie-menu-button-pane) &key width height)
   (declare (ignore width height))
-  (space-requirement+* (space-requirement+* (compose-label-space gadget :wider 5 :higher 10)
-                                            :min-width  (* 2 (pane-x-spacing gadget))
-                                            :width      (* 2 (pane-x-spacing gadget))
-                                            :max-width  +fill+ 
-                                            :min-height (* 2 (pane-y-spacing gadget))
-                                            :height     (* 2 (pane-y-spacing gadget))
-                                            :max-height (* 2 (pane-y-spacing gadget)))
-                       :min-width  (+ 17 (* 2 *3d-border-thickness*))
-                       :width      (+ 17 (* 2 *3d-border-thickness*))
+  (space-requirement+* (compose-label-space gadget
+					    :wider (+ (left-margin gadget)
+                                                      (right-margin gadget))
+					    :higher (+ 6 (* 2 *3d-border-thickness*)))
+                       :min-width  0
+                       :width      0
                        :max-width  +fill+
-                       :min-height (* 2 *3d-border-thickness*)
-                       :height     (* 2 *3d-border-thickness*)
-                       :max-height (* 2 *3d-border-thickness*)))
+                       :min-height 0
+                       :height     0
+                       :max-height 0))
 
 (defclass pixie-menu-button-leaf-pane (pixie-menu-button-pane menu-button-leaf-pane) ())
+(define-pixie-gadget menu-button-leaf-pane pixie-menu-button-leaf-pane)
 
 (defmethod handle-repaint ((pane pixie-menu-button-leaf-pane) region)
   (declare (ignore region))
@@ -725,25 +793,26 @@
                                :filled t)
               (when armed
                 (draw-edges-lines* pane +white+ 0 0 +black+ (1- w) (1- h)))
-              (draw-label* pane (+ x1 8) y1 (- x2 17) y2 :ink +black+))))))))
+	      (let ((x1 (+ x1 (left-margin pane)))
+		    (x2 (- x2 (right-margin pane))))
+		(if (gadget-active-p pane)
+		    (draw-label* pane x1 y1 x2 y2 :ink +black+)
+		    (draw-engraved-label* pane x1 y1 x2 y2))))))))))
 
 (defclass pixie-menu-button-submenu-pane (pixie-menu-button-pane menu-button-submenu-pane) ())
 
+(define-pixie-gadget menu-button-submenu-pane pixie-menu-button-submenu-pane)
+(define-pixie-gadget menu-button-vertical-submenu-pane pixie-menu-button-submenu-pane)
+
+
 (defmethod compose-space ((gadget pixie-menu-button-submenu-pane) &key width height)
   (declare (ignore width height))
-  (space-requirement+* (space-requirement+* (compose-label-space gadget :wider 5 :higher 10)
-                                            :min-width  (* 2 (pane-x-spacing gadget))
-                                            :width      (* 2 (pane-x-spacing gadget))
-                                            :max-width  +fill+ 
-                                            :min-height (* 2 (pane-y-spacing gadget))
-                                            :height     (* 2 (pane-y-spacing gadget))
-                                            :max-height (* 2 (pane-y-spacing gadget)))
-                       :min-width  (+ 17 (* 2 *3d-border-thickness*))
-                       :width      (+ 17 (* 2 *3d-border-thickness*))
-                       :max-width  +fill+
-                       :min-height (* 2 *3d-border-thickness*)
-                       :height     (* 2 *3d-border-thickness*)
-                       :max-height (* 2 *3d-border-thickness*)))
+  (if (typep (slot-value gadget 'client) 'menu-bar) ; XXX
+      (compose-label-space gadget
+			   :wider (+ (left-margin gadget)
+				     (right-margin gadget))
+			   :higher 10)
+      (call-next-method)))
 
 (defmethod handle-repaint ((pane pixie-menu-button-submenu-pane) region)
   (declare (ignore region))
@@ -759,28 +828,18 @@
                                :filled t)
               (when submenu-frame
                 (draw-edges-lines* pane +white+ 0 0 +black+ (1- w) (1- h)))
+	      
+	      (if (typep client 'menu-button)
+		  (let ((pattern +pixie-right-arrow+))
+		    (draw-label* pane (+ x1 (left-margin pane)) y1
+				 (- x2 (right-margin pane)) y2 :ink +black+)  
+		    (draw-pattern* pane pattern (- x2 10) (+ y1 (floor (- h (pattern-height pattern)) 2))))
+		  (draw-label* pane
+			       (+ x1 (left-margin pane))  y1
+			       (- x2 (right-margin pane)) y2
+			       :ink +black+)))))))))
+
 
-              (draw-label* pane (+ x1 8) y1 (- x2 17) y2 :ink +black+)
-  
-              (when (typep client 'menu-button-pane)
-                (let* ((x1 (- x2 17))
-                       (ym (/ (+ y1 y2) 2))
-                       (y1 (- ym  8))
-                       (y2 (+ ym  8)))
-                  (flet ((oddify (v) (let ((v (floor v))) (if (oddp v) v (+ v 1)))))
-                    (let* ((width  (oddify (- x2 x1)))
-                           (height (oddify (- y2 y1)))
-                           (arrow (list (make-point (floor (- x2 (* width 5/13)))
-                                                    (floor (/ (+ y1 y2) 2)))
-                                        (make-point (floor (+ x1 (* width 6/13)))
-                                                    (floor (+ y1 (* height 4/13))))
-                                        (make-point (floor (+ x1 (* width 5/13)))
-                                                    (floor (+ y1 (* height 4/13))))
-                                        (make-point (floor (+ x1 (* width 5/13)))
-                                                    (floor (- y2 (* height 4/13))))
-                                        (make-point (floor (+ x1 (* width 6/13)))
-                                                    (floor (- y2 (* height 4/13)))))))
-                      (draw-polygon pane arrow :filled t :ink +black+))))))))))))
 
 ; Image pane
 
@@ -789,6 +848,7 @@
 
 ; This is just test/proof-of-concept code :]
 
+#+NIL
 (defclass pixie-image-pane (pixie-gadget basic-gadget) (
   (image-pathname :initarg :pathname)
   (image-mask-pathname :initarg :mask-pathname :initform nil)
@@ -803,6 +863,7 @@
   (image-stencil  :initform nil)))
 
 ; TODO: allow pixmaps to be realized from unrealized media
+#+NIL
 (defmethod initialize-instance :after ((pane pixie-image-pane) &rest args)
   (declare (ignore args))
   (with-slots (image-pathname image-image image-width image-height) pane
@@ -817,6 +878,7 @@
       (let* ((data (image:read-image-file image-mask-pathname)))
         (setf image-stencil (make-stencil data))))))
 
+#+NIL
 (defmethod handle-repaint ((pane pixie-image-pane) region)
   (declare (ignore region))
   (with-slots (image-pixmap image-width image-height) pane
@@ -836,6 +898,7 @@
                             :clipping-region (make-rectangle* 0 0 image-width image-height))))))
     (copy-from-pixmap image-pixmap 0 0 image-width image-height pane 0 0)))
 
+#+NIL
 (defmethod compose-space ((pane pixie-image-pane) &key width height)
   (declare (ignore width height))
   (with-slots (image-width image-height) pane
@@ -849,6 +912,8 @@
 
 (defclass pixie-toggle-button-pane (pixie-gadget toggle-button-pane) ())
 
+(define-pixie-gadget toggle-button pixie-toggle-button-pane)
+
 (defmethod draw-toggle-button-indicator ((pane pixie-toggle-button-pane) (type (eql :one-of)) value x1 y1 x2 y2)
   (multiple-value-bind (cx cy) (values (/ (+ x1 x2) 2) (/ (+ y1 y2) 2))
     (let ((radius (/ (- y2 y1) 2)))
@@ -915,8 +980,11 @@
   (dragging
     :initform nil)))
 
+(define-pixie-gadget push-button pixie-push-button-pane)
+
 (defmethod compose-space ((gadget pixie-push-button-pane) &key width height)
   (declare (ignore width height))
+  ;; Why does a button have spacing options, anyway?
   (space-requirement+* (space-requirement+* (compose-label-space gadget)
                                             :min-width (* 2 (pane-x-spacing gadget))
                                             :width (* 2 (pane-x-spacing gadget))
@@ -924,12 +992,12 @@
                                             :min-height (* 2 (pane-y-spacing gadget))
                                             :height (* 2 (pane-y-spacing gadget))
                                             :max-height (* 2 (pane-y-spacing gadget)))
-                       :min-width  (* 2 *3d-border-thickness*)
-                       :width      (* 2 *3d-border-thickness*)
-                       :max-width  (* 2 *3d-border-thickness*)
-                       :min-height (* 2 *3d-border-thickness*)
-                       :height     (* 2 *3d-border-thickness*)
-                       :max-height (* 2 *3d-border-thickness*)))
+                       :min-width  (* 8 *3d-border-thickness*)
+                       :width      (* 8 *3d-border-thickness*)
+                       :max-width  (* 8 *3d-border-thickness*)
+                       :min-height (* 4 *3d-border-thickness*)
+                       :height     (* 4 *3d-border-thickness*)
+                       :max-height (* 4 *3d-border-thickness*)))
 
 ; factor out the dragging code into a mixin
 (defmethod handle-event ((pane pixie-push-button-pane) (event pointer-enter-event))
@@ -974,8 +1042,8 @@
             (y1 (+ y1 1))
             (x2 (- x2 1))
             (y2 (- y2 1)))
-        (let ((x2 (- x2 1))
-              (y2 (- y2 1)))
+        (let ((x2 (- x2 1))		; Removing this magic weirdness slightly uglifies the 
+              (y2 (- y2 1)))		; scroll bar. Not sure why, but FIXME.
           (cond
            ((or (not pressedp)
                 (eq dragging :outside))
@@ -983,10 +1051,31 @@
            (pressedp
             (draw-down-box pane x1 y1 x2 y2 (effective-gadget-foreground pane)))))))))
 
+(defclass pixie-submenu-border-pane (submenu-border)
+  ()
+  (:default-initargs :border-width 2))
+
+(define-pixie-gadget submenu-border pixie-submenu-border-pane)
+
+(defmethod handle-repaint ((pane pixie-submenu-border-pane) region)
+  (declare (ignore region))
+  (with-slots (border-width) pane
+    (with-bounding-rectangle* (x1 y1 x2 y2) (sheet-region pane)
+      (draw-rectangle* pane x1 y1 x2 y2 :filled nil :ink +black+)
+      ;; Why, having incremented the coordinates, and despite setting
+      ;; the border-width to 2, do I now get a single pixel border ?
+      ;; It's fine, that's the result I want, but an explanation is in order.
+      (draw-bordered-rectangle* pane (1+ x1) (1+ y1) (1- x2) (1- y2)
+				:style :outset
+				:border-width border-width))))
+
 ; Text Area
 
 (defclass pixie-text-field-pane (text-field-pane) ())
 
+;; Why does pixie need its own text area subclass? Leave it disabled for now.
+(define-pixie-gadget text-field-pane pixie-text-field-pane :enabled nil)
+
 (defmethod initialize-instance :after ((pane pixie-text-field-pane) &rest rest)
   (unless (getf rest :normal)
     (setf (slot-value pane 'current-color) +white+
@@ -1010,23 +1099,6 @@
         (display-gadget-background pane (gadget-current-color pane) 0 0 (- x2 x1) (- y2 y1))
         (goatee::redisplay-all (area pane))))))
 
-
-;;; Unilaterally declare a "focus follows mouse" policy.  I don't like this
-;;; much; the whole issue of keyboard focus needs a lot more thought,
-;;; especially when multiple application frames per port become possible.
-
-(defmethod armed-callback :after ((gadget pixie-text-field-pane) client id)
-  (declare (ignore client id))
-  (let ((port (port gadget)))
-    (setf (previous-focus gadget) (port-keyboard-input-focus port))
-    (setf (port-keyboard-input-focus port) gadget)))
-
-(defmethod disarmed-callback :after ((gadget pixie-text-field-pane) client id)
-  (declare (ignore client id))
-  (let ((port (port gadget)))
-    (setf (port-keyboard-input-focus port) (previous-focus gadget))
-    (setf (previous-focus gadget) nil)))
-
 (defmethod handle-event ((gadget pixie-text-field-pane) (event key-press-event))
   (let ((gesture (convert-to-gesture event))
 	(*activation-gestures* *standard-activation-gestures*))
@@ -1069,3 +1141,130 @@
 
 (defmethod allocate-space ((pane pixie-text-field-pane) w h)
   (resize-sheet pane w h))
+
+;;;; Pixie tab-layout. Reuses implementation of the generic tab-layout-pane.
+
+(define-pixie-gadget clim-tab-layout:tab-layout pixie-tab-layout-pane)
+(define-pixie-gadget clim-tab-layout::tab-bar-pane pixie-tab-bar-pane)
+
+(defclass pixie-tab-bar-view (gadget-view)
+  ((selected :initform nil
+	     :initarg :selected
+	     :reader pixie-tab-view-selected-p)))
+
+(defparameter +pixie-tab-bar-view+
+  (make-instance 'pixie-tab-bar-view :selected nil))
+
+(defparameter +pixie-selected-tab-bar-view+
+  (make-instance 'pixie-tab-bar-view :selected t))
+
+
+
+(defclass pixie-tab-layout-pane (clim-tab-layout:tab-layout-pane)
+  ()
+  (:default-initargs
+    :header-display-function 'pixie-display-tab-header))
+
+(defclass pixie-tab-bar-pane (application-pane pixie-gadget)
+  ()
+  (:default-initargs
+    :default-view +pixie-tab-bar-view+
+    :background +gray83+
+    :text-style (make-text-style :sans-serif :roman :small)))
+
+(defmethod compose-space ((pane pixie-tab-bar-pane) &key width height)
+  (declare (ignore width height))
+  (let ((h (+ 6				; padding on the top
+	      6				; padding on the bottom
+	      (text-style-ascent (pane-text-style pane) pane)
+	      (text-style-descent (pane-text-style pane) pane))))
+    (make-space-requirement :min-height h :height h :max-height h)))
+
+(defun draw-pixie-tab-bar-bottom (pane)
+  (let ((y0 (bounding-rectangle-min-y (sheet-region pane)))
+	(y1 (bounding-rectangle-max-y (sheet-region pane))))
+    (draw-line* pane 0 (- y1 6) +fill+ (- y1 6) :ink *3d-light-color*)
+    (draw-line* pane 0 (- y1 1) +fill+ (- y1 1) :ink *3d-dark-color*)
+    #+NIL (draw-line* pane 0 (1- y1) x1 (1- y1)   :ink +gray30+)))
+
+(defmethod draw-output-border-over
+    ((shape (eql 'pixie-tab-bar-border)) stream record &key &allow-other-keys)
+  (declare (ignore shape stream record)))
+
+(defmethod draw-output-border-under
+    ((shape (eql 'pixie-tab-bar-border)) stream record
+     &key background enabled &allow-other-keys)
+  (with-border-edges (stream record)
+    (declare (ignore bottom))
+    (with-bounding-rectangle* (x0 y0 x1 y1) (sheet-region stream)
+      (declare (ignore x0 x1 y0))
+      (let ((bottom (- y1 7))
+	    (left   (- left 4 (if enabled 2 0)))
+	    (right  (+ right 4 (if enabled 2 0)))
+	    (top    (- top 2 #+NIL (if enabled 2 0))))
+	(draw-rectangle* stream left top right (+ bottom (if enabled 2 1))
+			 :filled t :ink background)
+	(draw-line* stream (1+ left) (1- top) (- right 1) (1- top) :ink +white+)
+	(draw-point* stream left top :ink +white+)
+	(draw-line* stream (1- left) bottom (1- left) (1+ top) :ink +white+)
+	(draw-line* stream right bottom right top :ink +gray66+)
+	(draw-point* stream right top :ink +gray40+)
+	(draw-line* stream (1+ right) bottom (1+ right) (1+ top) :ink +gray40+)))))
+
+(define-default-highlighting-method 'pixie-tab-bar-border)
+
+(define-presentation-method present
+    (tab-page (type clim-tab-layout:tab-page) stream (view pixie-tab-bar-view) &key)
+  (stream-increment-cursor-position stream 5 0)
+  (surrounding-output-with-border (stream :shape 'pixie-tab-bar-border
+					  :enabled (pixie-tab-view-selected-p view)
+					  :highlight-background +gray94+
+					  :background +gray83+
+					  :move-cursor nil)
+    (apply #'invoke-with-drawing-options stream
+	   (lambda (rest)
+	     (declare (ignore rest))
+	     (write-string (clim-tab-layout:tab-page-title tab-page) stream))
+	   (clim-tab-layout:tab-page-drawing-options tab-page)))
+  (stream-increment-cursor-position stream 6 0))
+
+(defun pixie-display-tab-header (tab-layout pane)
+  (draw-pixie-tab-bar-bottom pane)
+  (setf (stream-cursor-position pane)
+	(values 3 (- (bounding-rectangle-height (sheet-region pane))
+		     7
+		     (text-style-descent (pane-text-style pane) pane)
+		     (text-style-ascent (pane-text-style pane) pane))))  
+  (let ((enabled-page-drawers nil))
+    (mapc (lambda (page)
+	    ;; This gets a little silly, but the tabs are laid out simply by
+	    ;; letting the cursor move from left to right. In order to make
+	    ;; the selected tab overlap, we can't draw it until after the other
+	    ;; tabs. We then draw it slightly larger in each direcetion. But the
+	    ;; cursor has to have moved as though it were smaller (so that it
+	    ;; overlaps its neighbors), so draw it initially, note its position,
+	    ;; and redraw a larger version once everything is done.	    
+	    (let ((enabled (sheet-enabled-p (clim-tab-layout:tab-page-pane page))))
+	      (when enabled
+		(multiple-value-bind (x y) (stream-cursor-position pane)
+		  (push (lambda ()
+			  (setf (stream-cursor-position pane)
+				(values x (- y 2)))
+			  (with-output-as-presentation
+			      (pane (clim-tab-layout:tab-page-pane page)
+				    (clim-tab-layout:tab-page-presentation-type page))
+			    (present page 'clim-tab-layout:tab-page :stream pane
+				     :view +pixie-selected-tab-bar-view+)))
+			enabled-page-drawers)))
+	      (let ((record 
+		     (with-output-as-presentation
+			 (pane (clim-tab-layout:tab-page-pane page)
+			       (clim-tab-layout:tab-page-presentation-type page))
+		       (present page 'clim-tab-layout:tab-page :stream pane))))
+		;; Because piling the presentations on top of each other confuses
+		;; CLIM as to which should be highlighted, erase the smaller one.
+		;; The cursor has already been moved, so we don't need it.
+		(when enabled
+		  (delete-output-record record (output-record-parent record))))))
+	  (clim-tab-layout:tab-layout-pages tab-layout))
+    (mapcar #'funcall enabled-page-drawers)))
