@@ -93,9 +93,15 @@
     (t nil global-command-table
      :gesture :select
      :tester ((presentation context-type)
-	      (presentation-subtypep (presentation-type presentation)
-				     context-type))
-     :tester-definitive t
+              ;; see the comments around DEFUN PRESENTATION-SUBTYPEP
+              ;; for some of the logic behind this.  Only when
+              ;; PRESENTATION-SUBTYPEP is unsure do we test the object
+              ;; itself for PRESENTATION-TYPEP.
+              (multiple-value-bind (yp sp)
+                  (presentation-subtypep (presentation-type presentation)
+                                         context-type)
+                (or yp (not sp))))
+     :tester-definitive nil
      :menu nil
      :documentation ((object presentation context-type frame event window x y stream)
                      (let* ((type (presentation-type presentation))
@@ -116,6 +122,10 @@
                                     :stream stream
                                     :sensitive nil)))))
   (object presentation)
+  ;; returning (PRESENTATION-TYPE PRESENTATION) as the ptype is
+  ;; formally undefined, as this means that the translator returns a
+  ;; presentation type which is not PRESENTATION-SUBTYPEP the
+  ;; translator's TO-TYPE.
   (values object (presentation-type presentation)))
 
 (define-presentation-action presentation-menu
@@ -133,7 +143,9 @@
   (presentation frame window x y)
   (call-presentation-menu presentation *input-context*
                           frame window x y
-                          :for-menu t))
+                          :for-menu t
+                          :label (format nil "Operation on ~A"
+                                         (presentation-type presentation))))
 
 ;;; Action for possibilities menu of complete-input
 ;;;
@@ -327,6 +339,67 @@
 	       (unread-char c stream))
 	     (return (values object ptype))))))
 
+(define-presentation-method accept ((type expression)
+                                    (stream input-editing-stream)
+                                    (view textual-view)
+				    &key)
+			    ;; This method is specialized to
+			    ;; input-editing-streams and has thus been
+			    ;; made slightly more tolerant of input
+			    ;; errors. It is slightly hacky, but seems
+			    ;; to work fine.
+			    (let* ((object nil)
+	 (ptype nil))
+    (if (and #-openmcl nil subform-read)
+	(multiple-value-bind (val valid)
+	    (funcall *sys-%read-list-expression* stream *dot-ok* *termch*)
+	  (if valid
+	      (setq object val)
+	      (return-from accept (values nil 'list-terminator))))
+	;; We don't want activation gestures like :return causing an
+	;; eof while reading a form. Also, we don't want spaces within
+	;; forms or strings causing a premature return either!
+	(with-delimiter-gestures (nil :override t)
+	  (with-activation-gestures (nil :override t)
+            (setq object
+                  ;; We loop in our accept of user input, if a reader
+                  ;; error is signalled, we merely ignore it and ask
+                  ;; for more input. This is so a single malplaced #\(
+                  ;; or #\, won't throw up a debugger with a
+                  ;; READER-ERROR and remove whatever the user wrote
+                  ;; to the stream.
+                  (loop for potential-object =
+                       (handler-case (funcall
+                                      (if preserve-whitespace
+                                          *sys-read-preserving-whitespace*
+                                          *sys-read*)
+                                      stream
+                                      *eof-error-p*
+                                      *eof-value*
+                                      *recursivep*)
+                         #+sbcl(sb-kernel:reader-package-error (e)
+                                 (progn
+                                   ;; Resignal the error.
+                                   (error e)))
+                         ((and reader-error) (e)
+                           (declare (ignore e))
+                           nil))
+                       unless (null potential-object)
+                       return potential-object)))))
+    (setq ptype (presentation-type-of object))
+    (unless (presentation-subtypep ptype 'expression)
+      (setq ptype 'expression))
+    (if (or subform-read auto-activate)
+	(values object ptype)
+	(loop
+	   for c = (read-char stream)
+	   until (or (activation-gesture-p c) (delimiter-gesture-p c))
+	   finally
+	     (when (delimiter-gesture-p c)
+	       (unread-char c stream))
+	     (return (values object ptype))))))
+
+
 (with-system-redefinition-allowed
 (defun read (&optional (stream *standard-input*)
 	     (eof-error-p t)
