@@ -45,7 +45,7 @@
 
 ;;; According to the specification, command menu items are stored as
 ;;; lists.  This way seems better, and I hope nothing will break.
-(defclass menu-item (command-item)
+(defclass %menu-item (command-item)
   ((menu-name :reader command-menu-item-name :initarg :menu-name)
    (type :initarg :type :reader command-menu-item-type)
    (value :initarg :value :reader command-menu-item-value)
@@ -53,7 +53,7 @@
    (text-style :initarg :text-style :initform nil)
    (keystroke :initarg :keystroke)))
 
-(defmethod print-object ((item menu-item) stream)
+(defmethod print-object ((item %menu-item) stream)
   (print-unreadable-object (item stream :identity t :type t)
     (when (slot-boundp item 'menu-name)
       (format stream "~S" (command-menu-item-name item)))
@@ -95,8 +95,9 @@
 
 (defparameter *command-tables* (make-hash-table :test #'eq))
 
-(define-condition command-table-error (error)
-  ())
+(define-condition command-table-error (simple-error)
+  ()
+  (:default-initargs :format-control "" :format-arguments nil))
 
 (define-condition command-table-not-found (command-table-error)
   ())
@@ -195,7 +196,7 @@
 	(when errorp
 	  (error 'command-not-present))
 	(progn 
-	  (when (typep item 'menu-item)
+	  (when (typep item '%menu-item)
 	    (remove-menu-item-from-command-table table
 						 (command-menu-item-name item)
                                                  :errorp nil))
@@ -351,7 +352,7 @@
   (when (and (consp text-style)
 	   (eq (first text-style) 'make-text-style))
     (setq text-style (apply #'make-text-style (rest text-style))))
-  (apply #'make-instance 'menu-item
+  (apply #'make-instance '%menu-item
 	 :menu-name name :type type :value value
 	 `(,@(and documentationp `(:documentation ,documentation))
 	   ,@(and keystrokep `(:keystroke ,keystroke))
@@ -402,7 +403,8 @@
 		       menu-name
 		       (and (slot-boundp item 'keystroke) keystroke)
 		       item)))
-	(slot-value (find-command-table command-table) 'menu)))
+	(slot-value (find-command-table command-table) 'menu))
+  (values))
 
 ;; At this point we should still see the gesture name as supplied by the
 ;; programmer in 'gesture'
@@ -430,7 +432,7 @@
   (let ((command-table (find-command-table command-table)))
     (%add-keystroke-item command-table
                          gesture
-			 (make-instance 'menu-item
+			 (make-instance '%menu-item
 					:type type :value value
 					:keystroke gesture
 					:documentation documentation)
@@ -480,8 +482,7 @@
 	nil)))
 
 (defun lookup-keystroke-item (gesture command-table
-			      &key (test #'event-matches-gesture-name-p)
-			      (errorp t))
+			      &key (test #'event-matches-gesture-name-p))
   (let ((command-table (find-command-table command-table)))
     (multiple-value-bind (item table)
 	(find-keystroke-item gesture command-table :test test :errorp nil)
@@ -494,15 +495,11 @@
 	     (multiple-value-bind (sub-item sub-command-table)
 		 (lookup-keystroke-item gesture
 					(command-menu-item-value item)
-					:test test
-					:errorp nil)
+					:test test)
 	       (when sub-command-table
 		 (return-from lookup-keystroke-item
 		   (values sub-item sub-command-table))))))
-       command-table))
-    (if errorp
-	(error 'command-not-present)
-	nil)))
+       command-table))))
 
 (defun partial-command-from-name (command-name)
   (let ((parser (gethash command-name *command-parser-table*)))
@@ -671,7 +668,7 @@
 		 keyword-args)
        (setq ,key-possibilities (nreverse ,key-possibilities))
        (when ,key-possibilities
-	 (input-editor-format ,stream "~%(keywords)")
+	 (input-editor-format ,stream "(keywords) ")
 	 (let ((,member-ptype `(token-or-type ,,key-possibilities empty)))
 	   (loop
 	     (let* ((,key-result (prog1 (accept ,member-ptype
@@ -1089,11 +1086,15 @@
 				    &key)
   (flet ((generator (string suggester)
 	   (declare (ignore string))
-	   (map-over-command-table-names
-	    (lambda (cline-name command-name)
-	      (when (command-enabled command-name *application-frame*)
-		(funcall suggester cline-name command-name)))
-	    command-table)))
+           (let ((possibilities nil))
+             (map-over-command-table-names
+              (lambda (cline-name command-name)
+                (when (command-enabled command-name *application-frame*)
+                  (pushnew (cons cline-name command-name) possibilities
+                           :key #'car :test #'string=)))
+              command-table)
+             (loop for (cline-name . command-name) in possibilities
+                   do (funcall suggester cline-name command-name)))))
     ;; Bind the frame's command table so that the command-enabled
     ;; test passes with this command table.
     (letf (((frame-command-table *application-frame*)
@@ -1146,18 +1147,18 @@
 					  (presentation-type-of arg))
 		       stream))))))))
 
-;;; Assume that stream is a goatee-based input editing stream for the moment...
-;;;
+;;; In order for this to work, the input-editing-stream must implement
+;;; a method for the nonstandard function
+;;; `input-editing-stream-bounding-rectangle'.
 (defun command-line-read-remaining-arguments-for-partial-command
     (command-table stream partial-command start-position)
   (declare (ignore start-position))
   (let ((partial-parser (partial-parser (gethash (command-name partial-command)
 						 *command-parser-table*))))
     (if (encapsulating-stream-p stream)
-	(let ((interactor (encapsulating-stream-stream stream))
-	      (editor-record (goatee::area stream)))
+	(let ((interactor (encapsulating-stream-stream stream)))
 	  (multiple-value-bind (x1 y1 x2 y2)
-	      (bounding-rectangle* editor-record)
+	      (input-editing-stream-bounding-rectangle stream)
 	    (declare (ignore y1 x2))
 	    ;; Start the dialog below the editor area
 	    (letf (((stream-cursor-position interactor) (values x1 y2)))
@@ -1170,7 +1171,6 @@
 	  (funcall partial-parser
 		 command-table stream  partial-command)))))
 
-
 (defparameter *command-parser* #'command-line-command-parser)
 
 (defparameter *command-unparser* #'command-line-command-unparser)
@@ -1364,7 +1364,8 @@
         (let ((command
                (lookup-keystroke-command-item (accelerator-gesture-event c)
                                               command-table)))
-          (if (partial-command-p command)
+          (if (and (listp command)
+                   (partial-command-p command))
               (funcall *partial-command-parser*
                        command-table stream command
                        (position *unsupplied-argument-marker* command))
@@ -1401,8 +1402,8 @@
 	  (if (member initial-char *command-dispatchers*)
 	      (progn
 		(read-gesture :stream stream)
-		(accept command-ptype :stream stream :view view :prompt nil))
-	      (accept 'form :stream stream :view view :prompt nil)))
+		(accept command-ptype :stream stream :view view :prompt nil :history 'command))
+	      (accept 'form :stream stream :view view :prompt nil :history 'command-or-form)))
       (t
        (funcall (cdar *input-context*) object type event options)))))
 
