@@ -82,10 +82,25 @@
 
 (defvar *application-frame* nil)
 
-;;; 3.2.1
+;;; 3.2.1.1 The Point Protocol
+
 (defgeneric point-x (point))
 (defgeneric point-y (point))
 
+;;; 3.2.2.1 The Polygon and Polyline Protocol
+
+(defgeneric polygon-points (polygon-or-polyline))
+(defgeneric map-over-polygon-coordinates (function polygon-or-polyline))
+(defgeneric map-over-polygon-segments (function polygon-or-polyline))
+(defgeneric polyline-closed (polyline))
+
+;;; 3.2.3.1 The Line Protocol
+
+(defgeneric line-start-point* (line))
+(defgeneric line-end-point* (line))
+(defgeneric line-start-point (line))
+(defgeneric line-end-point (line))
+
 ;;; 3.2.4.1
 
 (defgeneric rectangle-edges* (rectangle))
@@ -99,6 +114,42 @@
 (defgeneric rectangle-height (rectangle))
 (defgeneric rectangle-size (rectangle))
 
+;;; 3.2.5.1 The Ellipse and Elliptical Arc Protocol
+
+(defgeneric ellipse-center-point* (elliptical-object))
+(defgeneric ellipse-center-point (elliptical-object))
+(defgeneric ellipse-radii (elliptical-object))
+(defgeneric ellipse-start-angle (elliptical-object))
+(defgeneric ellipse-end-angle (elliptical-object))
+
+;;; 4.1.1 The Bounding Rectangle Protocol
+
+(defgeneric bounding-rectangle* (region))
+(defgeneric bounding-rectangle (region))
+
+;;; 4.1.2 Bounding Rectangle Convenience Functions
+
+(defgeneric bounding-rectangle-position (region))
+(defgeneric bounding-rectangle-min-x (region))
+(defgeneric bounding-rectangle-min-y (region))
+(defgeneric bounding-rectangle-max-x (region))
+(defgeneric bounding-rectangle-max-y (region))
+(defgeneric bounding-rectangle-width (region))
+(defgeneric bounding-rectangle-height (region))
+(defgeneric bounding-rectangle-size (region))
+
+;;; 5.3.1 Transformation Predicates
+
+(defgeneric transformation-equal (transformation1 transformation2))
+(defgeneric identity-transformation-p (transformation))
+(defgeneric invertible-transformation-p (transformation))
+(defgeneric translation-transformation-p (transformation))
+(defgeneric reflection-transformation-p (transformation))
+(defgeneric rigid-transformation-p (transformation))
+(defgeneric even-scaling-transformation-p (transformation))
+(defgeneric scaling-transformation-p (transformation))
+(defgeneric rectilinear-transformation-p (transformation))
+
 ;;; 5.3.2 Composition of Transformations
 
 (defgeneric compose-transformations (transformation1 transformation2))
@@ -121,6 +172,24 @@
 (defgeneric transform-rectangle* (transformation x1 y1 x2 y2))
 (defgeneric untransform-rectangle* (transformation x1 y1 x2 y2))
 
+;;; 7.2.1 Sheet Relationship Functions
+
+(defgeneric sheet-parent (sheet))
+(defgeneric sheet-children (sheet))
+(defgeneric sheet-adopt-child (sheet child))
+(defgeneric sheet-disown-child (sheet child &key errorp))
+(defgeneric sheet-siblings (sheet))
+(defgeneric sheet-enabled-children (sheet))
+(defgeneric sheet-ancestor-p (sheet putative-ancestor))
+(defgeneric raise-sheet (sheet))
+(defgeneric bury-sheet (sheet))
+(defgeneric reorder-sheets (sheet new-ordering))
+(defgeneric sheet-enabled-p (sheet))
+(defgeneric (setf sheet-enabled-p) (enabled-p sheet))
+(defgeneric sheet-viewable-p (sheet))
+(defgeneric sheet-occluding-sheets (sheet child))
+(defgeneric map-over-sheets (function sheet))
+
 ;;; 7.3.1 Sheet Geometry Functions [complete]
 
 (defgeneric sheet-transformation (sheet))
@@ -152,21 +221,57 @@
 ;;;; 8.1
 (defgeneric process-next-event (port &key wait-function timeout))
 
+(defgeneric port-keyboard-input-focus (port))
+(defgeneric (setf port-keyboard-input-focus) (focus port))
+
 ;;; 8.2 Standard Device Events
 
 (defgeneric event-timestamp (event))
+(defgeneric event-type (event))
+(defgeneric event-sheet (device-event))
+(defgeneric event-modifier-state (device-event))
+(defgeneric keyboard-event-key-name (keyboard-event))
+(defgeneric keyboard-event-character (keyboard-event))
+(defgeneric pointer-event-x (pointer-event))
+(defgeneric pointer-event-y (pointer-event))
+(defgeneric pointer-event-native-x (pointer-event))
+(defgeneric pointer-event-native-y (pointer-event))
+(defgeneric pointer-event-pointer (pointer-event))
+(defgeneric pointer-event-button (pointer-button-event))
+(defgeneric pointer-boundary-event-kind (pointer-boundary-event))
+(defgeneric window-event-region (window-event))
+(defgeneric window-event-native-region (window-event))
+(defgeneric window-event-mirrored-sheet (window-event))
+
+;;; 8.3.1 Output Properties
+
+(defgeneric medium-foreground (medium))
+(defgeneric (setf medium-foreground) (design medium))
+(defgeneric medium-background (medium))
+(defgeneric (setf medium-background) (design medium))
+(defgeneric medium-ink (medium))
+(defgeneric (setf medium-ink) (design medium))
+(defgeneric medium-transformation (medium))
+(defgeneric (setf medium-transformation) (transformation medium))
+(defgeneric medium-clipping-region (medium))
+(defgeneric (setf medium-clipping-region) (region medium))
+(defgeneric medium-line-style (medium))
+(defgeneric (setf medium-line-style) (line-style medium))
+(defgeneric medium-text-style (medium))
+(defgeneric (setf medium-text-style) (text-style medium))
+(defgeneric medium-default-text-style (medium))
+(defgeneric (setf medium-default-text-style) (text-style medium))
+(defgeneric medium-merged-text-style (medium))
+
 
 ;;;; 8.3.4 Associating a Medium with a Sheet
 
-;;;; 8.3.4.1 Grafting and Degrafting of Mediums
-
 ;; with-sheet-medium (medium sheet) &body body [Macro]
 ;; with-sheet-medium-bound (sheet medium) &body body [Macro]
 
 (defgeneric sheet-medium (sheet))
 (defgeneric medium-sheet (medium))
 (defgeneric medium-drawable (medium))
-(defgeneric port (medium))
 
 ;;; 8.3.4.1 Grafting and Degrafting of Mediums
 
@@ -187,6 +292,17 @@
 ;; (defgeneric portp (object))
 ;; find-port function
 
+;;; 9.2 Ports
+
+(defgeneric port (object))
+(defgeneric port-server-path (port))
+(defgeneric port-name (port))
+(defgeneric port-type (port))
+(defgeneric port-properties (port indicator))
+(defgeneric (setf port-properties) (property port indicator))
+(defgeneric restart-port (port))
+(defgeneric destroy-port (port))
+
 ;;; 9.3 Grafts
 
 (defgeneric sheet-grafted-p (sheet))
@@ -240,6 +356,13 @@
 			       align-x align-y
 			       toward-x toward-y transform-glyphs))
 
+;;; 10.1 Medium Components
+;;;
+;;; For reasons that are beyond me, many of the medium component
+;;; accessors are also specified in section 8.3.1.
+
+(defgeneric medium-current-text-style (medium))
+
 ;;;; 10.2
 (defgeneric invoke-with-drawing-options
     (medium continuation &rest drawing-options &key &allow-other-keys))
@@ -252,6 +375,31 @@
 
 (defgeneric invoke-with-first-quadrant-coordinates (medium continuation x y))
 
+;;; 11.1.1 Text Style Protocol and Text Style Suboptions
+
+(defgeneric text-style-components (text-style))
+(defgeneric text-style-family (text-style))
+(defgeneric text-style-face (text-style))
+(defgeneric text-style-size (text-style))
+(defgeneric merge-text-styles (style1 style2))
+(defgeneric text-style-ascent (text-style medium))
+(defgeneric text-style-descent (text-style medium))
+(defgeneric text-style-height (text-style medium))
+(defgeneric text-style-width (text-style medium))
+(defgeneric text-style-fixed-width-p (text-style medium))
+(defgeneric text-size (medium string &key text-style start end))
+
+;;; 11.2 Text Style Binding Forms
+
+(defgeneric invoke-with-text-style (medium continuation text-style))
+
+;;; 12.7.3 Other Medium-specific Output Functions
+
+(defgeneric medium-finish-output (medium))
+(defgeneric medium-force-output (medium))
+(defgeneric medium-clear-area (medium left top right bottom))
+(defgeneric medium-beep (medium))
+
 ;;;; 14.5
 (defgeneric draw-design
     (medium design
@@ -284,6 +432,7 @@
 (defgeneric (setf stream-text-cursor) (cursor stream))
 (defgeneric stream-cursor-position (stream))
 ;; (defgeneric (setf* stream-cursor-position) (x y stream)) unsure how to declare this, can somebody help? --GB
+(defgeneric stream-set-cusor-position (stream x y)) ; This is actually in 19.3.1 in CLIM 2.2
 (defgeneric stream-increment-cursor-position (stream dx dy))
 
 ;;; 15.4 Text Protocol [complete]
@@ -309,21 +458,65 @@
 (defgeneric (setf stream-end-of-page-action) (action stream))
 ;; with-end-of-page-action (stream action) &body body [Macro]
 
-;;; 16.4.3 Text Output Recording [complete]
+;;; 15.5 Attracting the User's Attention
 
+(defgeneric beep (&optional medium))
+
+;;; 15.6 Buffering of Output
+
+(defgeneric medium-buffering-output-p (medium))
+(defgeneric (setf medium-buffering-output-p) (buffer-p medium))
+
+;;; 16.3.3 Text Displayed Output Record
+
+(defgeneric add-character-output-to-text-record
+  (text-record character text-style width height baseline))
+
+(defgeneric add-string-output-to-text-record
+  (text-record string start end text-style width height baseline))
+
+(defgeneric text-displayed-output-record-string (text-record))
+
+;;; 16.4.1. The Output Recording Stream Protocol
+
+(defgeneric stream-recording-p (stream))
+(defgeneric (setf stream-recording-p) (recording-p stream))
+(defgeneric stream-drawing-p (stream))
+(defgeneric (setf stream-drawing-p) (drawing-p stream))
+(defgeneric stream-output-history (stream))
+(defgeneric stream-current-output-record (stream))
+(defgeneric (setf stream-current-output-record) (record stream))
+(defgeneric stream-add-output-record (stream record))
+(defgeneric stream-replay (stream &optional region))
+(defgeneric erase-output-record (record stream &optional errorp))
+
+;;; 16.4.3. Text Output Recording
 (defgeneric stream-text-output-record (stream text-style))
 (defgeneric stream-close-text-output-record (stream))
-(defgeneric stream-add-character-output (stream character text-style width height baseline))
-(defgeneric stream-add-string-output (stream string start end text-style width height baseline))
+(defgeneric stream-add-character-output
+  (stream character text-style width height baseline))
+(defgeneric stream-add-string-output
+  (stream string start end text-style width height baseline))
 
 ;;; 16.4.4 Output Recording Utilities [complete]
 
 ;; with-output-recording-options (stream &key record draw) &body body [Macro]
-(defgeneric invoke-with-output-recording-options (stream continuation record draw))
-;; with-new-output-record (stream &optional record-type record &rest initargs) &body body [MAcro]
-(defgeneric invoke-with-new-output-record (stream continuation record-type &rest initargs &key parent))
-;; with-output-to-output-record (stream &optional record-type record &rest initargs)) &body body [Macro]
-(defgeneric invoke-with-output-to-output-record (stream continuation record-type &rest initargs &key))
+(defgeneric invoke-with-output-recording-options 
+    (stream continuation record draw))
+
+;;; with-new-output-record (stream &optional record-type record &rest initargs) &body body [Macro]
+
+;;; The 'constructor' arg is absent from the CLIM 2.0 spec but is documented
+;;; in the Allegro CLIM 2 User Guide and appears to exist in other 'classic'
+;;; CLIM implementations. I'm assuming it's an omission from the spec.
+(defgeneric invoke-with-new-output-record
+    (stream continuation record-type constructor &key &allow-other-keys))
+
+;;; with-output-to-output-record (stream &optional record-type record &rest initargs)) &body body [Macro]
+(defgeneric invoke-with-output-to-output-record 
+    (stream continuation record-type constructor 
+            &rest initargs &key &allow-other-keys))
+
 (defgeneric make-design-from-output-record (record))
 
 ;;;; 21.2
@@ -331,14 +524,168 @@
     (stream continuation record-type unique-id id-test cache-value cache-test
 	    &key fixed-position all-new parent-cache))
 
-;;;; 24.4
+;;; 22.2.1 The Extended Stream Input Protocol
+
+(defgeneric stream-input-buffer (stream))
+(defgeneric (setf stream-input-buffer) (buffer stream))
+(defgeneric stream-pointer-position (stream &key pointer))
+;; (defgeneric (setf* stream-pointer-position))
+(defgeneric stream-set-input-focus (stream))
+(defgeneric stream-read-gesture 
+    (stream &key timeout peek-p input-wait-test 
+            input-wait-handler pointer-button-press-handler))
+(defgeneric stream-input-wait (stream &key timeout input-wait-test))
+(defgeneric stream-unread-gesture (stream gesture))
+
+;;; 22.2.2 Extended Input Stream Conditions
+
+(defgeneric abort-gesture-event (condition))
+(defgeneric accelerator-gesture-event (condition))
+(defgeneric accelerator-gesture-numeric-argument (condition))
+
+;;; 23.5 Context-dependent (Typed) Input
+
+(defgeneric stream-accept 
+    (stream 
+     type &key view default default-type provide-default insert-default 
+     replace-input history active-p prompt prompt-mode display-default
+     query-identifier activation-gestures additional-activation-gestures
+     delimiter-gestures additional-delimiter-gestures))
+(defgeneric prompt-for-accept (stream type view &rest accept-args &key))
+
+;;; 24.1 The Input Editor
+
+(defgeneric input-editor-format (stream format-string &rest args))
+(defgeneric redraw-input-buffer (stream &optional start-from))
+
+;;; 24.1.1 The Input Editing Stream Protocol
+
+(defgeneric stream-insertion-pointer (stream))
+(defgeneric (setf stream-insertion-pointer) (pointer stream))
+(defgeneric stream-scan-pointer (stream))
+(defgeneric (setf stream-scan-pointer) (pointer stream))
+(defgeneric stream-rescanning-p (stream))
+(defgeneric reset-scan-pointer (stream &optional scan-pointer))
+(defgeneric immediate-rescan (stream))
+(defgeneric queue-rescan (stream))
+(defgeneric rescan-if-necessary (stream &optional inhibit-activation))
+(defgeneric erase-input-buffer (stream &optional start-position))
+
+;;; McCLIM relies on a text editor class (by default
+;;; DREI-INPUT-EDITING-MIXIN) to perform the user interaction and
+;;; display for input editing. Also, that class must update the stream
+;;; buffer and the insertion pointer, cause rescans to happen, and
+;;; handle activation gestures.
+(defgeneric stream-process-gesture (stream gesture type))
+
+;;; 24.4 Reading and Writing of Tokens
+
+(defgeneric replace-input 
+    (stream new-input &key start end buffer-start rescan)
+  ;; XXX: Nonstandard behavior for :rescan.
+  (:documentation "Replaces the part of the input editing stream
+`stream's input buffer that extends from `buffer-start' to its
+scan pointer with the string `new-input'. `buffer-start' defaults
+to the current input position of stream, which is the position at
+which the current accept \"session\" starts. `start' and `end' can be
+supplied to specify a subsequence of `new-input'; start defaults to
+0 and end defaults to the length of `new-input'.
+
+`replace-input' will queue a rescan by calling `queue-rescan' if
+the new input does not match the old input, or `rescan' is
+true. If `rescan' is explicitly provided as NIL, no rescan will
+be queued in any case.
+
+The returned value is the position in the input buffer."))
+
 (defgeneric presentation-replace-input
     (stream object type view
-	    &key buffer-start rescan query-identifier for-context-type))
+	    &key buffer-start rescan query-identifier for-context-type)
+  (:documentation "Like `replace-input', except that the new
+input to insert into the input buffer is gotten by presenting
+`object' with the presentation type `type' and view
+`view'. `buffer-start' and `rescan' are as for `replace-input',
+and `query-identifier' and `for-context-type' as as for
+`present'.
 
-;;;; 28.3
+Typically, this function will be implemented by calling
+`present-to-string' on `object', `type', `view', and
+`for-context-type', and then calling `replace-input' on the
+resulting string.
+
+If the object cannot be transformed into an acceptable textual
+form, it may be inserted as a special \"accept result\" that is
+considered a single gesture. These accept result objects have no
+standardised form."))
+
+;;; 27.3 Command Menus
+
+(defgeneric display-command-table-menu (command-table stream &key max-width
+                                                      max-height n-rows n-columns x-spacing
+                                                      y-spacing initial-spacing row-wise
+                                                      cell-align-x cell-align-y move-cursor)
+  (:documentation "Display a menu of the commands accessible in
+`command-table' to `stream'.
+
+`max-width', `max-height', `n-rows', `n-columns', `x-spacing',
+`y-spacing', `row-wise', `initial-spacing', `cell-align-x',
+`cell-align-y', and `move-cursor' are as for
+`formatting-item-list'."))
+
+;;; 28.2 Specifying the Panes of a Frame
+
+(defgeneric destroy-frame (frame))
+(defgeneric raise-frame (frame))
+(defgeneric bury-frame (frame))
+
+;;; 28.3 Application Frame Functions
+
+(defgeneric frame-name (frame))
+(defgeneric frame-pretty-name (frame))
+(defgeneric (setf frame-pretty-name) (name frame))
+(defgeneric frame-command-table (frame))
+(defgeneric (setf frame-command-table) (command-table frame))
+(defgeneric frame-standard-output (frame))
+(defgeneric frame-standard-input (frame))
+(defgeneric frame-query-io (frame))
+(defgeneric frame-error-output (frame))
+(defgeneric frame-pointer-documentation-output (frame))
+(defgeneric frame-calling-frame (frame))
+(defgeneric frame-parent (frame))
+(defgeneric frame-panes (frame))
+(defgeneric frame-top-level-sheet (frame))
+(defgeneric frame-current-panes (frame))
+(defgeneric get-frame-pane (frame pane-name))
+(defgeneric fine-pane-named (frame pane-name))
+(defgeneric frame-current-layout (frame))
+(defgeneric (setf frame-current-layout) (layout frame))
+(defgeneric frame-all-layouts (frame))
+(defgeneric layout-frame (frame &optional width height))
+(defgeneric frame-exit-frame (condition))
+(defgeneric frame-exit (frame))
+(defgeneric pane-needs-redisplay (pane))
+(defgeneric (setf pane-needs-redisplay) (value pane))
+(defgeneric redisplay-frame-pane (frame pane &key force-p))
+(defgeneric redisplay-frame-panes (frame &key force-p))
+(defgeneric frame-replay (frame stream &optional region))
+(defgeneric notify-user (frame message &key associated-window title
+                               documentation exit-boxes name style text-style))
+(defgeneric frame-properties (frame property))
+(defgeneric (setf frame-properties) (value frame property))
+
+;;; 28.3.1 Interface with Presentation Types
+
+(defgeneric frame-maintain-presentation-histories (frame))
 (defgeneric frame-find-innermost-applicable-presentation
     (frame input-context stream x y &key event))
+(defgeneric frame-input-context-button-press-handler 
+    (frame stream button-prees-event))
+(defgeneric frame-document-highlighted-presentation
+    (frame presentation input-context window-context x y stream))
+(defgeneric frame-drag-and-drop-feedback
+    (frame presentation stream initial-x initial-y new-x new-y state))
+(defgeneric frame-drag-and-drop-highlighting
+    (frame presentation stream state))
 
 ;;;; 28.4
 (defgeneric default-frame-top-level
@@ -347,11 +694,49 @@
 (defgeneric run-frame-top-level (frame &key &allow-other-keys))
 (defgeneric command-enabled (command-name frame))
 (defgeneric (setf command-name) (enabled command-name frame))
+(defgeneric display-command-menu (frame stream &key command-table
+                                        initial-spacing row-wise max-width
+                                        max-height n-rows n-columns
+                                        cell-align-x cell-align-y)
+  (:documentation "Display the command table associated with
+`command-table' on `stream' by calling
+`display-command-table-menu'. If no command table is
+provided, (frame-command-table frame) will be used.
 
-;;;; 28.5
+The arguments `initial-spacing', `row-wise',
+`max-width', `max-height', `n-rows', `n-columns', `cell-align-x',
+and `cell-align-y' are as for `formatting-item-list'."))
+
+;;;; 28.5.2 Frame Manager Operations
+
+(defgeneric frame-manager (frame))
+(defgeneric (setf frame-manager) (frame-manager frame))
+(defgeneric frame-manager-frames (frame-manager))
+(defgeneric adopt-frame (frame-manager frame))
+(defgeneric disown-frame (frame-manager frame))
+(defgeneric frame-state (frame))
+(defgeneric enable-frame (frame))
+(defgeneric disable-frame (frame))
+(defgeneric shrink-frame (frame))
+
+(defgeneric note-frame-enabled (frame-manager frame))
+(defgeneric note-frame-disabled (frame-manager frame))
+(defgeneric note-frame-iconified (frame-manager frame))
+(defgeneric note-frame-deiconified (frame-manager frame))
 (defgeneric note-command-enabled (frame-manager frame command-name))
 (defgeneric note-command-disabled (frame-manager frame command-name))
 
+(defgeneric frame-manager-notify-user 
+    (framem message-string &key frame associated-window title
+            documentation exit-boxes name style text-style))
+(defgeneric generate-panes (frame-manager frame))
+(defgeneric find-pane-for-frame (frame-manager frame))
+
+;;; 28.5.3 Frame Manager Settings
+
+(defgeneric (setf client-setting) (value frame setting))
+(defgeneric reset-frame (frame &rest client-settings))
+
 ;;;; 29.2
 ;;;;
 ;;;; FIXME: should we have &key &allow-other-keys here, to cause
@@ -373,6 +758,8 @@
 (defgeneric pane-scroller (pane)) 
 (defgeneric scroll-extent (pane x y)) 
 
+(deftype scroll-bar-spec () '(member t :both :vertical :horizontal nil))
+
 ;;;; 29.3.4 The Layout Protocol
 
 ;; (define-protocol-class space-requirement ()) 
@@ -415,6 +802,12 @@ Returns a SPACE-REQUIREMENT object."))
 (defgeneric window-viewport-position (window))
 ;; (defgeneric (setf* window-viewport-position) (x y window))
 
+;;; D.2 Basic Stream Functions
+
+;;; Gray Streamoid functions, but not part of any Gray proposal.
+(defgeneric stream-pathname (stream))
+(defgeneric stream-truename (stream))
+
 ;;;;
 (defgeneric gadget-value (gadget))
 (defgeneric (setf gadget-value) (new-value gadget &key invoke-callback))
@@ -428,33 +821,8 @@ Returns a SPACE-REQUIREMENT object."))
 
 ;;;
 
-(defgeneric medium-foreground (medium))
-(defgeneric medium-background (medium))
-(defgeneric medium-ink (medium))
-(defgeneric medium-transformation (medium))
-(defgeneric medium-clipping-region (medium))
-(defgeneric medium-line-style (medium))
-(defgeneric medium-text-style (medium))
-(defgeneric medium-default-text-style (medium))
-(defgeneric text-size (medium string &key text-style start end)
-  (:documentation
-   "Computes the \"cursor motion\" in device units that would take
-place if STRING were output to MEDIUM starting at position (0,0).
-
-Returns total width, total height, final x cursor position, final y
-cursor position, baseline."))
-
 (defgeneric text-style-mapping (port text-style &optional character-set))
 
-(defgeneric (setf medium-foreground) (new-value medium))
-(defgeneric (setf medium-background) (new-value medium))
-(defgeneric (setf medium-ink) (new-value medium))
-(defgeneric (setf medium-transformation) (new-value medium))
-(defgeneric (setf medium-clipping-region) (new-value medium))
-(defgeneric (setf medium-line-style) (new-value medium))
-(defgeneric (setf medium-text-style) (new-value medium))
-(defgeneric (setf medium-default-text-style) (new-value medium))
-
 (defgeneric (setf text-style-mapping)
     (mapping port text-style &optional character-set))
 (defgeneric medium-miter-limit (medium)
