@@ -117,7 +117,7 @@ accept of this query")))
 ;;; The accepting-values state machine is controlled by commands. Each
 ;;; action (e.g., "select a text field") terminates 
 
-(define-command-table accepting-values)	; :inherit-from nil???
+(define-command-table accept-values)    ; :inherit-from nil???
 
 (defvar *default-command* '(accepting-values-default-command))
 
@@ -164,18 +164,22 @@ accept of this query")))
             x-position y-position width height command-table frame-class))
   (setq stream (stream-designator-symbol stream '*standard-input*))
   (with-gensyms (accepting-values-continuation)
-    (let ((return-form
-           `(flet ((,accepting-values-continuation (,stream)
-                     ,@body))
-              (invoke-accepting-values ,stream
-                                       #',accepting-values-continuation
-                                       ,@args))
-            ))
-      `(if  ,own-window
-            (with-stream-in-own-window (,stream *standard-input* *standard-output*)
-                                       (,label)
-                                       ,return-form)
-           ,return-form))))
+    (let* ((return-form
+            `(flet ((,accepting-values-continuation (,stream)
+                      ,@body))
+               (invoke-accepting-values ,stream
+                                        #',accepting-values-continuation
+                                        ,@args)))
+           (true-form `(with-stream-in-own-window (,stream *standard-input* *standard-output*)
+                         (,label)
+                         ,return-form)))
+      ;; To avoid unreachable-code warnings, if `own-window' is a
+      ;; boolean constant, don't generate the `if' form.
+      (cond ((eq own-window t) true-form)
+            ((eq own-window nil) return-form)
+            (t `(if ,own-window
+                    ,true-form
+                    ,return-form))))))
 
 (defun invoke-accepting-values
     (stream body
@@ -184,7 +188,7 @@ accept of this query")))
      modify-initial-query resynchronize-every-pass resize-frame
      align-prompts label scroll-bars
      x-position y-position width height
-     (command-table 'accepting-values)
+     (command-table 'accept-values)
      (frame-class 'accept-values))
   (declare (ignore own-window exit-boxes modify-initial-query
     resize-frame label scroll-bars x-position y-position
@@ -222,7 +226,7 @@ accept of this query")))
                         (when resynchronize-every-pass
                           (redisplay arecord stream)))
                     (with-input-context
-                        ('(command :command-table accepting-values))
+                        ('(command :command-table accept-values))
                       (object)
                       (progn
                         (apply (command-name current-command)
@@ -274,7 +278,9 @@ accept of this query")))
 			  additional-activation-gestures
 			  delimiter-gestures
 			  additional-delimiter-gestures)
-  (declare (ignore activation-gestures additional-activation-gestures
+  (declare (ignore default-type provide-default insert-default replace-input
+		   history active-p prompt-mode display-default
+		   activation-gestures additional-activation-gestures
 		   delimiter-gestures additional-delimiter-gestures))
   (let ((query (find query-identifier (queries stream)
 		     :key #'query-identifier :test #'equal))
@@ -294,7 +300,9 @@ accept of this query")))
 				 :default default
 				 :default-supplied-p default-supplied-p
 				 :value default))
-      (setf (queries stream) (nconc (queries stream) (list query))))
+      (setf (queries stream) (nconc (queries stream) (list query)))
+      (when default
+        (setf (changedp query) t)))
     (setf (accept-arguments query) rest-args)
     ;; If the program changes the default, that becomes the value.
     (unless (equal default (default query)) 
@@ -334,20 +342,20 @@ accept of this query")))
   (declare (ignore view))
   (apply #'prompt-for-accept-1 stream type :display-default nil args))
 
-(define-command (com-query-exit :command-table accepting-values
+(define-command (com-query-exit :command-table accept-values
 				:name nil
 				:provide-output-destination-keyword nil)
     ()
   (signal 'av-exit))
 
-(define-command (com-query-abort :command-table accepting-values
+(define-command (com-query-abort :command-table accept-values
 				 :name nil
 				 :provide-output-destination-keyword nil)
     ()
   (and (find-restart 'abort)
        (invoke-restart 'abort)))
 
-(define-command (com-change-query :command-table accepting-values
+(define-command (com-change-query :command-table accept-values
 				  :name nil
 				  :provide-output-destination-keyword nil)
     ((query-identifier t)
@@ -368,7 +376,7 @@ a query is selected for input. It is responsible for updating the
   (:documentation "Deselect a query field: turn the cursor off, turn off
 highlighting, etc." ))
 
-(define-command (com-select-query :command-table accepting-values
+(define-command (com-select-query :command-table accept-values
 				  :name nil
 				  :provide-output-destination-keyword nil)
     ((query-identifier t))
@@ -387,14 +395,14 @@ highlighting, etc." ))
 	(when query
 	  (setf selected-query query)
 	  (select-query *accepting-values-stream* query (record query))
-	  (let ((command-ptype '(command :command-table accepting-values)))
+	  (let ((command-ptype '(command :command-table accept-values)))
 	    (if (cdr query-list)
 	      (throw-object-ptype `(com-select-query ,(query-identifier
 						       (cadr query-list)))
 				  command-ptype)
 	      (throw-object-ptype '(com-deselect-query) command-ptype))))))))
 
-(define-command (com-deselect-query :command-table accepting-values
+(define-command (com-deselect-query :command-table accept-values
 				    :name nil
 				    :provide-output-destination-keyword nil)
     ()
@@ -439,28 +447,30 @@ is called. Used to determine if any editing has been done by user")))
   (declare (ignore present-p))
   (let* ((editing-stream nil)
 	 (record (updating-output (stream :unique-id query-identifier
-				   :cache-value (if default-supplied-p
-						    default
-						    *no-default-cache-value*)
-				   :record-type 'av-text-record)
-		   (with-output-as-presentation
-		       (stream query-identifier 'selectable-query
-			       :single-box t)
-		     (surrounding-output-with-border
-		         (stream :shape :inset :move-cursor t)
-		       (setq editing-stream
-			     (make-instance 'standard-input-editing-stream
-					    :stream stream
-					    :cursor-visibility nil
-					    :background-ink +grey90+
-					    :single-line t))))
-		   (when default-supplied-p
-		     (input-editing-rescan-loop ;XXX probably not needed
-		      editing-stream
-		      (lambda (s)
-			(presentation-replace-input s default type view
-						    :rescan t)
-			(goatee::update-input-editing-stream s)))))))
+                                          :cache-value (if default-supplied-p
+                                                           default
+                                                           *no-default-cache-value*)
+                                          :record-type 'av-text-record)
+                   (with-output-as-presentation
+                       (stream query-identifier 'selectable-query
+                               :single-box t)
+                     (surrounding-output-with-border
+                         (stream :shape :inset :move-cursor t)
+                       (setq editing-stream
+                             (make-instance (if *use-goatee*
+                                                'goatee-input-editing-stream
+                                                'standard-input-editing-stream)
+                                            :stream stream
+                                            :cursor-visibility nil
+                                            :background-ink +grey90+
+                                            :single-line t
+                                            :min-width t))))
+                   (when default-supplied-p
+                     (input-editing-rescan-loop ;XXX probably not needed
+                      editing-stream
+                      (lambda (s)
+                        (presentation-replace-input s default type view
+                                                    :rescan t)))))))
     (when editing-stream
       (setf (editing-stream record) editing-stream))
     record))
@@ -500,21 +510,19 @@ is called. Used to determine if any editing has been done by user")))
 		     ((error
 		       #'(lambda (c)
 			   (format *trace-output*
-				   "accepting-values accept condition: ~A~%"
-				   c)
+                                   "accepting-values accept condition: ~A~%"
+                                   c)
 			   (if interactive
-			       (progn
-				 (beep)
-				 (goatee::set-editing-stream-insertion-pointer
-				  estream
-				  (1- (stream-scan-pointer estream)))
-				 (immediate-rescan estream)
-				 (format *trace-output* "Ack!~%"))
-			       (progn
-				 (setf (accept-condition query) c)
-				 (return-from accept-condition-handler
-				   c))))))
-		   (goatee::update-input-editing-stream s)
+                               (progn
+                                 (beep)
+                                 (setf (stream-insertion-pointer estream)
+                                       (max 0 (1- (stream-scan-pointer estream))))
+                                 (immediate-rescan estream)
+                                 (format *trace-output* "Ack!~%"))
+                               (progn
+                                 (setf (accept-condition query) c)
+                                 (return-from accept-condition-handler
+                                   c))))))
 		   (if default-supplied-p
 		       (accept ptype :stream s
 			       :view view :prompt nil :default default)
@@ -583,7 +591,7 @@ is run for the last time"))
 
 
 (define-presentation-to-command-translator com-select-field
-    (selectable-query com-select-query accepting-values
+    (selectable-query com-select-query accept-values
      :gesture :select
      :documentation "Select field for input"
      :pointer-documentation "Select field for input"
@@ -596,7 +604,7 @@ is run for the last time"))
   `(,object))
 
 (define-presentation-to-command-translator com-exit-button
-    (exit-button com-query-exit accepting-values
+    (exit-button com-query-exit accept-values
      :gesture :select
      :documentation "Exit dialog"
      :pointer-documentation "Exit dialog"
@@ -605,7 +613,7 @@ is run for the last time"))
   ())
 
 (define-presentation-to-command-translator com-abort-button
-    (abort-button com-query-abort accepting-values
+    (abort-button com-query-abort accept-values
      :gesture :select
      :documentation "Abort dialog"
      :pointer-documentation "Abort dialog"
@@ -616,3 +624,97 @@ is run for the last time"))
 (defun accepting-values-default-command ()
   (loop
    (read-gesture :stream *accepting-values-stream*)))
+
+
+;;;; notify-user
+
+;;; See http://openmap.bbn.com/hypermail/clim/0028.html for example usage.
+
+;;; TODO:
+;;;   - associated-window argument?
+;;;   - What is the correct return value from notify-user? We currently return
+;;;     the name of the action given in the :exit-boxes argument.
+;;;   - Invoke abort restart? Not necessary as it is with accepting-values,
+;;;     but probably what "Classic CLIM" does.
+;;;   - What are the default exit boxes? Just "Okay"? Okay and cancel?
+;;;   - Reimplement using accepting-values, if accepting-values is ever
+;;;     improved to produce comparable dialogs.
+;;;   - Should the user really be able to close the window from the WM?
+
+(defmethod notify-user (frame message &rest args)
+  (apply #'frame-manager-notify-user
+	 (if frame (frame-manager frame) (find-frame-manager))
+	 message
+	 :frame frame
+	 args))
+
+(define-application-frame generic-notify-user-frame ()
+  ((message-string :initarg :message-string)
+   (exit-boxes :initarg :exit-boxes)
+   (title :initarg :title)
+   (style :initarg :style)
+   (text-style :initarg :text-style)
+   (return-value :initarg nil :initform :abort))
+  (:pane (generate-notify-user-dialog *application-frame*)))
+
+(defun generate-notify-user-dialog (frame)
+  (with-slots (message-string exit-boxes text-style) frame
+  (vertically ()
+    (spacing (:thickness 6)
+      (make-pane 'label-pane :label (or message-string "I'm speechless.") :text-style text-style))
+    (spacing (:thickness 4)
+      (make-pane 'hbox-pane :contents (cons '+fill+ (generate-exit-box-buttons exit-boxes)))))))
+
+(defun generate-exit-box-buttons (specs)
+  (mapcar
+   (lambda (spec)
+     (destructuring-bind (action string &rest args) spec
+       (spacing (:thickness 2)
+         (apply #'make-pane
+                'push-button
+                :label string
+                :text-style (make-text-style :sans-serif :roman :small) ; XXX
+                :activate-callback
+                (lambda (gadget)
+                  (declare (ignore gadget))
+                  ;; This is fboundp business is weird, and only implied by a 
+                  ;; random message on the old CLIM list. Does the user function
+                  ;; take arguments?
+                  (when (or (typep action 'function) (fboundp action))
+                    (funcall action))
+                  (setf (slot-value *application-frame* 'return-value) action)
+                  ;; This doesn't work:
+                  #+NIL 
+                  (when (eql action :abort)
+                    (and (find-restart 'abort)
+                         (invoke-restart 'abort)))
+                  (frame-exit *application-frame*))
+                args))))
+   specs))
+
+		  
+(defmethod frame-manager-notify-user 
+    (frame-manager message-string &key frame associated-window
+		   (title "")
+		   documentation
+		   (exit-boxes '((:exit "OK")))
+		   ; The 'name' arg is in the spec but absent from the Lispworks
+		   ; manual, and I can't imagine what it would do differently
+		   ; than 'title'.
+		   name
+		   style
+		   (text-style (make-text-style :sans-serif :roman :small)))
+  (declare (ignore associated-window documentation))
+  ;; Keywords from notify-user:
+  ;; associated-window title documentation exit-boxes name style text-style
+  (let ((frame (make-application-frame 'generic-notify-user-frame
+                                       :frame-event-queue (and frame (frame-event-queue frame))
+                                       :pretty-name title
+                                       :message-string message-string
+                                       :frame-manager frame-manager
+                                       :exit-boxes exit-boxes
+                                       :title (or name title)
+                                       :style style
+                                       :text-style text-style)))
+    (run-frame-top-level frame)
+    (slot-value frame 'return-value)))
