@@ -30,14 +30,21 @@
 
 ;;; Frame-Manager class
 
+;; FIXME: The spec says the port must "conform to options".
+;; I've added a check that the ports match, but we've no
+;; protocol for testing the other options. -Hefner
 (defun find-frame-manager (&rest options &key port &allow-other-keys)
   (declare (special *frame-manager*))
-  (if (boundp '*frame-manager*)
+  (if (and (boundp '*frame-manager*)
+           (or (null port)
+               (eql port (frame-manager-port *frame-manager*))))
       *frame-manager*
     (if (and *default-frame-manager*
-	     (frame-manager-p *default-frame-manager*))
+             (frame-manager-p *default-frame-manager*)
+             (or (null port)
+                 (eql port (frame-manager-port *default-frame-manager*))))
 	*default-frame-manager*
-      (first (frame-managers (or port (apply #'find-port options)))))))
+        (first (frame-managers (or port (apply #'find-port options)))))))
 
 (defmacro with-frame-manager ((frame-manager) &body body)
   `(let ((*frame-manager* ,frame-manager))
@@ -59,102 +66,12 @@
 ;;; XXX All these slots should move to a mixin or to standard-application-frame.
 ;;; -- moore
 
-;;; Generic operations
-(defgeneric frame-name (frame))
-(defgeneric frame-pretty-name (frame))
-(defgeneric (setf frame-pretty-name) (name frame))
-(defgeneric frame-command-table (frame))
-(defgeneric (setf frame-command-table) (command-table frame))
-(defgeneric frame-standard-output (frame)
-  (:documentation
-   "Returns the stream that will be used for *standard-output* for the FRAME."))
-(defgeneric frame-standard-input (frame)
-  (:documentation
-   "Returns the stream that will be used for *standard-input* for the FRAME."))
-(defgeneric frame-query-io (frame)
-  (:documentation
-   "Returns the stream that will be used for *query-io* for the FRAME."))
-(defgeneric frame-error-output (frame)
-  (:documentation
-   "Returns the stream that will be used for *error-output* for the FRAME."))
-(defgeneric frame-pointer-documentation-output (frame)
-  (:documentation
-   "Returns the stream that will be used for *pointer-documentation-output*
-for the FRAME."))
-(defgeneric frame-calling-frame (frame)
-  (:documentation
-   "Returns the application frame that invoked the FRAME."))
-(defgeneric frame-parent (frame)
-  (:documentation
-   "Returns the object that acts as the parent for the FRAME."))
-(defgeneric frame-panes (frame)
-  (:documentation
-   "Returns the pane that is the top-level pane in the current layout
-of the FRAME's named panes."))
-(defgeneric frame-top-level-sheet (frame)
-  (:documentation
-   "Returns the shhet that is the top-level sheet for the FRAME. This
-is the sheet that has as its descendants all of the panes of the FRAME."))
-(defgeneric frame-current-panes (frame)
-  (:documentation
-   "Returns a list of those named panes in the FRAME's current layout.
-If there are no named panes, only the single, top level pane is returned."))
-(defgeneric get-frame-pane (frame pane-name)
-  (:documentation
-   "Returns the named CLIM stream pane in the FRAME whose name is PANE-NAME."))
-(defgeneric find-pane-named (frame pane-name)
-  (:documentation
-   "Returns the pane in the FRAME whose name is PANE-NAME."))
-(defgeneric frame-current-layout (frame))
-(defgeneric (setf frame-current-layout) (layout frame))
-(defgeneric frame-all-layouts (frame))
-(defgeneric layout-frame (frame &optional width height))
-(defgeneric frame-exit-frame (condition)
-  (:documentation
-   "Returns the frame that is being exited from associated with the
-FRAME-EXIT condition."))
-(defgeneric frame-exit (frame)
-  (:documentation
-   "Exits from the FRAME."))
-(defgeneric pane-needs-redisplay (pane))
-(defgeneric (setf pane-needs-redisplay) (value pane))
-(defgeneric redisplay-frame-pane (frame pane &key force-p))
-(defgeneric redisplay-frame-panes (frame &key force-p))
-(defgeneric frame-replay (frame stream &optional region))
-(defgeneric notify-user (frame message &key associated-window title
-                         documentation exit-boxes name style text-style))
-(defgeneric frame-properties (frame property))
-(defgeneric (setf frame-properties) (value frame property))
-(defgeneric (setf client-setting) (value frame setting))
-(defgeneric reset-frame (frame &rest client-settings))
-(defgeneric frame-maintain-presentation-histories (frame))
-
 ; extension
 (defgeneric frame-schedule-timer-event (frame sheet delay token))
 
 (defgeneric note-input-focus-changed (pane state)
   (:documentation "Called when a pane receives or loses the keyboard
 input focus. This is a McCLIM extension."))
-
-(defmethod frame-geometry* ((frame application-frame))
-  "-> width height &optional top left"
-  (let ((pane (frame-top-level-sheet frame)))
-    (destructuring-bind (&key left top right bottom width height) (frame-geometry frame)
-      ;; Find width and height from looking at the respective options
-      ;; first, then at left/right and top/bottom and finally at what
-      ;; compose-space says.
-      (setf width (or width
-                      (and left right (- right left))
-                      (space-requirement-width (compose-space pane))))
-      (setf height (or height
-                       (and top bottom (- bottom top))
-                       (space-requirement-height (compose-space pane))))
-      ;; See if a position is wanted and return left, top.
-      (setf left (or left
-                     (and right (- right width))))
-      (setf top (or top
-                    (and bottom (- bottom height))))
-      (values width height left top))))
     
 (defclass standard-application-frame (application-frame
 				      presentation-history-mixin)
@@ -192,8 +109,6 @@ input focus. This is a McCLIM extension."))
    (manager :initform nil
 	    :reader frame-manager
             :accessor %frame-manager)
-   (keyboard-input-focus :initform nil
-                         :accessor keyboard-input-focus)
    (properties :accessor %frame-properties
 	       :initarg :properties
 	       :initform nil)
@@ -204,11 +119,7 @@ input focus. This is a McCLIM extension."))
 		     :reader frame-top-level-lambda)
    (hilited-presentation :initform nil
 			 :initarg :hilited-presentation
-			 :accessor frame-hilited-presentation)
-   (user-supplied-geometry :initform nil
-			   :initarg :user-supplied-geometry
-			   :reader frame-geometry
-                           :documentation "plist of defaulted :left, :top, :bottom, :right, :width and :height options.")
+			 :accessor frame-hilited-presentation)   
    (process :accessor frame-process :initform nil)
    (client-settings :accessor client-settings :initform nil)
    (event-queue :initarg :frame-event-queue
@@ -235,7 +146,49 @@ frame, if any")
    (documentation-record :accessor documentation-record
 			 :initform nil
 			 :documentation "updating output record for pointer
-documentation produced by presentations.")))
+documentation produced by presentations.")
+   (geometry-left :accessor geometry-left
+		  :initarg :left
+		  :initform nil)
+   (geometry-right :accessor geometry-right
+		  :initarg :right
+		  :initform nil)
+   (geometry-top :accessor geometry-top
+		  :initarg :top
+		  :initform nil)
+   (geometry-bottom :accessor geometry-bottom
+		  :initarg :bottom
+		  :initform nil)
+   (geometry-width :accessor geometry-width
+		  :initarg :width
+		  :initform nil)
+   (geometry-height :accessor geometry-height
+		  :initarg :height
+		  :initform nil)))
+
+(defmethod frame-geometry* ((frame standard-application-frame))
+  "-> width height &optional top left"
+  (let ((pane (frame-top-level-sheet frame)))
+    ;(destructuring-bind (&key left top right bottom width height) (frame-geometry frame)
+    (with-slots (geometry-left geometry-top geometry-right
+			       geometry-bottom geometry-width
+			       geometry-height) frame
+      ;; Find width and height from looking at the respective options
+      ;; first, then at left/right and top/bottom and finally at what
+      ;; compose-space says.
+      (let* ((width (or geometry-width
+			(and geometry-left geometry-right
+			     (- geometry-right geometry-left))
+			(space-requirement-width (compose-space pane))))
+	     (height (or geometry-height
+			 (and geometry-top geometry-bottom (- geometry-bottom geometry-top))
+			 (space-requirement-height (compose-space pane))))	   
+	     ;; See if a position is wanted and return left, top.
+	     (left (or geometry-left
+		       (and geometry-right (- geometry-right geometry-width))))
+	     (top (or geometry-top
+		      (and geometry-bottom (- geometry-bottom geometry-height)))))
+      (values width height left top)))))
 
 ;;; Support the :input-buffer initarg for compatibility with "real CLIM"
 
@@ -288,10 +241,11 @@ documentation produced by presentations.")))
     (sheet-adopt-child (graft frame) (frame-top-level-sheet frame)))
   ;; Find the size of the new frame
   (multiple-value-bind (w h x y) (frame-geometry* frame)
+    (declare (ignore x y))
     ;; automatically generates a window-configuation-event
     ;; which then calls allocate-space
     ;;
-    ;; Not any longer, we turn of CONFIGURE-NOTIFY events until the
+    ;; Not any longer, we turn off CONFIGURE-NOTIFY events until the
     ;; window is mapped and do the space allocation now, so that all
     ;; sheets will have their correct geometry at once. --GB
     (setf (sheet-region (frame-top-level-sheet frame))
@@ -300,8 +254,9 @@ documentation produced by presentations.")))
 
 (defmethod layout-frame ((frame application-frame) &optional width height)
   (let ((pane (frame-panes frame)))
-    (if (and  width (not height))
-	(error "LAYOUT-FRAME must be called with both WIDTH and HEIGHT or neither"))
+    (when (and (or width height)
+               (not (and width height)))
+      (error "LAYOUT-FRAME must be called with both WIDTH and HEIGHT or neither"))
     (if (and (null width) (null height))
 	(let ((space (compose-space pane))) ;I guess, this might be wrong. --GB 2004-06-01
 	  (setq width (space-requirement-width space))
@@ -437,22 +392,25 @@ documentation produced by presentations.")))
 
 (defmethod redisplay-frame-pane :around ((frame application-frame) pane
 					 &key force-p)
-  (multiple-value-bind (redisplayp clearp)
-      (pane-needs-redisplay pane)
-    (when force-p
-      (setq redisplayp (or redisplayp t)
-	    clearp t))
-    (when redisplayp
-      (let ((hilited (frame-hilited-presentation frame)))
-	(when hilited
-	  (highlight-presentation-1 (car hilited) (cdr hilited) :unhighlight)
-	  (setf (frame-hilited-presentation frame) nil)))
-      (with-possible-double-buffering (frame pane)
-	(when clearp
-	  (window-clear pane))
-	(call-next-method))
-      (unless (or (eq redisplayp :command-loop) (eq redisplayp :no-clear))
-	(setf (pane-needs-redisplay pane) nil)))))
+  (let ((pane-object (if (typep pane 'pane)
+			 pane
+			 (find-pane-named frame pane))))
+    (multiple-value-bind (redisplayp clearp)
+	(pane-needs-redisplay pane-object)
+      (when force-p
+	(setq redisplayp (or redisplayp t)
+	      clearp t))
+      (when redisplayp
+	(let ((hilited (frame-hilited-presentation frame)))
+	  (when hilited
+	    (highlight-presentation-1 (car hilited) (cdr hilited) :unhighlight)
+	    (setf (frame-hilited-presentation frame) nil)))
+	(with-possible-double-buffering (frame pane-object)
+	  (when clearp
+	    (window-clear pane-object))
+	  (call-next-method))
+	(unless (or (eq redisplayp :command-loop) (eq redisplayp :no-clear))
+	  (setf (pane-needs-redisplay pane-object) nil))))))
 
 (defmethod run-frame-top-level ((frame application-frame)
 				&key &allow-other-keys)
@@ -528,11 +486,10 @@ documentation produced by presentations.")))
 	       (redisplay-frame-panes frame :force-p first-time)
 	       (setq first-time nil)
 	       (if query-io
-                 ;; We don't need to turn the cursor on here, as Goatee has its own
-                 ;; cursor which will appear. In fact, leaving it on causes much
-                 ;; bit flipping and slows command output somewhat. So, leave it
-                 ;; off by default, and hope this doesn't violate the spec.  
+                   ;; For frames with an interactor:
 		   (progn
+                     ;; Hide cursor, so we don't need to toggle it during
+                     ;; command output.
 		     (setf (cursor-visibility (stream-text-cursor *query-io*))
 			   nil)
 		     (when (and prompt interactorp)
@@ -549,7 +506,9 @@ documentation produced by presentations.")))
 			 (execute-frame-command frame command))
 		       (when interactorp
 			 (fresh-line *query-io*))))
-		   (simple-event-loop)))
+                   ;; Frames without an interactor:
+                   (let ((command (read-frame-command frame :stream nil)))
+                     (when command (execute-frame-command frame command)))))
 	   (abort ()
 	     :report "Return to application command loop"
 	     (if interactorp
@@ -557,16 +516,16 @@ documentation produced by presentations.")))
 		 (beep))))))))
 
 (defmethod read-frame-command :around ((frame application-frame)
-				       &key (stream *standard-input*))  
+				       &key (stream *standard-input*))
   (with-input-context ('menu-item)
       (object)
       (call-next-method)
     (menu-item
      (let ((command (command-menu-item-value object)))
        (unless (listp command)
-	 (setq command (list command)))       
+	 (setq command (partial-command-from-name command)))
        (if (and (typep stream 'interactor-pane)
-		(member *unsupplied-argument-marker* command :test #'eq))
+		(partial-command-p command))
 	   (command-line-read-remaining-arguments-for-partial-command
 	    (frame-command-table frame) stream command 0)
 	   command)))))
@@ -579,9 +538,11 @@ documentation produced by presentations.")))
   ;; If we do things as the spec says, command accelerators will
   ;; appear to not work, confusing new users.
   #+NIL (read-command (frame-command-table frame) :use-keystrokes nil :stream stream)
-  (read-command (frame-command-table frame) :use-keystrokes t :stream stream))
+  (if stream
+      (read-command (frame-command-table frame) :use-keystrokes t :stream stream)
+      (simple-event-loop frame)))
 
-(defclass execute-command-event (window-manager-event)
+(define-event-class execute-command-event (window-manager-event)
   ((sheet :initarg :sheet :reader event-sheet)
    (command :initarg :command :reader execute-command-event-command)))
 
@@ -627,7 +588,6 @@ documentation produced by presentations.")))
 				 command-name)
 	  nil))))
 
-
 (defmethod make-pane-1 :around (fm (frame standard-application-frame) type
 				&rest args
 				&key (input-buffer nil input-buffer-p)
@@ -657,7 +617,7 @@ documentation produced by presentations.")))
          #+clim-mp (event-queue (sheet-event-queue t-l-s)))
     (setf (slot-value frame 'top-level-sheet) t-l-s)
     (generate-panes fm frame)
-    (setf (slot-value frame 'state)  :disabled)
+    (setf (slot-value frame 'state) :disabled)
     #+clim-mp
     (when (typep event-queue 'port-event-queue)
       (setf (event-queue-port event-queue)
@@ -677,15 +637,6 @@ documentation produced by presentations.")))
   (port-force-output (frame-manager-port fm))
   frame)
 
-(defgeneric enable-frame (frame))
-(defgeneric disable-frame (frame))
-(defgeneric destroy-frame (frame))
-(defgeneric raise-frame (frame))
-(defgeneric bury-frame (frame))
-
-(defgeneric note-frame-enabled (frame-manager frame))
-(defgeneric note-frame-disbled (frame-manager frame))
-
 (defmethod enable-frame ((frame application-frame))
   (setf (sheet-enabled-p (frame-top-level-sheet frame)) t)
   (setf (slot-value frame 'state) :enabled)
@@ -775,7 +726,7 @@ documentation produced by presentations.")))
   pane)
 
 (defun coerce-pane-name (pane name)
-  (when pane 
+  (when pane
     (setf (slot-value pane 'name) name)    
     (push pane (frame-named-panes (pane-frame pane))))
   pane)
@@ -861,6 +812,7 @@ documentation produced by presentations.")))
 	(others nil)	
 	(pointer-documentation nil)
 	(geometry nil)
+	(user-default-initargs nil)
 	(frame-arg (gensym "FRAME-ARG")))
     (loop for (prop . values) in options
 	do (case prop
@@ -876,7 +828,8 @@ documentation produced by presentations.")))
 	     (:top-level (setq top-level (first values)))
 	     (:pointer-documentation (setq pointer-documentation (car values)))
 	     (:geometry (setq geometry values))
-	     (t (push (cons prop values) others))))
+	     (:default-initargs (setq user-default-initargs values))
+	     (t (push (cons prop values) others))))    
     (when (eq command-definer t)
       (setf command-definer
             (intern (concatenate 'string
@@ -904,12 +857,10 @@ documentation produced by presentations.")))
          :top-level (list ',(car top-level) ,@(cdr top-level))
          :top-level-lambda (lambda (,frame-arg)
                              (,(car top-level) ,frame-arg
-                               ,@(cdr top-level))))
+                               ,@(cdr top-level)))
+	 ,@geometry
+	 ,@user-default-initargs)
         ,@others)
-      ;; We alway set the frame class default geometry, so that the
-      ;; user can undo the effect of a specified :geometry option.
-      ;; --GB 2004-06-01
-      (setf (get ',name 'application-frame-geometry) ',geometry)
       ,(if pane
            (make-single-pane-generate-panes-form name menu-bar pane)
            (make-panes-generate-panes-form name menu-bar panes layouts
@@ -923,9 +874,6 @@ documentation produced by presentations.")))
                       (command-table ',(first command-table)))
                   `(define-command (,name :command-table ,command-table ,@options) ,arguments ,@body))))))))
 
-(defun get-application-frame-class-geometry (name indicator)
-  (getf (get name 'application-frame-geometry) indicator nil))
-
 (defun make-application-frame (frame-name
 			       &rest options
 			       &key (pretty-name
@@ -933,25 +881,14 @@ documentation produced by presentations.")))
 			            (frame-manager nil frame-manager-p)
 			            enable
 			            (state nil state-supplied-p)
-				    (left (get-application-frame-class-geometry frame-name :left))
-				    (top (get-application-frame-class-geometry frame-name :top))
-				    (right (get-application-frame-class-geometry frame-name :right))
-				    (bottom (get-application-frame-class-geometry frame-name :bottom))
-				    (width (get-application-frame-class-geometry frame-name :width))
-				    (height  (get-application-frame-class-geometry frame-name :height))
 			            save-under (frame-class frame-name)
 			       &allow-other-keys)
   (declare (ignore save-under))
   (with-keywords-removed (options (:pretty-name :frame-manager :enable :state
-				   :left :top :right :bottom :width :height
 				   :save-under :frame-class))
     (let ((frame (apply #'make-instance frame-class
 			:name frame-name
 			:pretty-name pretty-name
-                        :user-supplied-geometry
-                        (list :left left :top top
-                              :right right :bottom bottom
-                              :width width :height height)
 			options)))
       (when frame-manager-p
 	(adopt-frame frame-manager frame))
@@ -1009,11 +946,14 @@ documentation produced by presentations.")))
    (graft :initform nil :accessor graft)
    (manager :initform nil :accessor frame-manager)))
 
+(defclass menu-unmanaged-top-level-sheet-pane (unmanaged-top-level-sheet-pane)
+  ())
+
 (defmethod adopt-frame ((fm frame-manager) (frame menu-frame))
   (setf (slot-value fm 'frames) (cons frame (slot-value fm 'frames)))
   (setf (frame-manager frame) fm)
   (let* ((t-l-s (make-pane-1 fm *application-frame*
-                             'unmanaged-top-level-sheet-pane
+                             'menu-unmanaged-top-level-sheet-pane
 			     :name 'top-level-sheet)))
     (setf (slot-value frame 'top-level-sheet) t-l-s)
     (sheet-adopt-child t-l-s (frame-panes frame))
@@ -1375,10 +1315,10 @@ documentation produced by presentations.")))
                                input-context)
   (frame-update-pointer-documentation frame input-context stream event))
 
-(defun simple-event-loop ()
+(defun simple-event-loop (&optional (frame *application-frame*))
   "An simple event loop for applications that want all events to be handled by
  handle-event methods"
-  (let ((queue (frame-event-queue *application-frame*)))
+  (let ((queue (frame-event-queue frame)))
     (loop for event = (event-queue-read queue)
        ;; EVENT-QUEUE-READ in single-process mode calls PROCESS-NEXT-EVENT itself.
        do (handle-event (event-sheet event) event))))
@@ -1388,13 +1328,9 @@ documentation produced by presentations.")))
   `(let ((,frame *application-frame*))
      ,@body))
 
-
 (defmethod note-input-focus-changed (pane state)
   (declare (ignore pane state)))
 
-(defmethod (setf keyboard-input-focus) :after (focus frame)
-  (%set-port-keyboard-focus (port frame) focus))
-
 (defmethod (setf client-setting) (value frame setting)
   (setf (getf (client-settings frame) setting) value))
 
@@ -1541,9 +1477,6 @@ documentation produced by presentations.")))
 	    (null
 	     ())))))))
 
-(defgeneric frame-drag-and-drop-feedback
-    (frame from-presentation stream initial-x initial-y x y state))
-
 (defmethod frame-drag-and-drop-feedback
     ((frame standard-application-frame) from-presentation stream
      initial-x initial-y x y state)
@@ -1568,9 +1501,6 @@ documentation produced by presentations.")))
 		(buffer-rectangle))
 	     (stream-replay stream buffer-rectangle))))))))
 
-(defgeneric frame-drag-and-drop-highlighting
-    (frame to-presentation stream state))
-
 (defmethod frame-drag-and-drop-highlighting
     ((frame standard-application-frame) to-presentation stream state)
   (highlight-presentation-1 to-presentation stream state))
