@@ -111,7 +111,7 @@
           (if (null line-style)              
               (setf line-style old-line-style))
 	  (when (or line-unit line-thickness dashes-p line-joint-shape line-cap-shape)
-            (setf changed-line-style T)
+            (setf changed-line-style t)
             (setf line-style (make-line-style
                               :unit (or line-unit
                                         (line-style-unit line-style))
@@ -130,7 +130,7 @@
 						  (medium-merged-text-style medium)))
 	    (setf text-style (medium-merged-text-style medium)))
 	  (when (or text-family-p text-face-p text-size-p)
-            (setf changed-text-style T)
+            (setf changed-text-style t)
             (setf text-style (merge-text-styles (make-text-style text-family
                                                                  text-face
                                                                  text-size)
@@ -528,9 +528,10 @@
 			toward-x toward-y transform-glyphs
 			ink clipping-region transformation
 			text-style text-family text-face text-size)
-" Draws a single character of filled text represented by the given element. 
-  element is a character or other object to be translated into a font index.
-  The given x and y specify the left baseline position for the character."
+"Draws a single character of filled text represented by the given
+element.  element is a character or other object to be translated
+into a font index.  The given x and y specify the left baseline
+position for the character."
   (declare (ignore ink clipping-region transformation
 		   text-style text-family text-face text-size))
   (with-medium-options (sheet args)
@@ -569,17 +570,24 @@
                (start    (sqrt (+ (expt (- x2 x1) 2)
                                   (expt (- y2 y1) 2))))
                (p end)
-               (q start)                                  
+               (q start)
 
-               (medium     (sheet-medium sheet))
-               (line-style (medium-line-style medium))
-               (thickness  (line-style-thickness line-style))
+               (line-style (medium-line-style sheet))
+               ;; FIXME: I believe this thickness is in "line-style-units",
+               ;; which are only coincidentally the same as pixel coorindates
+               ;; on screen backends, using :normal units. There is no function
+               ;; documented for converting the units to stream coordinates.
+               (thickness  (multiple-value-bind (dx dy)
+                               (transform-distance (invert-transformation (medium-transformation sheet))
+                                               (line-style-thickness line-style)
+                                               0)
+                             (sqrt (+ (* dx dx) (* dy dy)))))
                (width/2    (/ head-width 2))
 
                (a (atan (/ width/2 head-length)))
                (offset     (if (and head-length (not (zerop head-length)))
                                (/ thickness (* 2 (sin a )))
-                               0.0))               
+                               0.0))
                (tip-to-peak (+ head-length offset (- (* thickness 0.5 (sin a)))))) ;; okay, a guess..
          (when to-head   (incf p offset))
          (when from-head (decf q offset))
@@ -596,7 +604,7 @@
                                     start 0
                                      (/ start 2) (- width))
                               :filled t
-                              :line-thickness 0 ))
+                              :line-thickness 0))
              (progn
                (when to-head
                  (draw-polygon* sheet
@@ -642,10 +650,12 @@
       (with-medium-options (sheet args)
         (if (coordinate<= y-radius x-radius)
             (let ((x1 (- center-x x-radius)) (x2 (+ center-x x-radius))
-                  (y1 (- center-y y-radius)) (y2 (+ center-y y-radius)))              
+                  (y1 (- center-y y-radius)) (y2 (+ center-y y-radius)))
               (if filled
-                  (draw-rectangle* sheet x1 y1 x2 y2)
-                  (draw-lines* sheet (list x1 y1 x2 y1 x1 y2 x2 y2)))              
+                  ;; Kludge coordinates, sometimes due to rounding the lines don't connect.
+                  (draw-rectangle* sheet (floor x1) y1 (ceiling x2) y2)
+                  (draw-lines* sheet (list (floor x1) y1 (ceiling x2) y1
+                                           (floor x1) y2 (ceiling x2) y2)))
               (draw-circle* sheet x1 center-y y-radius
                             :filled filled
                             :start-angle (* pi 0.5)
@@ -868,7 +878,7 @@
 
 (defmacro def-graphic-op (name (&rest args))
   (let ((method-name (symbol-concat '#:medium- name '*)))
-    `(eval-when (eval load compile)
+    `(eval-when (:execute :load-toplevel :compile-toplevel)
        (defmethod ,method-name ((stream sheet) ,@args)
 	 (with-sheet-medium (medium stream)
 	   (,method-name medium ,@args))))))
@@ -999,12 +1009,87 @@
 (defmethod medium-draw-pattern* (medium pattern x y)
   (let ((width  (pattern-width pattern))
         (height (pattern-height pattern)))
-    #+NIL ;; debugging aid.
-    (draw-rectangle* medium x y (+ x width) (+ y height)
-                     :filled t
-                     :ink +red+)
-    (draw-rectangle* medium x y (+ x width) (+ y height)
-                     :filled t
-                     :ink (transform-region
-                           (make-translation-transformation x y)
-                           pattern))))
+    ;; As I read the spec, the pattern itself is not transformed, so
+    ;; we should draw the full (untransformed) pattern at the tranformed
+    ;; x/y coordinates. This requires we revert to the identity transformation
+    ;; before drawing the rectangle. -Hefner
+    (with-transformed-position ((medium-transformation medium) x y)
+      (with-identity-transformation (medium)
+	#+NIL ;; debugging aid.
+	(draw-rectangle* medium x y (+ x width) (+ y height)
+			 :filled t
+			 :ink +red+)
+	(draw-rectangle* medium x y (+ x width) (+ y height)
+			 :filled t
+			 :ink (transform-region
+			       (make-translation-transformation x y)
+			       pattern))))))
+
+(defun draw-rounded-rectangle* (sheet x1 y1 x2 y2
+                                      &rest args &key
+                                      (radius 7)
+                                      (radius-x radius)
+                                      (radius-y radius)
+                                      (radius-left  radius-x)
+                                      (radius-right radius-x)
+                                      (radius-top    radius-y)
+                                      (radius-bottom radius-y)
+                                      filled &allow-other-keys)
+  "Draw a rectangle with rounded corners"
+
+  (apply #'invoke-with-drawing-options sheet
+    (lambda (medium)
+      (declare (ignore medium))
+      (let ((medium sheet))
+        (if (not (and (>= (- x2 x1) (* 2 radius-x))
+                      (>= (- y2 y1) (* 2 radius-y))))
+            (draw-rectangle* medium x1 y1 x2 y2)       
+            (with-grown-rectangle* ((ix1 iy1 ix2 iy2) (x1 y1 x2 y2)
+                                    :radius-left   (- radius-left)
+                                    :radius-right  (- radius-right)
+                                    :radius-top    (- radius-top)
+                                    :radius-bottom (- radius-bottom))
+              (let ((zl (zerop radius-left))
+                    (zr (zerop radius-right))
+                    (zt (zerop radius-top))
+                    (zb (zerop radius-bottom)))
+                (if filled
+                    (progn              ; Filled
+                      (unless (or zl zt)
+                        (draw-ellipse* medium ix1 iy1 radius-left  0 0 radius-top    :filled t))
+                      (unless (or zr zt)
+                        (draw-ellipse* medium ix2 iy1 radius-right 0 0 radius-top    :filled t))
+                      (unless (or zl zb)
+                        (draw-ellipse* medium ix1 iy2 radius-left  0 0 radius-bottom :filled t))
+                      (unless (or zr zb)
+                        (draw-ellipse* medium ix2 iy2 radius-right 0 0 radius-bottom :filled t))
+                      (draw-rectangle* medium x1 iy1 x2 iy2 :filled t)
+                      (draw-rectangle* medium ix1 y1 ix2 iy1 :filled t)
+                      (draw-rectangle* medium ix1 iy2 ix2 y2 :filled t))                    
+                    (progn              ; Unfilled
+                      (unless (or zl zt)
+                        (draw-ellipse* medium ix1 iy1 (- radius-left) 0 0 (- radius-top)
+                                       :start-angle (/ pi 2) :end-angle pi
+                                       :filled nil))
+                      (unless (or zr zt)
+                        (draw-ellipse* medium ix2 iy1 (- radius-right) 0 0 (- radius-top)
+                                       :start-angle 0 :end-angle (/ pi 2)
+                                       :filled nil))
+                      (unless (or zl zb)
+                        (draw-ellipse* medium ix1 iy2 (- radius-left) 0 0 (- radius-bottom)
+                                       :start-angle pi :end-angle (* 3/2 pi)
+                                       :filled nil))
+                      (unless (or zr zb)
+                        (draw-ellipse* medium ix2 iy2 (- radius-right) 0 0 (- radius-bottom)
+                                       :start-angle (* 3/2 pi)
+                                       :filled nil))
+                      (labels ((fx (y p x1a x2a x1b x2b) (draw-line* medium (if p x1a x1b) y (if p x2a x2b) y))
+                               (fy (x p y1a y2a y1b y2b) (draw-line* medium x (if p y1a y1b) x (if p y2a y2b))))
+                        (fx y1 zt x1 x2 ix1 ix2)
+                        (fy x1 zl y1 y2 iy1 iy2)
+                        (fx y2 zb x1 x2 ix1 ix2)
+                        (fy x2 zr y1 y2 iy1 iy2)))))))))
+   (with-keywords-removed (args '(:radius :radius-x :radius-y
+                                  :radius-left :radius-right
+                                  :radius-top  :radius-bottom))
+     args)))
