@@ -307,6 +307,12 @@ spatially organized data structure.
       (loop for (r) in move-overlapping do (setf res (region-union res r)))
       (replay history stream res))))
 
+;;; FIXME: although this inherits from COMPLETE-MEDIUM-STATE, in fact
+;;; it needn't, as we only ever call SET-MEDIUM-CURSOR-POSITION on it.
+;;; Until 2006-05-28, we did also use the various medium attributes,
+;;; but with the reworking of REPLAY-OUTPUT-RECORD
+;;; (STANDARD-DISPLAYED-OUTPUT-RECORD) to use around methods and
+;;; WITH-DRAWING-OPTIONS, they are no longer necessary.
 (defclass updating-stream-state (complete-medium-state)
   ((cursor-x :accessor cursor-x :initarg :cursor-x :initform 0)
    (cursor-y :accessor cursor-y :initarg :cursor-y :initform 0)))
@@ -325,7 +331,7 @@ spatially organized data structure.
        (or (not y-supplied-p)
 	   (coordinate= (slot-value state 'cursor-y) cursor-y))))
 
-(defmethod set-medium-graphics-state :after
+(defmethod set-medium-cursor-position
     ((state updating-stream-state) (stream updating-output-stream-mixin))
   (setf (stream-cursor-position stream)
 	(values (cursor-x state) (cursor-y state))))
@@ -351,6 +357,7 @@ spatially organized data structure.
   (multiple-value-bind (cx cy) (stream-cursor-position stream)
     (with-sheet-medium (medium stream)
       ;; Note: We don't match the y coordinate.
+      (declare (ignore cy))
       (match-output-records state :cursor-x cx))))
 
 (defclass updating-output-record-mixin (updating-output-map-mixin
@@ -387,9 +394,6 @@ record is stored.")
    (stream :accessor updating-output-stream :initarg :stream :initform nil
 	   :documentation "Capture the screen in order to restrict update to
 					visible records")
-   (children-updating-output :accessor children-updating-output :initform nil
-			     :documentation "A list of updating-output records
-that are children (but not necessarily direct) of this record.")
    (parent-updating-output :accessor parent-updating-output
 			   :initarg :parent-updating-output :initform nil
 			   :documentation "A backlink to the
@@ -437,29 +441,15 @@ updating-output-parent above this one in the tree.")
   (let ((state (end-graphics-state record)))
     (setf (values (cursor-x state) (cursor-y state)) (values x y))))
 
-;;; XXX still needed?
-(defmethod add-output-record :after
-    ((child updating-output-record-mixin) record)
-  (declare (ignore record))
-  (if (parent-updating-output child)
-      (push child (children-updating-output (parent-updating-output child)))
-      nil))
-
-
 ;;; Prevent deleted output records from coming back from the dead.
 (defmethod delete-output-record :after ((child updating-output-record-mixin)
 					record
 					&optional errorp)
   (declare (ignore record errorp))
-  (let ((pcache (parent-cache child))
-	(parent-updating (parent-updating-output child)))
+  (let ((pcache (parent-cache child)))
     (delete-from-map pcache
 		     (output-record-unique-id child)
-		     (output-record-id-test child))
-    (when parent-updating
-      (setf (children-updating-output parent-updating)
-	    (delete child (children-updating-output parent-updating)
-		    :test #'eq)))))
+		     (output-record-id-test child))))
 
 
 (defclass standard-updating-output-record (updating-output-record-mixin
@@ -484,15 +474,12 @@ updating-output-parent above this one in the tree.")
 				     (record standard-updating-output-record)
 				     use-old-records)
   (funcall function record)
-  (cond (use-old-records
-	 (if (slot-boundp record 'old-children)
-	     (map-over-updating-output function
-				       (old-children record)
-				       use-old-records)
-	     nil))
-	(t (map-over-updating-output function
-				     (sub-record record)
-				     use-old-records))))
+  (let ((children (cond (use-old-records
+                         (when (slot-boundp record 'old-children)
+                           (old-children record)))
+                        (t (sub-record record)))))
+    (when children
+      (map-over-updating-output function children use-old-records))))
 
 
 (defmethod map-over-updating-output
@@ -748,75 +735,79 @@ in an equalp hash table"))
 
 (defmethod compute-difference-set ((record standard-updating-output-record)
 				   &optional (check-overlapping t)
-                                             offset-x offset-y
-                                             old-offset-x old-offset-y)
+                                   offset-x offset-y
+                                   old-offset-x old-offset-y)
   (declare (ignore offset-x offset-y old-offset-x old-offset-y))
   ;; (declare (values erases moves draws erase-overlapping move-overlapping))
   (let (was
         is
+        stay
+        come
         (everywhere (or +everywhere+
-                        (pane-viewport-region (updating-output-stream record)))))
-    ;; Collect what was there
-    (labels ((gather-was (record)
-               (cond ((displayed-output-record-p record)
-                      (push record was))
-                     ((updating-output-record-p record)
-                      (cond ((eq :clean (output-record-dirty record))
-                             (push record was))
-                            ((eq :moved (output-record-dirty record))
-                             (push (slot-value record 'old-bounds) was))
-                            (t
-                             (map-over-output-records-overlapping-region #'gather-was
-                                                                         (old-children record)
-                                                                         everywhere))))
+                        (pane-viewport-region (updating-output-stream record))))
+        (was-table (make-hash-table :test #'equalp))
+        (is-table (make-hash-table :test #'equalp)))
+    
+    (labels ((collect-1-was (record)
+               (push record was)
+               (push record (gethash (output-record-hash record) was-table)))
+             (collect-1-is (record)
+               (push record is)
+               (push record (gethash (output-record-hash record) is-table))
+               ;; come = is \ was
+               ;; stay = is ^ was
+               (cond ((updating-output-record-p record)
+                      (if (eq :clean (output-record-dirty record))
+                          (push record stay)
+                          (push record come)))
                      (t
-                      (map-over-output-records-overlapping-region #'gather-was record everywhere)) )))
-      (gather-was record))
-    ;; Collect what still is there
-    (labels ((gather-is (record)
-               (cond ((displayed-output-record-p record)
-                      (push record is))
-                     ((updating-output-record-p record)
-                      (cond ((eq :clean (output-record-dirty record))
-                             (push record is))
-                            ((eq :moved (output-record-dirty record))
-                             (push record is))
-                            (t
-                             (map-over-output-records-overlapping-region #'gather-is
-                                                                         (sub-record record)
-                                                                         everywhere))))
-                     (t
-                      (map-over-output-records-overlapping-region #'gather-is record everywhere) ))))
-      (gather-is record))
+                      (let ((q (gethash (output-record-hash record) was-table)))
+                        (if (some #'(lambda (x) (output-record-equal record x)) q)
+                            (push record stay)
+                            (push record come)))))))
+      ;; Collect what was there
+      (labels ((gather-was (record)
+                 (cond ((displayed-output-record-p record)
+                        (collect-1-was record))
+                       ((updating-output-record-p record)
+                        (cond ((eq :clean (output-record-dirty record))
+                               (collect-1-was record))
+                              ((eq :moved (output-record-dirty record))
+                               (collect-1-was (slot-value record 'old-bounds)))
+                              (t
+                               (map-over-output-records-overlapping-region #'gather-was
+                                                                           (old-children record)
+                                                                           everywhere))))
+                       (t
+                        (map-over-output-records-overlapping-region #'gather-was record everywhere)))))
+        (gather-was record))
+      ;; Collect what still is there
+      (labels ((gather-is (record)
+                 (cond ((displayed-output-record-p record)
+                        (collect-1-is record))
+                       ((updating-output-record-p record)
+                        (cond ((eq :clean (output-record-dirty record))
+                               (collect-1-is record))
+                              ((eq :moved (output-record-dirty record))
+                               (collect-1-is record))
+                              (t
+                               (map-over-output-records-overlapping-region #'gather-is
+                                                                           (sub-record record)
+                                                                           everywhere))))
+                       (t
+                        (map-over-output-records-overlapping-region #'gather-is record everywhere) ))))
+        (gather-is record)))
     ;;
-    (let ((was-table (make-hash-table :test #'equalp))
-          (is-table (make-hash-table :test #'equalp))
-          gone
-          stay
-          come)
-      (loop for w in was do (push w (gethash (output-record-hash w) was-table)))
-      (loop for i in is do (push i (gethash (output-record-hash i) is-table)))
+    (let (gone)
       ;; gone = was \ is
       (loop for w in was do
-            (cond ((updating-output-record-p w)
-                   (unless (eq :clean (output-record-dirty w))
-                     (push (old-children w) gone)))
-                  (t
-                   (let ((q (gethash (output-record-hash w) is-table)))
-                     (unless (some #'(lambda (x) (output-record-equal w x)) q)
-                       (push w gone))))))
-      ;; come = is \ was
-      ;; stay = is ^ was
-      (loop for i in is do
-            (cond ((updating-output-record-p i)
-                   (if (eq :clean (output-record-dirty i))
-                       (push i stay)
-                       (push i come)))
-                  (t
-                   (let ((q (gethash (output-record-hash i) was-table)))
-                     (if (some #'(lambda (x) (output-record-equal i x)) q)
-                         (push i stay)
-                         (push i come))))))
+        (cond ((updating-output-record-p w)
+               (unless (eq :clean (output-record-dirty w))
+                 (push (old-children w) gone)))
+              (t
+               (let ((q (gethash (output-record-hash w) is-table)))
+                 (unless (some #'(lambda (x) (output-record-equal w x)) q)
+                   (push w gone))))))
       ;; Now we essentially want 'gone', 'stay', 'come'
       (let ((gone-overlap nil)
             (come-overlap nil))
@@ -825,27 +816,26 @@ in an equalp hash table"))
                 (loop for k in gone
                       if (some (lambda (x) (region-intersects-region-p k x))
                                stay)
-                      collect k into gone-overlap*
-                      else collect k into gone*
+                        collect (list k k) into gone-overlap*
+                      else collect (list k k) into gone*
                       finally (return (values gone* gone-overlap*))))
           (setf (values come come-overlap)
                 (loop for k in come
                       if (some (lambda (x) (region-intersects-region-p k x))
                                stay)
-                      collect k into come-overlap*
-                      else collect k into come*
+                        collect (list k k) into come-overlap*
+                      else collect (list k k) into come*
                       finally (return (values come* come-overlap*)))))
         ;; Hmm, we somehow miss come-overlap ...
         (values
          ;; erases
-         (loop for k in gone collect (list k k))
+         gone
          ;; moves
          nil
          ;; draws
-         (loop for k in come collect (list k k))
+         come
          ;; erase overlapping
-         (append (loop for k in gone-overlap collect (list k k))
-                 (loop for k in come-overlap collect (list k k)))
+         (append gone-overlap come-overlap)
          ;; move overlapping
          nil)))))
 
@@ -944,7 +934,7 @@ in an equalp hash table"))
                                                record
                                                nil)
                      (add-output-record record (stream-current-output-record stream))
-                     (set-medium-graphics-state (end-graphics-state record) stream)
+                     (set-medium-cursor-position (end-graphics-state record) stream)
                      (setf (parent-cache record) parent-cache) )) ))))
       record)))
 
@@ -1002,7 +992,7 @@ in an equalp hash table"))
       (unwind-protect
 	   (progn
 	     (letf (((do-note-output-record stream) nil))
-	       (set-medium-graphics-state (start-graphics-state record) stream)
+	       (set-medium-cursor-position (start-graphics-state record) stream)
 	       (compute-new-output-records record stream)
 	       (when *dump-updating-output*
 		 (dump-updating record :both *trace-output*)))
@@ -1019,7 +1009,7 @@ in an equalp hash table"))
 	       (incremental-redisplay stream nil erases moves draws
 				      erase-overlapping move-overlapping))
 	     (delete-stale-updating-output record))
-	(set-medium-graphics-state current-graphics-state stream)))))
+	(set-medium-cursor-position current-graphics-state stream)))))
 
 (defun erase-rectangle (stream bounding)
   (with-bounding-rectangle* (x1 y1 x2 y2)
