@@ -2,6 +2,8 @@
 
 ;;;  (c) copyright 2001 by 
 ;;;           Tim Moore (moore@bricoworks.com)
+;;;  (c) copyright 2006 by
+;;;           Troels Henriksen (athas@sigkill.dk)
 
 ;;; This library is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU Library General Public
@@ -18,195 +20,22 @@
 ;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 ;;; Boston, MA  02111-1307  USA.
 
+;;; This file provides definitions of every part of input-editing that
+;;; can be defined without actually having loaded the input
+;;; editor. This is so more input-editor using code can be loaded
+;;; before loading Drei.
+
 (in-package :clim-internals)
 
+(defvar *use-goatee* nil
+  "I true, use the Goatee editing component instead of Drei. The
+Goatee component is faster and more mature than Drei.")
+
 (defvar *activation-gestures* nil)
 (defvar *standard-activation-gestures* '(:newline :return))
 
 (defvar *delimiter-gestures* nil)
 
-(defgeneric interactive-stream-p (stream)
-  (:method (stream)
-    (cl:interactive-stream-p stream)))
-
-(defclass empty-input-mixin ()
-  ()
-  (:documentation "A mixin class used for detecting empty input"))
-
-(defclass standard-input-editing-stream (empty-input-mixin
-					 goatee:goatee-input-editing-mixin
-					 input-editing-stream
-					 standard-encapsulating-stream)
-  ((buffer :reader stream-input-buffer
-	   :initform (make-array 16 :adjustable t :fill-pointer 0))
-   (insertion-pointer :accessor stream-insertion-pointer :initform 0)
-   (scan-pointer :accessor stream-scan-pointer :initform 0)
-   (rescan-queued :accessor rescan-queued :initform nil)
-   (rescanning-p :reader stream-rescanning-p :initform nil)
-   (activation-gesture :accessor activation-gesture :initform nil)))
-
-;;; Markers for noise strings in the input buffer.
-
-(defclass noise-string-property ()
-  ())
-
-(defclass noise-string-start-property (noise-string-property)
-  ())
-
-(defparameter *noise-string* (make-instance 'noise-string-property))
-
-(defparameter *noise-string-start*
-  (make-instance 'noise-string-start-property))
-
-;;; Have to reexamine how many of the keyword arguments to stream-read-gesture
-;;; should really be passed to the encapsulated stream.
-;;;
-;;; OK, now I know :)  They should all be passed, except for peek-p.
-;;; However, the loop that calls stream-read-gesture on the
-;;; encapsulated stream needs to return null if we see a :timeout or :eof.
-;;;
-;;; Activation gesture handling has been moved out of stream-process-gesture to
-;;; stream-read-gesture and stream-unread-gesture. This allows a gesture to be
-;;; read in while it is not an activation gesture, unread, and then read again
-;;; as an activation gesture. This kind of game seems to be needed for reading
-;;; forms properly. -- moore
-#-(and)
-(defmethod stream-read-gesture ((stream standard-input-editing-stream)
-				&rest rest-args &key peek-p
-				&allow-other-keys)
-  (with-keywords-removed (rest-args (:peek-p))
-    (rescan-if-necessary stream)
-    (with-slots (buffer insertion-pointer scan-pointer)
-	stream
-      (loop
-       (cond ((< scan-pointer insertion-pointer)
-	      (let ((gesture (aref buffer scan-pointer)))
-		(cond ((typep gesture 'noise-string-property)
-		       (incf scan-pointer))
-		      ;; XXX What about if peek-p is true?
-		      ;; I'm thinking that accept should look for accept
-		      ;; results explicitly. -- moore
-		      ((and (not peek-p)
-			    (typep gesture 'goatee::accept-result-extent))
-		       (incf scan-pointer)
-		       (throw-object-ptype (goatee::object gesture)
-					   (goatee::result-type gesture)))
-		      (t (unless peek-p
-			   (when (activation-gesture-p gesture)
-			     (setf (stream-activated stream) t))
-			   (incf scan-pointer))
-			 (return-from stream-read-gesture gesture)))))
-	     ;; If activated, insertion pointer is at fill pointer XXX get rid
-	     ;; of this
-	     #+(or)
-	     ((stream-activated stream)
-	      (return-from stream-read-gesture (values nil :eof)))
-	     (t (when (eql scan-pointer (fill-pointer buffer))
-		  (setf (slot-value stream 'rescanning-p) nil))
-		(loop for result = (multiple-value-bind (gesture type)
-				       (apply #'stream-read-gesture
-					      (encapsulating-stream-stream
-					       stream)
-					      rest-args)
-				     (when (null gesture)
-				       (return-from stream-read-gesture
-					 (values gesture type)))
-				     (stream-process-gesture stream
-							     gesture
-							     type))
-		      until result)))))))
-
-(defmethod stream-read-gesture ((stream standard-input-editing-stream)
-				&rest rest-args &key peek-p
-				&allow-other-keys)
-  (with-keywords-removed (rest-args (:peek-p))
-    (rescan-if-necessary stream)
-    (with-slots (buffer insertion-pointer scan-pointer activation-gesture)
-	stream
-      (loop
-	 (loop
-	    while (< scan-pointer insertion-pointer)
-	    do (let ((gesture (aref buffer scan-pointer)))
-		 ;; Skip noise strings.
-		 ;; XXX We should skip accept results too; I think that they
-		 ;; should be consumed by ACCEPT-1. That's not happening yet.
-		 (cond ((characterp gesture)
-			(unless peek-p
-			 (incf scan-pointer))
-			(return-from stream-read-gesture gesture))
-		       ((and (not peek-p)
-			     (typep gesture 'goatee::accept-result-extent))
-			(incf scan-pointer)
-			(throw-object-ptype (goatee::object gesture)
-					    (goatee::result-type gesture)))
-		       (t (incf scan-pointer)))))
-	 ;; The scan pointer should not be greater than the insertion pointer
-	 ;; because the code that set the insertion pointer should have queued
-	 ;; a rescan.
-	 (when (> scan-pointer insertion-pointer)
-	   (warn "scan-pointer ~S > insertion-pointer ~S; shouldn't happen"
-		 scan-pointer insertion-pointer)
-	   (immediate-rescan stream))
-	 (when activation-gesture
-	   (return-from stream-read-gesture
-	     (prog1
-		 activation-gesture
-	       (unless peek-p
-		 (setf activation-gesture nil)))))
-	 (setf (slot-value stream 'rescanning-p) nil)
-	 ;; In McCLIM stream-process-gesture is responsible for inserting
-	 ;; characters into the buffer, changing the insertion pointer and
-	 ;; possibly setting up the activation-gesture slot.
-	 (loop
-	    with gesture and type 
-	    do (setf (values gesture type)
-		     (apply #'stream-read-gesture
-			    (encapsulating-stream-stream stream) rest-args))
-	    when (null gesture)
-	      do (return-from stream-read-gesture (values gesture type))
-	    when (stream-process-gesture stream gesture type)
-	      do (loop-finish))))))
-
-#-(and)
-(defmethod stream-unread-gesture ((stream standard-input-editing-stream)
-				  gesture)
-  (declare (ignore gesture))
-  (when (> (stream-scan-pointer stream) 0)
-    (setf (stream-activated stream) nil)
-    (decf (stream-scan-pointer stream))))
-
-(defmethod stream-unread-gesture ((stream standard-input-editing-stream)
-				  gesture)
-  (with-slots (buffer scan-pointer activation-gesture)
-      stream
-    (when (> scan-pointer 0)
-      (if (and (eql scan-pointer (fill-pointer buffer))
-	       (activation-gesture-p gesture))
-	  (setf activation-gesture gesture)
-	  (decf scan-pointer)))))
-
-(defgeneric activate-stream (stream gesture)
-  (:documentation "Cause the input editing stream STREAM to be activated with
-  GESTURE"))
-
-(defmethod activate-stream ((stream standard-input-editing-stream) gesture)
-  (setf (activation-gesture stream) gesture)
-  (setf (stream-insertion-pointer stream)
-	(fill-pointer (stream-input-buffer stream)))
-  (goatee::set-editing-stream-insertion-pointer
-   stream
-   (stream-insertion-pointer stream)))
-
-(defgeneric stream-process-gesture (stream gesture type)
-  (:documentation "McCLIM relys on a text editor class (by default
-  GOATEE-INPUT-EDITING-MIXIN) to perform the user interaction and display for
-  input editing. Also, that class must update the stream buffer and the
-  insertion pointer, cause rescans to happen, and handle activation gestures."))
-
-;;; The editing functions of stream-process-gesture are performed by the
-;;; primary method on goatee-input-editing-mixin
-;;;
-
 ;;; These helper functions take the arguments of ACCEPT so that they
 ;;; can be used directly by ACCEPT.
 
@@ -234,33 +63,37 @@
 	(t existing-delimiter-gestures)))
 
 (defmacro with-activation-gestures ((gestures &key override) &body body)
-  ;; XXX Guess this implies that gestures need to be defined at compile time.
-  ;; Sigh.
-  (let ((gesture-form (if (and (symbolp gestures)
-			       (gethash gestures *gesture-names*))
-			  `(list ',gestures)
-			  gestures))
+  ;; XXX Guess this implies that gestures need to be defined at
+  ;; compile time.  Sigh.  We permit both CLIM 2.0-style gesture names
+  ;; and CLIM 2.2 style characters.
+  (let ((gesture-form (cond ((or (and (symbolp gestures)
+                                      (gethash gestures *gesture-names*))
+                                 (characterp gestures))
+                             `(list ',gestures))
+                            (t gestures)))
 	(gestures (gensym))
 	(override-var (gensym)))
     `(let* ((,gestures ,gesture-form)	;Preserve evaluation order of arguments
 	    (,override-var ,override)
-	    (*activation-gestures* (make-activation-gestures
-				    (if ,override-var
-					:activation-gestures
-					:additional-activation-gestures)
-				    ,gestures)))
+	    (*activation-gestures* (apply #'make-activation-gestures
+                                          (if ,override-var
+                                              :activation-gestures
+                                              :additional-activation-gestures)
+                                          (list ,gestures))))
        ,@body)))
 
 (defmacro with-delimiter-gestures ((gestures &key override) &body body)
-  ;; XXX Guess this implies that gestures need to be defined at compile time.
-  ;; Sigh.
-  (let ((gesture-form (if (and (symbolp gestures)
-			       (gethash gestures *gesture-names*))
-			  `(list ',gestures)
-			  gestures))
+  ;; XXX Guess this implies that gestures need to be defined at
+  ;; compile time.  Sigh.  We permit both CLIM 2.0-style gesture names
+  ;; and CLIM 2.2 style characters.
+  (let ((gesture-form (cond ((or (and (symbolp gestures)
+                                      (gethash gestures *gesture-names*))
+                                 (characterp gestures))
+                             `(list ',gestures))
+                            (t gestures)))
 	(gestures (gensym))
 	(override-var (gensym)))
-    `(let* ((,gestures ,gesture-form)	;Preserve evaluation order of arguments
+    `(let* ((,gestures ,gesture-form) ;Preserve evaluation order of arguments
 	    (,override-var ,override)
 	    (*delimiter-gestures* (make-delimiter-gestures
 				   (if ,override-var
@@ -284,62 +117,37 @@
 (defmacro with-input-editing ((&optional (stream t)
 			       &rest args
 			       &key input-sensitizer (initial-contents "")
-			       (class ''standard-input-editing-stream))
+			       (class ''standard-input-editing-stream class-provided-p))
 			      &body body)
   (setq stream (stream-designator-symbol stream '*standard-input*))
   (with-keywords-removed (args (:input-sensitizer :initial-contents :class))
     `(invoke-with-input-editing ,stream
-				#'(lambda (,stream) ,@body)
-				,input-sensitizer ,initial-contents ,class
-				,@args)))
+                                #'(lambda (,stream) ,@body)
+                                ,input-sensitizer ,initial-contents
+                                ,(if class-provided-p
+                                     class
+                                     `(if *use-goatee* 'goatee-input-editing-stream
+                                          ,class))
+                                ,@args)))
 
-(define-condition rescan-condition (condition)
-  ())
-
-(defgeneric finalize (editing-stream input-sensitizer)
-  (:documentation "Do any cleanup on an editing stream, like turning off the
-  cursor, etc."))
-
-(defgeneric invoke-with-input-editing
-    (stream continuation input-sensitizer initial-contents class))
-
-(defmethod invoke-with-input-editing :around ((stream extended-output-stream)
-					      continuation
-					      input-sensitizer
-					      initial-contents
-					      class)
-  (declare (ignore continuation input-sensitizer initial-contents class))
-  (letf (((cursor-visibility (stream-text-cursor stream)) nil))
-    (call-next-method)))
-
-(defmethod invoke-with-input-editing ((stream extended-input-stream)
-				      continuation
-				      input-sensitizer
-				      initial-contents
-				      class)
-  (let ((editing-stream (make-instance class
-				       :stream stream
-				       :initial-contents initial-contents)))
-    (goatee::redisplay-area (goatee::area editing-stream))
-    (unwind-protect
-	 (loop
-	  (block rescan
-	    (handler-bind ((rescan-condition #'(lambda (c)
-						 (declare (ignore c))
-						 (reset-scan-pointer
-						  editing-stream)
-						 (return-from rescan nil))))
-	      (return-from invoke-with-input-editing
-		(funcall continuation editing-stream)))))
-      (finalize editing-stream input-sensitizer))))
+(defmacro with-input-position ((stream) &body body)
+  (let ((stream-var (gensym "STREAM")))
+    `(let* ((,stream-var ,stream)
+	    (*current-input-stream* (and (typep ,stream-var
+						'input-editing-stream)
+					 ,stream-var))
+	    (*current-input-position* (and *current-input-stream*
+					   (stream-scan-pointer ,stream-var))))
+       ,@body)))
 
 (defun input-editing-rescan-loop (editing-stream continuation)
   (loop
    (block rescan
-     (handler-bind ((rescan-condition #'(lambda (c)
-					  (declare (ignore c))
-					  (reset-scan-pointer editing-stream)
-					  (return-from rescan nil))))
+     (handler-bind ((rescan-condition
+                     #'(lambda (c)
+                         (declare (ignore c))
+                         (reset-scan-pointer editing-stream)
+                         (return-from rescan nil))))
        (return-from input-editing-rescan-loop
 	 (funcall continuation editing-stream))))))
 
@@ -348,37 +156,14 @@
   (declare (ignore input-sensitizer initial-contents class))
   (funcall continuation stream))
 
-(defgeneric reset-scan-pointer (stream &optional scan-pointer))
-
-(defmethod reset-scan-pointer ((stream standard-input-editing-stream)
-			       &optional (scan-pointer 0))
-  (setf (stream-scan-pointer stream) scan-pointer)
-  (setf (slot-value stream 'rescanning-p) t))
-
-(defgeneric immediate-rescan (stream))
-
-(defmethod immediate-rescan ((stream standard-input-editing-stream))
-  (signal 'rescan-condition))
-
-(defgeneric queue-rescan (stream))
-
-(defmethod queue-rescan ((stream standard-input-editing-stream))
-  (setf (rescan-queued stream) t))
-
-(defgeneric rescan-if-necessary (stream &optional inhibit-activation))
-
-(defmethod rescan-if-necessary ((stream standard-input-editing-stream)
-				&optional inhibit-activation)
-  (declare (ignore inhibit-activation))
-  (when (rescan-queued stream)
-    (setf (rescan-queued stream) nil)
-    (immediate-rescan stream)))
-
-(defgeneric input-editor-format (stream format-string &rest format-args))
+(defgeneric input-editing-stream-bounding-rectangle (stream)
+  (:documentation "Return the bounding rectangle of `stream' as
+four values. This function does not appear in the spec but is
+used by the command processing code for layout."))
 
 (defmethod input-editor-format ((stream t) format-string &rest format-args)
   (unless (and (typep stream '#.*string-input-stream-class*)
-	       (input-stream-p stream))
+               (input-stream-p stream))
     (apply #'format stream format-string format-args)))
 
 (defun make-room (buffer pos n)
@@ -394,19 +179,6 @@
 (defvar *current-input-stream* nil)
 (defvar *current-input-position* 0)
 
-(defmacro with-input-position ((stream) &body body)
-  (let ((stream-var (gensym "STREAM")))
-    `(let* ((,stream-var ,stream)
-	    (*current-input-stream* (and (typep ,stream-var
-						'input-editing-stream)
-					 ,stream-var))
-	    (*current-input-position* (and *current-input-stream*
-					   (stream-scan-pointer ,stream-var))))
-       ,@body)))
-
-(defgeneric replace-input (stream new-input
-			   &key start end buffer-start rescan))
-
 (defun read-token (stream &key
 		   (input-wait-handler *input-wait-handler*)
 		   (pointer-button-press-handler
@@ -536,12 +308,16 @@
                          (unread-gesture gesture :stream stream)
                          (return-from complete-input-rescan
                            (values object t input)))
+                       ;; Do we actually want to signal this here? I
+                       ;; don't think we should terminate the
+                       ;; input-editing-session just because the user
+                       ;; tries to complete invalid input.
 		       (error 'simple-completion-error
-                            :format-control "complete-input: While rescanning,~
+                              :format-control "complete-input: While rescanning,~
                                              can't match ~A~A"
-                            :format-arguments (list so-far  gesture)
+                              :format-arguments (list so-far  gesture)
 			    
-                            :input-so-far so-far))))
+                              :input-so-far so-far))))
 	  end
 	  do (vector-push-extend gesture so-far)
 	  finally (when gesture
@@ -603,67 +379,68 @@
 					*accelerator-gestures*)))
     (with-input-position (stream)
       (flet ((insert-input (input)
-	       (adjust-array so-far (length input)
-			     :fill-pointer (length input))
-	       (replace so-far input)
-	       (replace-input stream input :rescan nil)))
- 	(multiple-value-bind (object success input)
-	    (complete-input-rescan stream func partial-completers
-				   so-far allow-any-input)
-	  (when success
-	    (return-from complete-input (values object success input))))
-	(loop
-	 (multiple-value-bind (gesture mode)
-	     (read-completion-gesture stream
-				      partial-completers
-				      help-displays-possibilities)
-	   (if mode
-	       (multiple-value-bind
-		     (input success object nmatches possibilities)
-		   (funcall func (subseq so-far 0) mode)
-		 (when (and (zerop nmatches)
-			    (eq mode :complete-limited)
-			    (complete-gesture-p gesture))
-		   ;; Gesture is both a partial completer and a
-		   ;; delimiter e.g., #\space.  If no partial match,
-		   ;; try again with a total match.
-		   (setf (values input success object nmatches possibilities)
-			 (funcall func (subseq so-far 0) :complete))
-		   (setf mode :complete))
-		 ;; Preserve the delimiter
-		 (when (and success (eq mode :complete))
-		   (unread-gesture gesture :stream stream))
-		 ;; Get completion from menu
-		 (when *trace-complete-input*
-		   (format *trace-output* "nmatches = ~A, mode = ~A~%"
-			   nmatches mode))
-		 (when (and (> nmatches 0) (eq mode :possibilities))
-		   (multiple-value-bind (menu-object item event)
-		       (menu-choose (possibilities-for-menu possibilities))
-		     (declare (ignore event))
-		     (if item
-			 (progn
-			   (setf (values input success object nmatches)
-				 (values (car item) t menu-object 1)))
-			 (setf success nil
-			       nmatches 0))))
-		 (unless (and (eq mode :complete) (not success))
-		   (if (> nmatches 0)
-		       (insert-input input)
-		       (beep)))
-                 (cond ((and success (eq mode :complete))
-			(return-from complete-input
-			  (values object success input)))
-		       ((activation-gesture-p gesture)
-			(if allow-any-input
-			    (return-from complete-input
-			      (values nil t (subseq so-far 0)))
-			    (error 'simple-completion-error
-				   :format-control "Input ~S does not match"
-				   :format-arguments (list so-far)
-				   :input-so-far so-far)))))
-	       (vector-push-extend gesture so-far))))))))
-
+               (adjust-array so-far (length input)
+                             :fill-pointer (length input))
+               (replace so-far input)
+               ;; XXX: Relies on non-specified behavior of :rescan.
+               (replace-input stream input :rescan nil)))
+        (multiple-value-bind (object success input)
+            (complete-input-rescan stream func partial-completers
+                                   so-far allow-any-input)
+          (when success
+            (return-from complete-input (values object success input))))
+        (loop
+           (multiple-value-bind (gesture mode)
+               (read-completion-gesture stream
+                                        partial-completers
+                                        help-displays-possibilities)
+             (if mode
+                 (multiple-value-bind
+                       (input success object nmatches possibilities)
+                     (funcall func (subseq so-far 0) mode)
+                   (when (and (zerop nmatches)
+                              (eq mode :complete-limited)
+                              (complete-gesture-p gesture))
+                     ;; Gesture is both a partial completer and a
+                     ;; delimiter e.g., #\space.  If no partial match,
+                     ;; try again with a total match.
+                     (setf (values input success object nmatches possibilities)
+                           (funcall func (subseq so-far 0) :complete))
+                     (setf mode :complete))
+                   ;; Preserve the delimiter
+                   (when (and success (eq mode :complete))
+                     (unread-gesture gesture :stream stream))
+                   ;; Get completion from menu
+                   (when *trace-complete-input*
+                     (format *trace-output* "nmatches = ~A, mode = ~A~%"
+                             nmatches mode))
+                   (when (and (> nmatches 0) (eq mode :possibilities))
+                     (multiple-value-bind (menu-object item event)
+                         (menu-choose (possibilities-for-menu possibilities)
+                                      :label "Possibilities"
+                                      :n-columns 1)
+                       (declare (ignore event))
+                       (if item
+                           (setf (values input success object nmatches)
+                                 (values (car item) t menu-object 1))
+                           (setf success nil
+                                 nmatches 0))))
+                   (unless (and (eq mode :complete) (not success))
+                     (if (> nmatches 0)
+                         (insert-input input)
+                         (beep)))
+                   (cond ((and success (eq mode :complete))
+                          (return-from complete-input
+                            (values object success input)))
+                         ((activation-gesture-p gesture)
+                          (if allow-any-input
+                              (return-from complete-input
+                                (values nil t (subseq so-far 0)))
+                              (error 'simple-completion-error
+                                     :format-control "Input ~S does not match"
+                                     :format-arguments (list so-far)
+                                     :input-so-far so-far)))))
+                 (vector-push-extend gesture so-far))))))))
 
 ;;; helper function
 
@@ -752,12 +529,12 @@
 		    (and (eql matches 1)
 			 (not partial-match-before-end)))
 		(if (eq action :complete-maximal)
-		    (cond ((and (eql matches 2) identical-match)
-			   object)
-			  ((and identical-match (eql matches 1))
-			   identical-object)
-			  ((eql matches 1)
-			   object))
+                    (cond ((and (eql matches 2) identical-match)
+                           object)
+                          ((and identical-match (eql matches 1))
+                           identical-object)
+                          ((eql matches 1)
+                           object))
 		    (and (or identical (and (eql matches 1)
 					    (not partial-match-before-end)))
 			 object))
@@ -767,7 +544,7 @@
 ;;; The possibilities action is different enough that I don't want to add to
 ;;; the spaghetti above...
 
-(defun complete-from-generator-possibilities  
+(defun complete-from-generator-possibilities
     (initial-string generator predicate)
   (let ((possibilities nil)
 	(nmatches 0)
@@ -783,12 +560,13 @@
       (funcall generator initial-string #'suggester)
       (if (and (eql nmatches 1)
 	       (string-equal initial-string (caar possibilities)))
+	  ;; return values are as from complete-from-generator, qv.
 	  (values (caar possibilities)
 		  t
 		  (cdar possibilities)
 		  nmatches
 		  possibilities)
-	  (values initial-string nil nil nmatches possibilities)))))
+	  (values initial-string nil nil nmatches (sort possibilities #'string-lessp :key #'car))))))
 
 (defun complete-from-possibilities (initial-string completions delimiters &key
 				    (action :complete)
@@ -916,5 +694,3 @@
 			    (return-from empty-input nil)))))
 	(return-from invoke-handle-empty-input (funcall input-continuation))))
     (funcall handler-continuation)))
-
-
