@@ -67,17 +67,6 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
 
-(defgeneric text-style-components (text-style))
-(defgeneric text-style-family (text-style))
-(defgeneric text-style-face (text-style))
-(defgeneric text-style-size (text-style))
-(defgeneric merge-text-styles (text-style-1 text-style-2))
-(defgeneric text-style-ascent (text-style medium))
-(defgeneric text-style-descent (text-style medium))
-(defgeneric text-style-height (text-style medium))
-(defgeneric text-style-width (text-style medium))
-(defgeneric text-style-fixed-width-p (text-style medium))
-
 (defgeneric text-style-equalp (style1 style2))
 (defmethod text-style-equalp ((style1 text-style) (style2 text-style)) nil)
 
@@ -137,14 +126,25 @@
   (defvar *text-style-hash-table* (make-hash-table :test #'eql)))
 
 (defun make-text-style (family face size)
-  (let ((key (text-style-key family face size)))
-    (declare (type fixnum key))
-    (or (gethash key *text-style-hash-table*)
-	(setf (gethash key *text-style-hash-table*)
-	      (make-instance 'standard-text-style
-			     :text-family family
-			     :text-face face
-			     :text-size size)))))
+  (if (and (symbolp family)
+	   (or (symbolp face)
+	       (and (listp face) (every #'symbolp face))))
+      ;; Portable text styles have always been cached in McCLIM like this:
+      ;; (as permitted by the CLIM spec for immutable objects, section 2.4)
+      (let ((key (text-style-key family face size)))
+	(declare (type fixnum key))
+	(or (gethash key *text-style-hash-table*)
+	    (setf (gethash key *text-style-hash-table*)
+		  (make-text-style-1 family face size))))
+      ;; Extended text styles using string components could be cached using
+      ;; an appropriate hash table, but for now we just re-create them:
+      (make-text-style-1 family face size)))
+
+(defun make-text-style-1 (family face size)
+  (make-instance 'standard-text-style
+    :text-family family
+    :text-face face
+    :text-size size))
 
 ) ; end eval-when
 
@@ -154,8 +154,8 @@
 
 (defmethod text-style-equalp ((style1 standard-text-style)
 			      (style2 standard-text-style))
-  (and (eql (text-style-family style1) (text-style-family style2))
-       (eql (text-style-face style1) (text-style-face style2))
+  (and (equal (text-style-family style1) (text-style-family style2))
+       (equal (text-style-face style1) (text-style-face style2))
        (eql (text-style-size style1) (text-style-size style2))))
 
 (defconstant *default-text-style* (make-text-style :fix :roman :normal))
@@ -464,14 +464,6 @@
                 :type (or (member t nil)
                           sequence)) ))
 
-(defmethod line-style-p ((x line-style))
-  (declare (ignorable x))
-  t)
-
-(defmethod line-style-p ((x t))
-  (declare (ignorable x))
-  nil)
-
 (defun make-line-style (&key (unit :normal) (thickness 1)
 			     (joint-shape :miter) (cap-shape :butt)
 			     (dashes nil))
@@ -507,6 +499,7 @@
        (eql (line-style-joint-shape style1) (line-style-joint-shape style2))
        (eql (line-style-cap-shape style1) (line-style-cap-shape style2))
        (eql (line-style-dashes style1) (line-style-dashes style2))))
+
 
 ;;; Misc ops
 
