@@ -119,6 +119,12 @@
 	(sheet-children (first (sheet-children (frame-panes submenu-frame))))
 	'())))
 
+(defclass submenu-border (border-pane) ())
+
+(defclass submenu-border-pane (raised-pane)
+  ()
+  (:default-initargs :border-width 2 :background *3d-normal-color*))
+
 (defun create-substructure (sub-menu client)
   (let* ((frame *application-frame*)
 	 (manager (frame-manager frame))
@@ -130,7 +136,7 @@
 				    'menu)))
 	 (rack (make-pane-1 manager frame 'vrack-pane
 			    :background *3d-normal-color* :contents items))
-	 (raised (make-pane-1 manager frame 'raised-pane :border-width 2 :background *3d-normal-color* :contents (list rack))))
+	 (raised (make-pane-1 manager frame 'submenu-border :contents (list rack))))
     (with-slots (bottomp) sub-menu
       (multiple-value-bind (xmin ymin xmax ymax)
 	  (bounding-rectangle* (sheet-region sub-menu))
@@ -277,6 +283,7 @@
                           :label name
                           :text-style *enabled-text-style*
                           :client client
+                          :vertical vertical
                           :value-changed-callback
                           #'(lambda (gadget val)
                               (declare (ignore gadget val))
@@ -285,6 +292,7 @@
                             :label name
                             :text-style *disabled-text-style*
                             :client client
+                            :vertical vertical
                             :value-changed-callback
                             #'(lambda (gadget val)
                                 (declare (ignore gadget val))
@@ -296,6 +304,7 @@
                      :label name
                      :text-style *enabled-text-style*
                      :client client
+                     :vertical vertical
                      :value-changed-callback
                      #'(lambda (gadget val)
                          (declare (ignore gadget val))
@@ -308,6 +317,7 @@
       (:divider
        (make-pane-1 manager frame 'menu-divider-leaf-pane
                     :label name
+                    :vertical vertical
                     :client client))
       (:menu
         (make-pane-1 manager frame (if vertical
@@ -315,6 +325,7 @@
                                        'menu-button-submenu-pane)
 		     :label name
 		     :client client
+                     :vertical vertical
 		     :frame-manager manager
 		     :command-table value
 		     :bottomp bottomp))
@@ -362,20 +373,96 @@
 		           (max-width +fill+) max-height
 			   min-width min-height)
   (with-slots (menu) (find-command-table command-table)
-    (raising ()
-      (make-pane-1 *pane-realizer* *application-frame*
-                   'menu-bar
-                   :background *3d-normal-color*
-                   :width width :height height
-                   :max-width max-width :max-height max-height
-                   :min-width min-width :min-height min-height
-                   :contents
-                   (append
-                    (loop for item in menu
-                          collect 
-                          (make-menu-button-from-menu-item
-			   item nil
-			   :bottomp t
-                           :vertical nil
-			   :command-table command-table))
-                    (list +fill+))))))
+    (make-pane-1 *pane-realizer* *application-frame*
+		 'menu-bar
+		 :background *3d-normal-color*
+		 :width width :height height
+		 :max-width max-width :max-height max-height
+		 :min-width min-width :min-height min-height
+		 :contents
+		 (append
+		  (loop for item in menu
+		      collect 
+                        (make-menu-button-from-menu-item
+			 item nil
+			 :bottomp t
+			 :vertical nil
+			 :command-table command-table))
+		  (list +fill+)))))
+
+(defmethod handle-repaint ((pane menu-bar) region)
+  (declare (ignore region))
+  (with-slots (border-width) pane
+    (multiple-value-call #'draw-bordered-rectangle*
+      pane
+      (bounding-rectangle* (sheet-region pane))
+      :style :outset
+      :border-width 2)))
+
+(defmethod compose-space ((pane menu-bar) &key width height)
+  (declare (ignore width height))
+  (space-requirement+
+   (call-next-method)
+   (make-space-requirement :height 4 :max-height 4 :min-height 4)))
+
+(defmethod box-layout-mixin/horizontally-allocate-space
+    ((pane menu-bar) real-width real-height)
+  (with-slots (x-spacing) pane
+    (let ((widths
+	   (box-layout-mixin/horizontally-allocate-space-aux*
+	    pane real-width real-height))
+	  (x 2))
+      (loop
+	  for child in (box-layout-mixin-clients pane)
+	  for width in widths
+	  do
+	    (when (box-client-pane child)
+	      (layout-child (box-client-pane child)
+			    :expand
+			    :expand
+			    x
+			    2
+			    width
+			    (- real-height 4)))
+	    (incf x width)
+	    (incf x x-spacing)))))
+
+(defmethod display-command-table-menu ((command-table standard-command-table)
+                                       (stream fundamental-output-stream)
+                                       &rest args
+                                       &key max-width max-height n-rows n-columns
+                                       x-spacing y-spacing initial-spacing
+                                       row-wise (cell-align-x :left)
+                                       (cell-align-y :top) (move-cursor t))
+  (formatting-item-list (stream :max-width max-width :max-height max-height :n-rows n-rows
+                                :n-columns n-columns :x-spacing x-spacing :y-spacing y-spacing
+                                :initial-spacing initial-spacing :row-wise row-wise
+                                :move-cursor move-cursor)
+    (map-over-command-table-menu-items
+     #'(lambda (item-name accelerator item)
+         (declare (ignore accelerator))
+         (formatting-cell (stream :align-x cell-align-x :align-y cell-align-y)
+           (cond ((eq (command-menu-item-type item) :menu)
+                  (with-text-style (stream (make-text-style :serif '(:bold :italic) nil))
+                    (write-string item-name stream)
+                    (terpri stream))
+                  (surrounding-output-with-border (stream)
+                    (apply #'display-command-table-menu
+                           (find-command-table (command-menu-item-value item))
+                           stream args)))
+                 ((eq (command-menu-item-type item) :command)
+                  (let ((name (command-name (command-menu-item-value item))))
+                    (when (command-line-name-for-command name command-table :errorp nil)
+                      (present name 'command-name :stream stream)))))))
+     command-table)))
+
+(defmethod display-command-menu (frame (stream fundamental-output-stream)
+                                 &rest args &key
+                                 (command-table (frame-command-table frame))
+                                 initial-spacing row-wise max-width
+                                 max-height n-rows n-columns
+                                 (cell-align-x :left) (cell-align-y :top))
+  (declare (ignore initial-spacing row-wise max-width max-height
+                   n-rows n-columns cell-align-x cell-align-y))
+  (with-keywords-removed (args (:command-table))
+    (apply #'display-command-table-menu command-table stream args)))
