@@ -100,8 +100,9 @@
 (defmethod presentation-type-of ((object structure-object))
   (multiple-value-bind (name lambda-list)
       (get-ptype-from-class-of object)
-    (if (or (null lambda-list)
-            (member lambda-list lambda-list-keywords))
+    (if (and name
+	     (or (null lambda-list)
+		 (member lambda-list lambda-list-keywords)))
         name
         (call-next-method))))
 
@@ -161,16 +162,126 @@
 	       (block presentation-subtypep
 		 ,@body))))))))
 
+;;; PRESENTATION-SUBTYPEP suffers from some of the same problems as
+;;; CL:SUBTYPEP, most (but sadly not all) of which were solved in
+;;; H. Baker "A Decision Procedure for SUBTYPEP"; additionally, it
+;;; suffers from the behaviour being underspecified, as CLIM
+;;; documentation did not have the years of polish that CLtS did.
+;;;
+;;; So you might wonder why, instead of copying or using directly some
+;;; decent Public Domain subtype code (such as that found in SBCL,
+;;; implementing CL:SUBTYPEP), there's this slightly wonky
+;;; implementation here.  Well, some of the answer lies in the fact
+;;; that the subtype relationships answered by this predicate are not
+;;; in fact analogous to CL's type system.  The major use of
+;;; PRESENTATION-SUBTYPEP seems to be for determining whether a
+;;; presentation is applicable as input to a translator (including the
+;;; default translator, transforming an object to itself); actually,
+;;; the first step is taken by STUPID-SUBTYPEP, but that I believe is
+;;; simply intended to be a short-circuiting conservative version of
+;;; PRESENTATION-SUBTYPEP.
+;;;
+;;; Most presentation types in CLIM are hierarchically arranged by
+;;; single-inheritance, and SUBTYPEP relations on the hierarchy are
+;;; easy to determine: simply walk up the hierarchy until you find the
+;;; putative supertype (in which case the answer is T, T unless the
+;;; type's parameters are wrong) or you find the universal supertype
+;;; (in which case the answer is NIL, T.  There are numerous wrinkles,
+;;; however...
+;;;
+;;; (1) the NIL presentation type is the universal subtype, breaking
+;;;     the single-inheritance of the hierarchy.  This isn't too bad,
+;;;     because it can be special-cased.
+;;;
+;;; (2) union types can be constructed, destroying the
+;;;     single-inheritance hierarchy (when used as a subtype).
+;;;
+;;; (3) union types can give rise to ambiguity.  For example, is the
+;;;     NUMBER presentation type subtypep (OR REAL COMPLEX)?  What
+;;;     about (INTEGER 3 6) subtypep (OR (INTEGER 3 4) (INTEGER 5 6))?
+;;;     Is (OR A B) subtypep (OR B A)?  The answer to this last
+;;;     question is not obvious, as the two types have different
+;;;     ACCEPT behaviour if A and B have any Lisp objects in common,
+;;;     even if the presentation types are hierarchically unrelated...
+;;;
+;;; (4) intersection types can be constructed, destroying the
+;;;     single-inheritance hierarchy (when used as a supertype).  This
+;;;     is partially mitigated by the explicit documentation that the
+;;;     first type in the AND type's parameters is privileged and
+;;;     treated specially by ACCEPT.
+;;;
+;;; Given these difficulties, I'm aiming for roughly expected
+;;; behaviour from STUPID- and PRESENTATION-SUBTYPEP, rather than
+;;; something which has a comprehensive understanding of presentation
+;;; types and the Lisp object universe (as this would be unachievable
+;;; anyway: the user can write arbitrary PRESENTATION-TYPEP
+;;; functions); PRESENTATION-SUBTYPEP should not be thought of as a
+;;; predicate over sets of Lisp objects, but simply a formal predicate
+;;; over a graph of names.  This gives rise to the implementation
+;;; below for OR and AND types, and the hierarchical walk for all
+;;; other types.  CSR, 2007-01-10
 (defun presentation-subtypep (type maybe-supertype)
-  (when (equal type maybe-supertype)
+  ;; special shortcuts: the universal subtype is privileged (and
+  ;; doesn't in fact fit into a hierarchical lattice); the universal
+  ;; supertype is easy to identify.
+  (when (or (eql type nil) (eql maybe-supertype t))
+    (return-from presentation-subtypep (values t t)))
+  (when (eql type maybe-supertype)
     (return-from presentation-subtypep (values t t)))
   (with-presentation-type-decoded (super-name super-parameters)
-    maybe-supertype
-    (when (eq super-name 'or)
-      (loop for or-type in super-parameters
-	    when (presentation-subtypep type or-type)
-	    do (return-from presentation-subtypep (values t t))
-	    finally (return-from presentation-subtypep (values nil t))))
+      maybe-supertype
+    (with-presentation-type-decoded (type-name type-parameters)
+        type
+      (cond
+        ;; DO NOT BE TEMPTED TO REARRANGE THESE CLAUSES
+        ((eq type-name 'or)
+         (dolist (or-type type-parameters 
+                  (return-from presentation-subtypep (values t t)))
+           (multiple-value-bind (yesp surep)
+               (presentation-subtypep or-type maybe-supertype)
+             (unless yesp
+               (return-from presentation-subtypep (values yesp surep))))))
+        ((eq super-name 'and)
+         (let ((result t))
+           (dolist (and-type super-parameters 
+                    (return-from presentation-subtypep (values result result)))
+             (cond
+               ((and (consp and-type) (eq (car and-type) 'satisfies))
+                (setq result nil))
+               ((and (consp and-type) (eq (car and-type) 'not))
+                (multiple-value-bind (yp sp)
+                    (presentation-subtypep type (cadr and-type))
+                  (if yp
+                      (return-from presentation-subtypep (values nil t))
+                      (setq result nil))))
+               (t (multiple-value-bind (yp sp)
+                      (presentation-subtypep type and-type)
+                    (unless yp
+                      (if sp
+                          (return-from presentation-subtypep (values nil t))
+                          (setq result nil)))))))))
+        ((eq super-name 'or)
+         (assert (not (eq type-name 'or)))
+         ;; FIXME: this would be the right method were it not for the
+         ;; fact that there can be unions 'in disguise' in the
+         ;; subtype; examples: 
+         ;;   (PRESENTATION-SUBTYPEP 'NUMBER '(OR REAL COMPLEX))
+         ;;   (PRESENTATION-SUBTYPEP '(INTEGER 3 6)
+         ;;                          '(OR (INTEGER 2 5) (INTEGER 4 7)))
+         ;; Sorry about that.
+         (let ((surep t))
+           (dolist (or-type super-parameters 
+                    (return-from presentation-subtypep (values nil surep)))
+             (multiple-value-bind (yp sp)
+                 (presentation-subtypep type or-type)
+               (cond
+                 (yp (return-from presentation-subtypep (values t t)))
+                 ((not sp) (setq surep nil)))))))
+        ((eq type-name 'and)
+         (assert (not (eq super-name 'and)))
+         (multiple-value-bind (yp sp)
+             (presentation-subtypep (car type-parameters) maybe-supertype)
+           (return-from presentation-subtypep (values yp yp))))))
     (map-over-presentation-type-supertypes
      #'(lambda (name massaged)
 	 (when (eq name super-name)
@@ -193,6 +304,23 @@
 	  (values t t)
 	  (values nil nil)))))
 
+(define-presentation-generic-function
+    %presentation-type-specifier-p
+    presentation-type-specifier-p
+  (type-class type))
+
+(define-default-presentation-method presentation-type-specifier-p (type)
+  t)
+
+(defun presentation-type-specifier-p (object)
+  "Return true if `object' is a valid presentation type specifier,
+otherwise return false."
+  ;; Apparently, this funtion has to handle arbitrary objects.
+  (let ((name (presentation-type-name object)))
+    (when (and (typep name '(or symbol class))
+               (get-ptype-metaclass name))
+      (funcall-presentation-generic-function presentation-type-specifier-p object))))
+
 (defun default-describe-presentation-type (description stream plural-count)
   (if (symbolp description)
       (setq description (make-default-description (symbol-name description))))
@@ -433,8 +561,20 @@ type histories"))
 (defun presentation-type-history (type)
   (funcall-presentation-generic-function presentation-type-history type))
 
-(defclass presentation-history-ring (goatee::ring)
-  ())
+(defclass presentation-history ()
+  ((stack :accessor presentation-history-array
+          :initform (make-array 1 :fill-pointer 0
+                                  :adjustable t)
+          :documentation "The history, with the newest objects at
+the end of the array. Should contain conses with the car being
+the object and the cdr being the type.")
+   (pointer :accessor presentation-history-pointer
+            :initform nil
+            :documentation "The index of the \"current\" object,
+used when navigating the history. If NIL, means that no
+navigation has yet been performed."))
+  (:documentation "Class for objects that contain the history for
+a specific type."))
 
 (define-default-presentation-method presentation-type-history (type)
   (if (and *application-frame*
@@ -450,7 +590,7 @@ type histories"))
 		    (history-object (gethash name history-table)))
 	       (unless history-object
 		 (setf history-object
-		       (make-instance 'presentation-history-ring)
+		       (make-instance 'presentation-history)
 		       (gethash name history-table)
 		       history-object))
 	       history-object))
@@ -479,46 +619,104 @@ call-next-method to get the \"real\" answer based on the stream type."))
 
 (define-presentation-method presentation-type-history-for-stream
     ((type t) (stream input-editing-stream))
-  (if (not (stream-rescanning-p stream))
-      (funcall-presentation-generic-function presentation-type-history type)
-      nil))
+  ;; What is the purpose of this? Makes stuff harder to do, so
+  ;; commented out...
+  ;;(if (not (stream-rescanning-p stream))
+  ;;       (funcall-presentation-generic-function presentation-type-history type)
+  ;;       nil)
+  (funcall-presentation-generic-function presentation-type-history type))
 
 (defun presentation-history-insert (history object ptype)
-  (goatee::ring-obj-insert (cons object ptype) history))
+  "Unconditionally insert `object' as an input of presentation
+type `type' at the top of the presentation history `history', as
+the most recently added object."
+  (vector-push-extend (cons object ptype)
+                      (presentation-history-array history)))
 
-(defun presentation-history-head (history ptype)
+(defun presentation-history-top (history ptype)
+  "Find the topmost (most recently added object) of `history'
+that is of the presentation type `ptype' or a subtype. Two values
+will be returned, the object and the presentation type of the
+object. If no applicable object can be found, these values will
+both be NIL."
   (loop
-   for cell = (goatee::dbl-head history) then (goatee::next cell)
-   for (object . object-ptype) = (and cell (goatee::contents cell))
-   while cell
-   if (presentation-subtypep object-ptype ptype)
-   return (values object object-ptype)
-   finally (return (values nil nil))))
+     with array = (presentation-history-array history)
+     for index from (1- (fill-pointer array)) downto 0
+     for (object . object-ptype) = (aref array index)
+     do
+     (when (presentation-subtypep object-ptype ptype)
+       (return (aref array index)))
+     finally (return (values nil nil))))
+
+(defun presentation-history-reset-pointer (history)
+  "Set the pointer to point at the object most recently added
+object."
+  (setf (presentation-history-pointer history) nil))
 
 (defun presentation-history-next (history ptype)
-  (let ((first-object (goatee::forward history)))
-    (loop
-     for first-time = t then nil
-     for cell = first-object then (goatee::forward history)
-     for (object . object-ptype) = (goatee::contents cell)
-     while (or first-time (not (eq first-object cell)))
-     if (presentation-subtypep object-ptype ptype)
-       return (values object object-ptype)
-     end
-     finally (return (values nil nil)))))
+  "Go to the next input (forward in time) in `history' that is a
+presentation-subtype of `ptype', respective to the pointer in
+`history'. Returns two values: the found object and its
+presentation type, both of which will be NIL if no applicable
+object can be found."
+  (with-accessors ((pointer presentation-history-pointer)
+                   (array presentation-history-array)) history
+    ;; If no navigation has been performed, we have no object to go
+    ;; forwards to.
+    (if (or (null pointer) (>= (1+ pointer) (length array)))
+        (values nil nil)
+        (progn
+          (incf pointer)
+          (destructuring-bind (object . object-ptype)
+              (aref array pointer)
+            (if object-ptype
+                (if (presentation-subtypep object-ptype ptype)
+                    (values object object-ptype)
+                    (presentation-history-next history ptype))
+                (values nil nil)))))))
+
+(defun presentation-history-previous (history ptype)
+  "Go to the previous input (backward in time) in `history' that
+is a presentation-subtype of `ptype', respective to the pointer
+in `history'. Returns two values: the found object and its
+presentation type, both of which will be NIL if no applicable
+object can be found."
+  (with-accessors ((pointer presentation-history-pointer)
+                   (array presentation-history-array)) history
+    (if (and (numberp pointer) (zerop pointer))
+        (values nil nil)
+        (progn
+          (cond ((and (numberp pointer) (plusp pointer))
+                 (decf pointer))
+                ((plusp (length array))
+                 (setf pointer (1- (fill-pointer array)))))
+          (if (and (numberp pointer) (array-in-bounds-p array pointer))
+              (destructuring-bind (object . object-ptype)
+                  (aref array pointer)
+                (if object-ptype
+                    (if (presentation-subtypep object-ptype ptype)
+                        (values object object-ptype)
+                        (progn (presentation-history-previous history ptype)))
+                    (values nil nil)))
+              (values nil nil))))))
 
 (defmacro with-object-on-history ((history object ptype) &body body)
-  `(goatee::with-object-on-ring ((cons ,object ,ptype) ,history)
-     ,@body))
+  "Evaluate `body' with `object' as `ptype' as the head (most
+recently added object) on `history', and remove it again after
+`body' has run. If `body' as `ptype' is already the head, the
+history will be unchanged."
+  (with-gensyms (added)
+    `(let ((,added (presentation-history-add ,history ,object ,ptype)))
+       (unwind-protect (progn ,@body)
+         (when ,added
+           (decf (fill-pointer (presentation-history-array ,history))))))))
 
 (defun presentation-history-add (history object ptype)
   "Add OBJECT and PTYPE to the HISTORY unless they are already at the head of
  HISTORY"
-  (let* ((cell (goatee::dbl-head history))
-	 (contents (and cell (goatee::contents cell))))
-    (unless (and cell
-		 (eql object (car contents))
-		 (equal ptype (cdr contents)))
+  (multiple-value-bind (top-object top-ptype)
+      (presentation-history-top history ptype)
+    (unless (and top-ptype (eql object top-object) (equal ptype top-ptype))
       (presentation-history-insert history object ptype))))
 
 ;;; Context-dependent input
@@ -697,61 +895,45 @@ call-next-method to get the \"real\" answer based on the stream type."))
         ;; presentation history. In addition, we'll implement the Genera
         ;; behavior of temporarily putting the default on the history
         ;; stack so the user can conveniently suck it in.
-        (flet ((do-accept (args)
-                 (apply #'stream-accept stream real-type args))
-               (get-history ()
-                 (when real-history-type
-                   (funcall-presentation-generic-function
-                    presentation-type-history-for-stream
-                    real-history-type stream))))
+        (labels ((get-history ()
+                   (when real-history-type
+                     (funcall-presentation-generic-function
+                      presentation-type-history-for-stream
+                      real-history-type stream)))
+                 (do-accept (args)
+                   (apply #'stream-accept stream real-type args)))
           (let* ((default-from-history (and (not defaultp) provide-default))
                  (history (get-history))
                  (results
-                  (multiple-value-list 
+                  (multiple-value-list
                    (if history
-                       (let ((*active-history-type* real-history-type))
-                         (cond (defaultp
-                                (with-object-on-history
-                                    (history default real-default-type)
-                                  (do-accept rest-args)))
-                               (default-from-history
-                                (multiple-value-bind
-                                      (history-default history-type)
-                                    (presentation-history-head history
-                                                               real-default-type)
-                                  (do-accept (if history-type
-                                                 (list* :default history-default
-                                                        :default-type history-type
-                                                        rest-args)
-                                                 rest-args))))
-                               (t (do-accept rest-args))))
+                       (unwind-protect
+                            (let ((*active-history-type* real-history-type))
+                              (cond (defaultp
+                                     (with-object-on-history
+                                         (history default real-default-type)
+                                       (do-accept rest-args)))
+                                    (default-from-history
+                                     (multiple-value-bind
+                                           (history-default history-type)
+                                         (presentation-history-top history
+                                                                   real-default-type)
+                                       (do-accept (if history-type
+                                                      (list* :default history-default
+                                                             :default-type history-type
+                                                             rest-args)
+                                                      rest-args))))
+                                    (t (do-accept rest-args))))
+                         (unless *recursive-accept-p*
+                           (presentation-history-reset-pointer (get-history))))
                        (do-accept rest-args))))
                  (results-history (get-history)))
             (when results-history
               (presentation-history-add results-history
                                         (car results)
-                                        (cadr results)))
+                                        real-type))
             (values-list results)))))))
 
-(defgeneric stream-accept (stream type
-			   &key
-			   view
-			   default
-			   default-type
-			   provide-default
-			   insert-default
-			   replace-input
-			   history
-			   active-p
-			   prompt
-			   prompt-mode
-			   display-default
-			   query-identifier
-			   activation-gestures
-			   additional-activation-gestures
-			   delimiter-gestures
-			   additional-delimiter-gestures))
-
 (defmethod stream-accept ((stream standard-extended-input-stream) type
 			  &rest args
 			  &key (view (stream-default-view stream))
@@ -759,13 +941,6 @@ call-next-method to get the \"real\" answer based on the stream type."))
   (apply #'prompt-for-accept stream type view args)
   (apply #'accept-1 stream type args))
 
-(defmethod stream-accept ((stream standard-input-editing-stream) type
-			  &rest args
-			  &key (view (stream-default-view stream))
-			  &allow-other-keys)
-  (apply #'prompt-for-accept stream type view args)
-  (apply #'accept-1 stream type args))
-
 (defmethod stream-accept ((stream #.*string-input-stream-class*) type
 			  &key (view (stream-default-view stream))
 			  (default nil defaultp)
@@ -830,13 +1005,13 @@ call-next-method to get the \"real\" answer based on the stream type."))
 				 (declare (ignore stream))
 				 (funcall cont))))
       (with-input-position (stream)	; support for calls to replace-input
-        (when insert-default
+        (when (and insert-default
+                   (not (stream-rescanning-p stream)))
           ;; Insert the default value to the input stream. It should
-          ;; become fully keyboard-editable.
-          (presentation-replace-input stream
-                                       default
-                                       default-type
-                                       view))
+          ;; become fully keyboard-editable. We do not want to insert
+          ;; the default if we're rescanning, only during initial
+          ;; setup.
+          (presentation-replace-input stream default default-type view))
 	(setf (values sensitizer-object sensitizer-type)
 	      (with-input-context (type)
 		  (object object-type event options)
@@ -859,7 +1034,7 @@ call-next-method to get the \"real\" answer based on the stream type."))
                                       :default-type default-type)
 				     (funcall-presentation-generic-function
 				      accept type stream view))))
-			;; User entered activation or delimeter
+			;; User entered activation or delimiter
 			;; gesture without any input.
 			(if defaultp
 			    (progn
@@ -891,8 +1066,6 @@ call-next-method to get the \"real\" answer based on the stream type."))
 	;; Just to make it clear that we're returning values
 	(values sensitizer-object sensitizer-type)))))
 
-(defgeneric prompt-for-accept (stream type view &rest accept-args &key))
-
 (defmethod prompt-for-accept ((stream t)
 			      type view
 			      &rest accept-args
@@ -1033,21 +1206,28 @@ call-next-method to get the \"real\" answer based on the stream type."))
 					   stream
 					   state)))
 
+(defgeneric highlight-output-record-tree (record stream state))
+
+(defmethod highlight-output-record-tree (record stream state)
+  (declare (ignore record stream state))
+  (values))
+
+(defmethod highlight-output-record-tree ((record compound-output-record) stream state)
+  (map-over-output-records
+   (lambda (record)
+     (highlight-output-record-tree record stream state))
+   record))
+
+(defmethod highlight-output-record-tree ((record displayed-output-record) stream state)
+  (highlight-output-record record stream state))
+
 (define-default-presentation-method highlight-presentation
     (type record stream state)
   (declare (ignore type))
   (if (or (eq (presentation-single-box record) t)
 	  (eq (presentation-single-box record) :highlighting))
-      (highlight-output-record-rectangle record stream state)
-      (labels ((highlighter (record)
-		 (typecase record
-		   (displayed-output-record
-		    (highlight-output-record record stream state))
-		   (compound-output-record
-		    (map-over-output-records #'highlighter record))
-		   (t nil))))
-	(highlighter record))))
-
+      (highlight-output-record record stream state)
+      (highlight-output-record-tree record stream state)))
 
 (define-default-presentation-method present
     (object type stream (view textual-view) &key acceptably for-context-type)
@@ -1102,12 +1282,14 @@ call-next-method to get the \"real\" answer based on the stream type."))
   t)
 
 (define-presentation-method present (object (type t) 
-				     stream
-				     (view textual-view)
-				     &key acceptably for-context-type)
+                                            stream
+                                            (view textual-view)
+                                            &key acceptably for-context-type)
   (declare (ignore for-context-type))
   (let ((*print-readably* acceptably))
-    (princ object stream)))
+    (if acceptably
+        (prin1 object stream)
+        (princ object stream))))
 
 (define-presentation-type nil ())
 
@@ -1436,8 +1618,10 @@ call-next-method to get the \"real\" answer based on the stream type."))
 (define-presentation-method present (object (type string) stream
 				     (view textual-view)
 				     &key acceptably for-context-type)
-  (declare (ignore acceptably for-context-type))
-  (princ object stream))
+  (declare (ignore for-context-type))
+  (if acceptably
+      (prin1 object stream)
+      (princ object stream)))
 
 (define-presentation-method accept ((type string) stream (view textual-view)
 				    &key (default nil defaultp)
@@ -1458,27 +1642,151 @@ call-next-method to get the \"real\" answer based on the stream type."))
 (define-presentation-method presentation-typep (object (type pathname))
   (pathnamep object))
 
+(define-presentation-method present ((object pathname) (type pathname)
+                                     stream (view textual-view) &key)
+  ;; XXX: We can only visually represent the pathname if it has a name
+  ;; - making it wild is a compromise. If the pathname is completely
+  ;; blank, we leave it as-is, though.
+
+  ;; The above comment was meant to indicate that if the pathname had
+  ;; neither a name NOR a directory, then it couldn't be visually
+  ;; represented.  Some discussion has ensued on the possbility of
+  ;; emitting something like "A pathname of type <foo>"
+  ;; [2007/01/08:rpg]
+  (let ((pathname (if (equal object #.(make-pathname))
+                      object
+                      (merge-pathnames object (make-pathname :name :wild)))))
+    (princ object stream))
+  )
+
+(define-presentation-method present ((object string) (type pathname)
+                                     stream (view textual-view)
+                                     &rest args &key)
+  (apply-presentation-generic-function
+   present (pathname object) type stream view args))
+
 (defmethod presentation-type-of ((object pathname))
   'pathname)
 
-(define-presentation-method present (object (type pathname) stream
-				     (view textual-view)
-				     &key acceptably for-context-type)
-  (declare (ignore acceptably for-context-type))
-  (princ object stream))
+(defun filename-completer (so-far mode)
+  (flet ((remove-trail (s)
+           (subseq s 0 (let ((pos (position #\/ s :from-end t)))
+                         (if pos (1+ pos) 0)))))
+    (let* ((directory-prefix
+            (if (and (plusp (length so-far)) (eql (aref so-far 0) #\/))
+                ""
+                (namestring #+sbcl *default-pathname-defaults*
+                            #+cmu (ext:default-directory)
+                            #-(or sbcl cmu) *default-pathname-defaults*)))
+           (full-so-far (concatenate 'string directory-prefix so-far))
+           (pathnames
+            (loop with length = (length full-so-far)
+               and wildcard = (concatenate 'string (remove-trail so-far) "*.*")
+               for path in
+               #+(or sbcl cmu lispworks) (directory wildcard)
+               #+openmcl (directory wildcard :directories t)
+               #+allegro (directory wildcard :directories-are-files nil)
+               #+cormanlisp (nconc (directory wildcard)
+                                   (cl::directory-subdirs dirname))
+               #-(or sbcl cmu lispworks openmcl allegro cormanlisp)
+               (directory wildcard)
+               when (let ((mismatch (mismatch (namestring path) full-so-far)))
+                      (or (null mismatch) (= mismatch length)))
+               collect path))
+           (strings (mapcar #'namestring pathnames))
+           (first-string (car strings))
+           (length-common-prefix nil)
+           (completed-string nil)
+           (full-completed-string nil)
+           (input-is-directory-p (when (plusp (length so-far))
+                                   (char= (aref so-far (1- (length so-far))) #\/))))
+      (unless (null pathnames)
+        (setf length-common-prefix
+              (loop with length = (length first-string)
+                 for string in (cdr strings)
+                 do (setf length (min length (or (mismatch string first-string) length)))
+                 finally (return length))))
+      (unless (null pathnames)
+        (setf completed-string
+              (subseq first-string (length directory-prefix)
+                      (if (null (cdr pathnames)) nil length-common-prefix)))
+        (setf full-completed-string
+              (concatenate 'string directory-prefix completed-string)))
+      (case mode
+        ((:complete-limited :complete-maximal)
+         (cond ((null pathnames)
+                (values so-far nil nil 0 nil))
+               ((null (cdr pathnames))
+                (values completed-string (plusp (length so-far)) (car pathnames) 1 nil))
+               (input-is-directory-p
+                (values completed-string t (parse-namestring so-far) (length pathnames) nil))
+               (t
+                (values completed-string nil nil (length pathnames) nil))))
+        (:complete
+         ;; This is reached when input is activated, if we did
+         ;; completion, that would mean that an input of "foo" would
+         ;; be expanded to "foobar" if "foobar" exists, even if the
+         ;; user actually *wants* the "foo" pathname (to create the
+         ;; file, for example).
+         (values so-far t so-far 1 nil))
+        (:possibilities
+         (values nil nil nil (length pathnames)
+                 (loop with length = (length directory-prefix)
+                    for name in pathnames
+                    collect (list (subseq (namestring name) length nil)
+                                  name))))))))
 
-(define-presentation-method accept 
-    ((type pathname) stream (view textual-view)
-     &key (default *default-pathname-defaults*))
-  (let* ((namestring (read-token stream))
-	 (path (parse-namestring namestring)))
-    (if merge-default
-	(merge-pathnames 
-	 path 
-	 (merge-pathnames (make-pathname :type default-type
-					 :version default-version)
-			  default))
-	path)))
+(define-presentation-method accept ((type pathname) stream (view textual-view)
+                                    &key (default *default-pathname-defaults* defaultp)
+                                    ((:default-type accept-default-type) type))
+  (multiple-value-bind (pathname success string)
+      (complete-input stream
+                      #'filename-completer
+                      :allow-any-input t)
+    (cond ((and pathname success)
+           (values (if merge-default
+                       (progn
+                         (unless (or (pathname-type pathname)
+                                     (null default-type))
+                           (setf pathname (make-pathname :defaults pathname
+                                                         :type default-type)))
+                         (merge-pathnames pathname default default-version))
+                       pathname)
+                   type))
+          ((and (zerop (length string))
+                defaultp)
+           (values default accept-default-type))
+          (t (values string 'string)))))
+
+(defmethod presentation-replace-input :around
+    ((stream input-editing-stream)
+     (object pathname) (type (eql 'pathname))
+     view &rest args &key &allow-other-keys)
+  ;; This is fully valid and compliant, but it still smells slightly
+  ;; like a hack.
+  (let ((name (pathname-name object))
+        (directory (when (pathname-directory object)
+                     (directory-namestring object)))
+        (type (pathname-type object))
+        (string "")
+        (old-insp (stream-insertion-pointer stream)))
+    (setf string (or directory string))
+    (setf string (concatenate 'string string
+                              (cond ((and name type)
+                                     (file-namestring object))
+                                    (name name)
+                                    (type (subseq
+                                           (namestring
+                                            (make-pathname
+                                             :name " "
+                                             :type type))
+                                           1)))))
+    (apply #'replace-input stream string args)
+    (when directory
+      (setf (stream-insertion-pointer stream)
+            (+ old-insp (if directory (length directory) 0)))
+      ;; If we moved the insertion pointer, this might be a good idea.
+      (redraw-input-buffer stream old-insp))))
 
 (defgeneric default-completion-name-key (item))
 
@@ -1711,6 +2019,11 @@ call-next-method to get the \"real\" answer based on the stream type."))
   :inherit-from 't
   :parameters-are-types t)
 
+(define-presentation-method presentation-type-specifier-p ((type sequence))
+  (and (listp type)
+       (consp (rest type))
+       (presentation-type-specifier-p (second type))))
+
 (define-presentation-method presentation-typep (object (type sequence))
   ;; XXX TYPE here is the sequence element type, not the whole type specifier
   (unless (or (listp object) (vectorp object))
@@ -1965,6 +2278,15 @@ call-next-method to get the \"real\" answer based on the stream type."))
 	   :acceptably acceptably
 	   :for-context-type for-context-type))
 
+(define-presentation-method accept 
+    ((type and) (stream input-editing-stream) (view textual-view) &rest args &key)
+  (let ((subtype (first types)))
+    (multiple-value-bind (obj ptype)
+        (apply-presentation-generic-function accept subtype stream view args)
+      (unless (presentation-typep obj type)
+        (simple-parse-error "Input object ~S is not of type ~S" obj type))
+      obj)))
+
 (define-presentation-type-abbreviation token-or-type (tokens type)
   `(or (member-alist ,tokens) ,type))
 
