@@ -29,11 +29,9 @@
 ;;; Standard-Output-Stream class
 
 (defclass standard-output-stream (fundamental-character-output-stream)
-  ()
-  )
+  ())
 
 (defmethod stream-recording-p ((stream t)) nil)
-
 (defmethod stream-drawing-p ((stream t)) t)
 
 #+ignore(defmethod stream-write-char ((stream standard-output-stream) char)
@@ -53,23 +51,8 @@
 
 ;;; Cursor class
 
-(defgeneric cursor-sheet (cursor))
-
-(defgeneric cursor-position (cursor))
-
 (defgeneric* (setf cursor-position) (x y cursor))
 
-(defgeneric cursor-active (cursor))
-(defgeneric (setf cursor-active) (value cursor))
-
-(defgeneric cursor-state (cursor))
-(defgeneric (setf cursor-state) (value cursor))
-
-(defgeneric cursor-focus (cursor))
-
-(defgeneric cursor-visibility (cursor))
-(defgeneric (setf cursor-visibility) (visibility cursor))
-
 ;;; Cursor-Mixin class
 (defclass cursor-mixin ()
   ((sheet :initarg :sheet
@@ -107,8 +90,8 @@
 (defun decode-cursor-visibility (visibility)
   "Given :on, :off, or nil, returns the needed active and state attributes for the cursor."
   (ecase visibility
-    ((:on T) (values T T))
-    (:off    (values T nil))
+    ((:on t) (values t t))
+    (:off    (values t nil))
     ((nil)   (values nil nil))))
 
 (defmethod cursor-visibility ((cursor cursor-mixin))
@@ -116,7 +99,7 @@
         (s (cursor-state cursor)))
     (cond ((and a s) :on)
           ((and a (not s)) :off)
-          (T nil))))
+          (t nil))))
 
 (defmethod (setf cursor-visibility) (nv (cursor cursor-mixin))
   (multiple-value-bind (active state)
@@ -187,35 +170,8 @@
 
 ;;; Extended-Output-Stream class
 
-;;; Stream text cursor protocol
-(defgeneric stream-text-cursor (stream))
-(defgeneric (setf stream-text-cursor) (cursor stream))
-
-(defgeneric stream-cursor-position (stream))
 (defgeneric* (setf stream-cursor-position) (x y stream))
 
-(defgeneric stream-increment-cursor-position (stream dx dy))
-
-;;; Text protocol
-(defgeneric stream-character-width (stream character &key text-style))
-
-(defgeneric stream-string-width (stream character &key start end text-style))
-
-(defgeneric stream-text-margin (stream))
-(defgeneric (setf stream-text-margin) (margin stream))
-
-(defgeneric stream-line-height (stream &key text-style))
-
-(defgeneric stream-vertical-spacing (stream))
-
-(defgeneric stream-baseline (stream))
-
-(defgeneric stream-end-of-line-action (stream))
-(defgeneric (setf stream-end-of-line-action) (action stream))
-
-(defgeneric stream-end-of-page-action (stream))
-(defgeneric (setf stream-end-of-page-action) (action stream))
-
 ;;; Standard-Extended-Output-Stream class
 
 (defclass standard-extended-output-stream (extended-output-stream
@@ -267,7 +223,7 @@
 (defmethod* (setf stream-cursor-position) (x y (stream standard-extended-output-stream))
   (setf (cursor-position (stream-text-cursor stream)) (values x y)))
 
-(defun stream-set-cursor-position (stream x y)
+(defmethod stream-set-cursor-position ((stream standard-extended-output-stream) x y)
   (setf (stream-cursor-position stream) (values x y)))
 
 (defmethod stream-increment-cursor-position ((stream standard-extended-output-stream) dx dy)
